//
// C++ Implementation: cavity_search
//
// Description:
//
//
// Author: mirco <Mirco.till@asamnet.de>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "cavity_search.h"

int search_cav(iGrid &solvgrid, iGrid &cavgrid, int ngraph) {
  double_LL<point> li; // Linked list of visited but not evaluated points
  point *cur;
  // For al grid points marked as solvent
  for (int i = solvgrid.imin[1]; i <= solvgrid.imax[1]; i++) {
    for (int j = solvgrid.imin[2]; j <= solvgrid.imax[2]; j++) {
      for (int k = solvgrid.imin[3]; k <= solvgrid.imax[3]; k++) {
        if (cavgrid.grid[i][j][k] == 0 && solvgrid.grid[i][j][k] != 0) {
          // put this point to the list
          li.add();
          li.tail->x = i;
          li.tail->y = j;
          li.tail->z = k;
          ngraph++;
          int nelements = 0;
          //evaluate list
          while (li.head != NULL) {
            // take the last point we have put on the list
            cur = li.get_last();
            // assign graph number
            cavgrid.grid[cur->x][cur->y][cur->z] = ngraph;
            // Check all points around this point, if they are on marked as solvent
            for (int ii = cur->x - 1; ii <= cur->x + 1; ii++) {
              for (int jj = cur->y - 1; jj <= cur->y + 1; jj++) {
                for (int kk = cur->z - 1; kk <= cur->z + 1; kk++) {
                  if (ii >= solvgrid.imin[1] && ii <= solvgrid.imax[1] &&
                      jj >= solvgrid.imin[2] && jj <= solvgrid.imax[2] &&
                      kk >= solvgrid.imin[3] && kk <= solvgrid.imax[3] && solvgrid.grid[ii][jj][kk] > 0 && cavgrid.grid[ii][jj][kk] == 0) {
                    // put all points marked as solvent on the list
                    li.add();
                    li.tail->x = ii;
                    li.tail->y = jj;
                    li.tail->z = kk;
                    // And assign them to the same graph
                    cavgrid.grid[ii][jj][kk] = ngraph;
                  };
                }
              }
            }
            // It we have evaluated this point, increase the number of points in this graph and delete the current point
            nelements++;
            delete cur;
          };
          // If thel list is empty, the current graph is finished. Search the remaining points for points not visited and start a new graph.
        };
      };
    };
  }
  return ngraph;
}


/** Searching the surface for clefts. Method 2

If Cleft search method 2 is called, this function will do the work. The idea is, that for ead point containing to the solvent after all internal cavities are isolated we check how many protein is around a singel point. Therefore we go over all points, check in each of the 6 directions if there is protein (or another point marked as celft) nearby. If we can loate protein in a direction and a certain distance from the evaluated point, we increase a counter. After searching in all 6 directions, we check if this counter is higher than a certain value. If this is the case, meaning if the point is surrounded by protein (or cleft points) by more than x percent, we mark this point as cleft. This algorithm is applied iteratively, as long as there are new points marked as cleft.
\param cavgrid Grid containing the informatino about cavities from the MC run
\param cavdim Distance from the evaluated point each direction s searched for protein
\param nsour Percentage for accepting a point as cleft
\return 1 :-)
*/
int check_for_clefts_first_solv(iGrid &cavgrid, int cavdim, int nsour) {
  ofstream out("test2.pdb");
  int flag1;
  int nprint = 0;
  int startx, starty, startz, stopx, stopy, stopz;
  //cout << "checking clefts " << cavdim << " " << nsour << endl;
  tensor<int> tmp_ten(cavgrid.imin[1], cavgrid.imax[1], cavgrid.imin[2], cavgrid.imax[2], cavgrid.imin[3], cavgrid.imax[3]);
  for (int x = cavgrid.imin[1]; x <= cavgrid.imax[1]; x++) {
    for (int y = cavgrid.imin[2]; y <= cavgrid.imax[2]; y++) {
      for (int z = cavgrid.imin[3]; z <= cavgrid.imax[3]; z++) {
        tmp_ten(x, y, z) = cavgrid.grid[x][y][z];
      };
    };
  };

  int npoints = (cavdim * 2 + 1) * (cavdim * 2 + 1) * (cavdim * 2 + 1); // Number of points in the sourrounding box
  double ratio = ((double)nsour) / 100;
  double nneeded = npoints * ratio;
  // lets try this iterative
  int nchanged = 1;
  int niterations = 0;
  while (nchanged > 0) {
    niterations++;
    nchanged = 0;
    for (int x = cavgrid.imin[1]; x <= cavgrid.imax[1]; x++) {
      for (int y = cavgrid.imin[2]; y <= cavgrid.imax[2]; y++) {
        for (int z = cavgrid.imin[3]; z <= cavgrid.imax[3]; z++) {
          if (cavgrid.grid[x][y][z] == 1) {
            nprint++;
            flag1 = 0;
            // check surrounding points
            if (x - cavdim < cavgrid.imin[1]){startx = cavgrid.imin[1];}
            else{startx = x - cavdim;}
            if (y - cavdim < cavgrid.imin[2]){starty = cavgrid.imin[2];}
            else{starty = y - cavdim;}
            if (z - cavdim < cavgrid.imin[3]){startz = cavgrid.imin[3];}
            else{startz = z - cavdim;}
            if (x + cavdim > cavgrid.imax[1]){stopx = cavgrid.imax[1];}
            else{stopx = x + cavdim;}
            if (y + cavdim > cavgrid.imax[2]){stopy = cavgrid.imax[2];}
            else{stopy = y + cavdim;}
            if (z + cavdim > cavgrid.imax[3]){stopz = cavgrid.imax[3];}
            else{stopz = z + cavdim;}

            npoints = 0;
            for (int x2 = startx; x2 <= stopx; x2++) {
              for (int y2 = starty; y2 <= stopy; y2++) {
                for (int z2 = startz; z2 <= stopz; z2++) {
                  npoints++;
                  if (cavgrid.grid[x2][y2][z2] == 0) {
                    flag1++;
                  };
                };
              };
            };
            // If mor than needed percent of the point is sourrounded by protein, change it to cleft
            if (flag1 >= nneeded) {
              /*
              out << "ATOM" << setw(7) << nprint << setw(10) << "DUM  DUM" << setw(5) << 1;
              out << setw(12) << fixed << setprecision(3) << x*cavgrid.spacing;
              out << setw(8) << fixed << setprecision(3) << y*cavgrid.spacing;
              out << setw(8) << fixed << setprecision(3) << z*cavgrid.spacing;
              out << setw(8) << fixed << setprecision(3) << (double) cavgrid.grid[x][y][z];
              out << setw(10) << fixed << setprecision(3) << (double) cavgrid.grid[x][y][z];
              out << endl;
              */
              //cout << flag1 << " " << cavdim << " " << x << " " << y << " " << z << " " << npoints << endl;
              nchanged++;
              cavgrid.grid[x][y][z] = 0;
              tmp_ten(x, y, z) = 0;
            }
          }
        }
      }
    }
    cout << "Iteration: " << niterations << " atoms changed to cleft " << nchanged << endl;
  }


  for (int x = cavgrid.imin[1]; x <= cavgrid.imax[1]; x++) {
    for (int y = cavgrid.imin[2]; y <= cavgrid.imax[2]; y++) {
      for (int z = cavgrid.imin[3]; z <= cavgrid.imax[3]; z++) {
        cavgrid.grid[x][y][z] = tmp_ten(x, y, z);
      };
    };
  };
  return 1;
}
/** Clear a grid use by MC runs

since Monte Carlo points are only possible from min-maxrad + (max + maxrad)-(min-maxrad) and grids are assigned to integers, there are regions at the outer grids which are assgned as protein since they can never be reached by a monte Carlo point
*/
void smooth_solvgrid(double *min, double *max, double maxrad, iGrid &solvgrid) {
  for (int x = solvgrid.imin[1] ; x <= solvgrid.imax[1]; x++) {
    for (int y = solvgrid.imin[2]; y <= solvgrid.imax[2]; y++) {
      for (int z = solvgrid.imin[3]; z <= solvgrid.imax[3]; z++) {
        if (solvgrid.grid[x][y][z] == 0) {
          // in general this means the point belongs to the protein but...
          if (x <= round(min[1] - maxrad) || x >= round(max[1] + maxrad)) {
            solvgrid.grid[x][y][z] = 1;
          }
          if (y <= round(min[2] - maxrad) || y >= round(max[2] + maxrad)) {
            solvgrid.grid[x][y][z] = 1;
          }
          if (z <= round(min[3] - maxrad) || z >= round(max[3] + maxrad)) {
            solvgrid.grid[x][y][z] = 1;
          }
        }
      }
    }
  }
}


/** Check for clefts Method 1

If cleft searching with method 1 is called, this fucntion does the work. For each solvent point, it tries to put a box as large as possible in aroud the point. This box is limited by the smalles distance to a protein point. Afterwards the number of directions where we hav protein around the evaluated point is called. If there are more than x directions where protein is around this point, the whole box is marked as cleft.
\param min_directions Minimum number of directions which have to be surrounded by protein
\param sphere_max Maximum size of the cube placed around each point.
\param nsour Percentage for accepting a point as cleft
\return number of points changed to cleft :-)
*/
int check_for_clefts_sphere(iGrid &cavgrid, double min_directions) {
  tensor<int> tmp_ten(cavgrid.imin[1], cavgrid.imax[1], cavgrid.imin[2], cavgrid.imax[2], cavgrid.imin[3], cavgrid.imax[3]);
  for (int x = cavgrid.imin[1]; x <= cavgrid.imax[1]; x++) {
    for (int y = cavgrid.imin[2]; y <= cavgrid.imax[2]; y++) {
      for (int z = cavgrid.imin[3]; z <= cavgrid.imax[3]; z++) {
        tmp_ten(x, y, z) = cavgrid.grid[x][y][z];
      };
    };
  };
  int next_x_min, next_x_max, next_y_min, next_y_max, next_z_min, next_z_max;
  int natoms = 1;
  int boxdim = absolute(cavgrid.imax[1]) + absolute(cavgrid.imax[2]) + absolute(cavgrid.imax[3]) + absolute(cavgrid.imin[1]) + absolute(cavgrid.imin[2]) + absolute(cavgrid.imin[3]);
  for (int x = cavgrid.imin[1]; x <= cavgrid.imax[1]; x++) {
    for (int y = cavgrid.imin[2]; y <= cavgrid.imax[2]; y++) {
      for (int z = cavgrid.imin[3]; z <= cavgrid.imax[3]; z++) {
        if (tmp_ten(x, y, z) == 1) {
          // search for the next protein point in each direktion
          boxdim = absolute(cavgrid.imax[1]) + absolute(cavgrid.imax[2]) + absolute(cavgrid.imax[3]) + absolute(cavgrid.imin[1]) + absolute(cavgrid.imin[2]) + absolute(cavgrid.imin[3]);
          next_x_min = cavgrid.imin[1] - 1;
          for (int xx = x; xx >= cavgrid.imin[1]; xx--) {
            if (tmp_ten(xx, y, z) == 0) {
              next_x_min = xx;
              if (absolute(xx - x) < boxdim){boxdim = absolute(xx - x);}
              break;
            }
          };
          next_x_max = cavgrid.imax[1] + 1;
          for (int xx = x; xx <= cavgrid.imax[1]; xx++) {
            if (tmp_ten(xx, y, z) == 0) {
              next_x_max = xx;
              if (absolute(xx - x) < boxdim){boxdim = absolute(xx - x);}
              ;break;
            }
          };
          next_y_min = cavgrid.imin[2] - 1;
          for (int yy = y; yy >= cavgrid.imin[2]; yy--) {
            if (tmp_ten(x, yy, z) == 0) {
              next_y_min = yy;
              if (absolute(yy - y) < boxdim){boxdim = absolute(yy - y);};
              break;
            }
          };
          next_y_max = cavgrid.imax[2] + 1;
          for (int yy = y; yy <= cavgrid.imax[2]; yy++) {
            if (tmp_ten(x, yy, z) == 0) {
              next_y_max = yy;
              if (absolute(yy - y) < boxdim){boxdim = absolute(yy - y);};
              break;
            }
          };
          next_z_min = cavgrid.imin[3] - 1;
          for (int zz = z; zz >= cavgrid.imin[3]; zz--) {
            if (tmp_ten(x, y, zz) == 0) {
              next_z_min = zz;
              if (absolute(zz - z) < boxdim){boxdim = absolute(zz - z);};
              break;
            }
          };
          next_z_max = cavgrid.imax[3] + 1;
          for (int zz = z; zz <= cavgrid.imax[3]; zz++) {
            if (tmp_ten(x, y, zz) == 0) {
              next_z_max = zz;
              if (absolute(zz - z) < boxdim){boxdim = absolute(zz - z);};
              break;
            }
          };

          int nsites = 0;

          if (next_x_min >= cavgrid.imin[1]) {
            //cout << "next_x_min" << endl;
            nsites++;
          }
          if (next_x_max <= cavgrid.imax[1]) {
            //cout << "next_x_max" << endl;
            nsites++;
          }
          if (next_y_min >= cavgrid.imin[2]) {
            //cout << "next_y_min" << endl;
            nsites++;
          }
          if (next_y_max <= cavgrid.imax[2]) {
            //cout << "next_y_max" << endl;
          }
          if (next_z_min >= cavgrid.imin[3]) {
            //cout << "next_z_min" << endl;
            nsites++;
          }
          if (next_z_max <= cavgrid.imax[3]) {
            //cout << "next_z_max" << endl;
            nsites++;
          }
          if (nsites > 0) {
            //cout << nsites << endl;
          };
          //Debug Print


          // Now we set the box with the smallest dimensions to cleft values.
          if (nsites > min_directions) {
            for (int x2 = x - boxdim; x2 <= x + boxdim; x2++) {
              for (int y2 = y - boxdim; y2 <= y + boxdim; y2++) {
                for (int z2 = z - boxdim; z2 <= z + boxdim; z2++) {
                  if (x2 >= cavgrid.imin[1] && x2 <= cavgrid.imax[1] &&
                      y2 >= cavgrid.imin[2] && y2 <= cavgrid.imax[2] &&
                      z2 >= cavgrid.imin[3] && z2 <= cavgrid.imax[3] && tmp_ten(x2, y2, z2) == 1 && get_dist(x, y, z, x2, y2, z2) <= boxdim) {
                    cavgrid.grid[x2][y2][z2] = 0;
                    natoms++;
                  }
                }
              }
            }
          }
        }
      };
    };
  };
  return natoms;
}
