/* Matthias Ullmann, 2.12.1994				*/
/* Routines to read atoms from CDR-files.		*/
/*							*/
/* The energy parameters read from CHARMM-Parameter 	*/
/* files are changed from kcal to kJ by multipling the 	*/
/* value with CAL2J from "constant.h"			*/
/*......................................................*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "mathematik.h"
#include "crd_atom.h"
#include "util_c.h"
#include "nrutil.h"

/*
#define CAL2J 1.0
#define J2CAL 1.0
*/
/*..............................................................*/
/* The procedure reads a PARAM.PRM file				*/
/* Zeilen fortsetzung mit "-" ist nicht erlaubt			*/
void read_PARAM(char *filename, PARAM_record *param)
{
 /* Variables */
 FILE *fpparam;

 fpparam=file_open(filename,"r");
 count_records(fpparam,param);
 rewind(fpparam);
 allocate_PARAM(param);
 read_records(fpparam,param);
}
/*..............................................................*/
/* Counting the number of records 				*/
void count_records(FILE *fpparam, PARAM_record *param)
{
 char buff[LINELENGTH], buff1[LINELENGTH];
 int i, n;

 /* count bond entries */
 i=0;
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if (!strncmp(buff,"BOND",4)) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4))) 
               {goto endbond;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) {i++;}
           }
      }
 endbond: /* label */
 (*param).number_of_bonds=i;

 /* count angle entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4))) 
               {goto endangle;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1))  {i++;}
           }
      }
 endangle: /* label */
 (*param).number_of_angles=i=i;

 /* count dihedral entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4))) 
               {goto enddihe;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) i++;
           }
      }
 enddihe: /* label */
 (*param).number_of_torsions=i;

 /* count improper angles entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"NONBONDED",4))||(!strncmp(buff,"NBONDED",4))) 
               {goto endimprop;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) i++;
           }
      }
 endimprop: /* label */
 (*param).number_of_improps=i;

 /* count nonbonded entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NONBONDED",4))) 
         {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if (buff[strlen(buff)]!='-')
                 break;
                }
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if ((!strncmp(buff,"NBFIX",4))||(!strncmp(buff,"HBOND",4))) 
                    {goto endnonbonded;
                    }
                 n=sscanf(buff,"%s",buff1);
                 if ((buff1[0]!='!')&&(n==1)) {i++;}
                }
         }
     }
 endnonbonded: /* label */
 (*param).number_of_nonbonded=i;

 /* count nbfix entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NBFIX",4))) 
      while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"HBOND",4))) 
               {goto endnbfix;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)){i++;}
           }
     }
 endnbfix: /* label */
 (*param).number_of_nbfix=i;

 /* count hbond entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"HBOND",4))) 
         {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if (buff[strlen(buff)]!='-')
                    break;
                }
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if ((!strncmp(buff,"END",3))) 
                    {goto endhbond;
                    }
                 n=sscanf(buff,"%s",buff1);
                 if ((buff1[0]!='!')&&(n==1)) {i++;}
                }
         }
     }
 endhbond: /* label */
 (*param).number_of_hbond=i;

}

/*..............................................................*/
/* Allocation of the structure elements				*/
void allocate_PARAM(PARAM_record *param)
{
    fprintf(stderr,"number_of_bonds:     %d\n",(*param).number_of_bonds);
    fprintf(stderr,"number_of_angles:    %d\n",(*param).number_of_angles);
    fprintf(stderr,"number_of_torsions:  %d\n",(*param).number_of_torsions);
    fprintf(stderr,"number_of_improps:   %d\n",(*param).number_of_improps);
    fprintf(stderr,"number_of_nonbonded: %d\n",(*param).number_of_nonbonded);
    fprintf(stderr,"number_of_nbfix:     %d\n",(*param).number_of_nbfix);
    fprintf(stderr,"number_of_hbond:     %d\n",(*param).number_of_hbond);


 if ((*param).number_of_bonds>0)
    {(*param).bond1	=charmatrix(1,(*param).number_of_bonds,0,4); 	/*[5][N]*/
     (*param).bond2	=charmatrix(1,(*param).number_of_bonds,0,4); 	/*[5][N]*/
     (*param).K_bond	=dvector(1,(*param).number_of_bonds);	   
     (*param).bond_0	=dvector(1,(*param).number_of_bonds);	   
    }

 if ((*param).number_of_angles>0)
    {(*param).angle1	=charmatrix(1,(*param).number_of_angles,0,4); 	/*[5][N]*/
     (*param).angle2	=charmatrix(1,(*param).number_of_angles,0,4); 	/*[5][N]*/
     (*param).angle3	=charmatrix(1,(*param).number_of_angles,0,4); 	/*[5][N]*/
     (*param).K_angle=dvector(1,(*param).number_of_angles);	   
     (*param).angle_0=dvector(1,(*param).number_of_angles);	   
     (*param).K_ub=dvector(1,(*param).number_of_angles);	   
     (*param).dist_ub_0=dvector(1,(*param).number_of_angles);	   
    }

 if ((*param).number_of_torsions>0)
    {(*param).torsion1 =charmatrix(1,(*param).number_of_torsions,0,4); 	/*[5][N]*/
     (*param).torsion2 =charmatrix(1,(*param).number_of_torsions,0,4); 	/*[5][N]*/
     (*param).torsion3 =charmatrix(1,(*param).number_of_torsions,0,4); 	/*[5][N]*/
     (*param).torsion4 =charmatrix(1,(*param).number_of_torsions,0,4); 	/*[5][N]*/
     (*param).K_torsion=dvector(1,(*param).number_of_torsions);	   
     (*param).delta	  =dvector(1,(*param).number_of_torsions);	   
     (*param).n	  =ivector(1,(*param).number_of_torsions);   
    }

 if ((*param).number_of_improps>0)
    {(*param).improp1 =charmatrix(1,(*param).number_of_improps,0,4); 	/*[5][N]*/
     (*param).improp2 =charmatrix(1,(*param).number_of_improps,0,4); 	/*[5][N]*/
     (*param).improp3 =charmatrix(1,(*param).number_of_improps,0,4); 	/*[5][N]*/
     (*param).improp4 =charmatrix(1,(*param).number_of_improps,0,4); 	/*[5][N]*/
     (*param).K_improp=dvector(1,(*param).number_of_improps);	   
     (*param).improp_0=dvector(1,(*param).number_of_improps);	   
    }

 if ((*param).number_of_nonbonded>0)
    {(*param).atom 		=charmatrix(1,(*param).number_of_nonbonded,0,4); /*[5][N]*/
     (*param).polarizability	=dvector(1,(*param).number_of_nonbonded);	   
     (*param).Emin		=dvector(1,(*param).number_of_nonbonded);	   
     (*param).Rmin		=dvector(1,(*param).number_of_nonbonded);	   
     (*param).polarizability1_4	=dvector(1,(*param).number_of_nonbonded);	   
     (*param).Emin1_4	=dvector(1,(*param).number_of_nonbonded);	   
     (*param).Rmin1_4	=dvector(1,(*param).number_of_nonbonded);	   
    }

 if ((*param).number_of_nbfix>0)
    {(*param).atom1	  =charmatrix(1,(*param).number_of_nbfix,0,4);
     (*param).atom2	  =charmatrix(1,(*param).number_of_nbfix,0,4);
     (*param).EminNBfix=dvector(1,(*param).number_of_nbfix);
     (*param).RminNBfix=dvector(1,(*param).number_of_nbfix);
    }	

 if ((*param).number_of_hbond>0)
    {(*param).hatom1=charmatrix(1,(*param).number_of_hbond,0,4);
     (*param).hatom2=charmatrix(1,(*param).number_of_hbond,0,4);
     (*param).para1=dvector(1,(*param).number_of_hbond);		
     (*param).para2=dvector(1,(*param).number_of_hbond);	
    }	
}
/*------------------------------------------------------*/
/* Now read the parameter file				*/
/* The read energy constants are in kJ/mol              */

void read_records(FILE *fpparam,PARAM_record *param)
{
 char buff[LINELENGTH], buff1[LINELENGTH], help[LINELENGTH], 
      help1[LINELENGTH], help2[LINELENGTH];
 int i, n, m;

 /* Read Bond Records */
 i=0;
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if (!strncmp(buff,"BOND",4))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4)))
               {goto endbond;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
               {i++;
                if (i>(*param).number_of_bonds) muerror("too many bond entries");
	        sscanf(buff,"%s %s %lf %lf",(*param).bond1[i],
				            (*param).bond2[i],
				           &(*param).K_bond[i],
				           &(*param).bond_0[i]);
	        (*param).K_bond[i]=CAL2J*(*param).K_bond[i];
               }
           }
      }
 endbond: /* label */
 if (i!=(*param).number_of_bonds) muerror("too few bond entries");

 /* Read Angle Records */
 /* CHANGE the Urey-Bradley terms is not red */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4)))
               {goto endangle;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
               {i++;
                if (i>(*param).number_of_angles) muerror("too many angles entries");
	        m=sscanf(buff,"%s %s %s %lf %lf %s %s",(*param).angle1[i], 
					       (*param).angle2[i],
					       (*param).angle3[i],
				       	      &(*param).K_angle[i],
					      &(*param).angle_0[i],
                                              help1, help2);
	        if (m>=5)
                   {if (help1[0]!='!')
                       {(*param).K_ub[i]=atof(help1);
                        (*param).dist_ub_0[i]=atof(help2);
                        (*param).K_ub[i]=CAL2J*(*param).K_ub[i];
                       }
                   }
                else {(*param).K_ub[i]=0.0; (*param).dist_ub_0[i]=0.0;} 

	        (*param).K_angle[i]=CAL2J*(*param).K_angle[i];
	        (*param).angle_0[i]=TORAD((*param).angle_0[i]);
               }
           }
      }
 endangle: /* label */
 if (i!=(*param).number_of_angles) muerror("too few angles entries");

 /* Read Torsion Records */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4)))
               {goto enddihe;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
                {i++;
                 if (i>(*param).number_of_torsions) muerror("too many torsion entries");
	         sscanf(buff,"%s %s %s %s %lf %d %lf",(*param).torsion1[i], 
					  	      (*param).torsion2[i],
					  	      (*param).torsion3[i],
					  	      (*param).torsion4[i],
				       	  	     &(*param).K_torsion[i],
						     &(*param).n[i],
					  	     &(*param).delta[i]);
	         (*param).K_torsion[i]=CAL2J*(*param).K_torsion[i];
	         (*param).delta[i]=TORAD((*param).delta[i]);
                }
           }
      }
 enddihe: /* label */
 if (i!=(*param).number_of_torsions) muerror("too few  torsion entries");

 /* Read Improper torsion records */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"NONBONDED",4))||(!strncmp(buff,"NBONDED",4)))
               {goto endimprop;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
                {i++;
                 if (i>(*param).number_of_improps) muerror("too many improper entries");
	         sscanf(buff,"%s %s %s %s %lf %*d %lf",(*param).improp1[i], 
					              (*param).improp2[i],
					              (*param).improp3[i],
					              (*param).improp4[i],
				       	             &(*param).K_improp[i],
					             &(*param).improp_0[i]);
	         (*param).K_improp[i]=CAL2J*(*param).K_improp[i];
	         (*param).improp_0[i]=TORAD((*param).improp_0[i]);
                }
           }
      }
 endimprop: /* label */
 if (i!=(*param).number_of_improps) muerror("too few improper entries");

 /* Read nonbonded interactions */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NONBONDED",4))||(!strncmp(buff,"NBONDED",4)))
         {/* CHANGE: the nonbonded settings need to be read somehow!!! */
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                if (buff[strlen(buff)]!='-') break;
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
               {if ((!strncmp(buff,"NBFIX",4))||(!strncmp(buff,"HBOND",4)))
                   {goto endnonbonded;}
                n=sscanf(buff,"%s",buff1);
                if ((buff1[0]!='!')&&(n==1))
                   {i++;
                    if (i>(*param).number_of_nonbonded) 
                       muerror("too many nonbonded entries");
	            if (StringPerLine(buff)>=7)
	               {sscanf(buff,"%s %*f %lf %lf %s %lf %lf",(*param).atom[i], 
					     	    &(*param).Emin[i],
					     	    &(*param).Rmin[i],
					     	    help,
					     	    &(*param).Emin1_4[i],
					     	    &(*param).Rmin1_4[i]);
                        if (help[0]=='!')
                           {(*param).Emin1_4[i]=(*param).Emin[i];
                            (*param).Rmin1_4[i]=(*param).Rmin[i];
                           }
                       }
	             else {if ((StringPerLine(buff)<7) && (StringPerLine(buff)>=4))
	                      {sscanf(buff,"%s %*f %lf %lf",(*param).atom[i], 
					     	       &(*param).Emin[i],
					     	       &(*param).Rmin[i]);
		               (*param).Emin1_4[i]=(*param).Emin[i];
		               (*param).Rmin1_4[i]=(*param).Rmin[i];
	                      }
		           else {muwarn("Number of elements not correct in Line.");
		                 fprintf(stderr,"Number of words in Line: %d\n", 
                                         StringPerLine(buff));
		                 fprintf(stderr,"%s", buff);
		                }
                          }
                   }
               }
 	    (*param).Emin[i]=CAL2J*(*param).Emin[i];
	    (*param).Emin1_4[i]=CAL2J*(*param).Emin1_4[i];
         }
     }
 endnonbonded: /* label */
 if (i!=(*param).number_of_nonbonded) muerror("too few nonbonded entries");

 /* Read nbfix interactions */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NBFIX",4)))
      while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if ((!strncmp(buff,"HBOND",4)))
               {goto endnbfix;
               }
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
	       {i++;
                if (i>(*param).number_of_nbfix) muerror("too many nbfix entries");
	           {sscanf(buff,"%s %s %lf %lf",(*param).atom1[i], 
				                (*param).atom2[i],
				               &(*param).EminNBfix[i],
				               &(*param).RminNBfix[i]);
	            (*param).EminNBfix[i]=CAL2J*(*param).EminNBfix[i];
                   }
               }
           }
     }
 endnbfix: /* label */

 if (i!=(*param).number_of_nbfix) muerror("too few nbfix entries");

 /* CHANGE -- HBOND not read yet - to be implemented */


}

/*..............................................................*/
/* Print the parameter						*/

void fprint_PARAM(FILE *fpout, PARAM_record *param)
{
int i;
fprintf(fpout,"%s !%d\n",BONDS,(*param).number_of_bonds);
for (i=1; i<=(*param).number_of_bonds; i++)
    fprintf(fpout,"%5s %5s %8.3f %8.3f\n",(*param).bond1[i],
				  (*param).bond2[i],
				  J2CAL*(*param).K_bond[i],
				  (*param).bond_0[i]);
fprintf(fpout,"%s !%d\n",THETA,(*param).number_of_angles);
for (i=1; i<=(*param).number_of_angles; i++)
    fprintf(fpout,"%5s %5s %5s %8.3f %8.3f %8.3f %8.3f\n",(*param).angle1[i], 
				     (*param).angle2[i],
				     (*param).angle3[i],
				     J2CAL*(*param).K_angle[i],
				     TOGRD((*param).angle_0[i]),
                                     J2CAL*(*param).K_ub[i],
                                     (*param).dist_ub_0[i]);

/* Read Torsion Records */
fprintf(fpout,"%s !%d\n",PHI,(*param).number_of_torsions);
for (i=1; i<=(*param).number_of_torsions; i++)
    fprintf(fpout,"%5s %5s %5s %5s %8.3f %5d %8.3f\n",(*param).torsion1[i], 
					    (*param).torsion2[i],
					    (*param).torsion3[i],
					    (*param).torsion4[i],
				       	    J2CAL*(*param).K_torsion[i],
					    (*param).n[i],
					    TOGRD((*param).delta[i]));
/* Read Improper torsion records */
fprintf(fpout,"%s !%d\n",IMPHI,(*param).number_of_improps);
for (i=1; i<=(*param).number_of_improps; i++)
    fprintf(fpout,"%5s %5s %5s %5s %8.3f %8.3f\n",(*param).improp1[i], 
					(*param).improp2[i],
					(*param).improp3[i],
					(*param).improp4[i],
				       	J2CAL*(*param).K_improp[i],
					TOGRD((*param).improp_0[i]));
/* Read nonbonded interactions */
fprintf(fpout,"%s !%d\n",NBONDED,(*param).number_of_nonbonded);
for (i=1; i<=(*param).number_of_nonbonded; i++)
    fprintf(fpout,"%5s %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
				    (*param).atom[i],
				    0.0,
				    J2CAL*(*param).Emin[i],
				    (*param).Rmin[i],
				    0.0,
				    J2CAL*(*param).Emin1_4[i],
				    (*param).Rmin1_4[i]);
/* Read nbfix interactions */
fprintf(fpout,"%s !%d\n",NBFIX,(*param).number_of_nbfix);
for (i=1; i<=(*param).number_of_nbfix; i++)
    fprintf(fpout,"%5s %5s %8.3f %8.3f\n",(*param).atom1[i], 
				    (*param).atom2[i],
				    J2CAL*(*param).EminNBfix[i],
				    (*param).RminNBfix[i]);
}


/*..............................................................*/
/* The procedure reads a MASS RTF file				*/
/*								*/

void read_MASS(char *filename, MASS_record *mass)
{
FILE *fpmass;
char buff[LINELENGTH], help[LINELENGTH];
int i=0;

fpmass=file_open(filename,"r");
while (fgets(buff,LINELENGTH,fpmass)!=NULL)
      {if (strncmp(buff,"MASS",4)==0) i++;
      }
(*mass).number_of_atomtypes=i;
fprintf(stderr,"Number of Atom Types: %d\n",(*mass).number_of_atomtypes);
/* Allocation */
(*mass).number=ivector(1,(*mass).number_of_atomtypes);
(*mass).atom_type=charmatrix(1,(*mass).number_of_atomtypes,0,5);
(*mass).mass=dvector(1,(*mass).number_of_atomtypes);
(*mass).element=charmatrix(1,(*mass).number_of_atomtypes,0,2);

rewind(fpmass);
i=0;
while (fgets(buff,LINELENGTH,fpmass)!=NULL)
      {if (strncmp(buff,"MASS",4)==0) 
	  {i++;
	   sscanf(buff,"%s %d %s %lf %s", help, 
					  &(*mass).number[i],
					   (*mass).atom_type[i],
					  &(*mass).mass[i],
					   (*mass).element[i]);
	 }
      }
}
/*..............................................................*/
/* Procedure reads a CRD_atom from a string taken from a CRD-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/*								*/
/* Syntax: read_CRDatom(line, &record)				*/
/*								*/
void read_CRDatom(CRD_atom *record, int i,char *line)
{
sscanf(line,    "%d",   &((*record).atomno[i]));
sscanf(line+5,  "%d",   &((*record).res_no[i]));
sscanf(line+11, "%c",    &((*record).residue[i][0]));
sscanf(line+12, "%c",    &((*record).residue[i][1]));
sscanf(line+13, "%c",    &((*record).residue[i][2]));
sscanf(line+14, "%c",    &((*record).residue[i][3]));
(*record).residue[i][4]='\0';
sscanf(line+16, "%c",   &((*record).atomid[i][0]));
sscanf(line+17, "%c",   &((*record).atomid[i][1]));
sscanf(line+18, "%c",   &((*record).atomid[i][2]));
sscanf(line+19, "%c",   &((*record).atomid[i][3]));
(*record).atomid[i][4]='\0';
sscanf(line+20, "%lf", &((*record).xyz[i][1]));
sscanf(line+30, "%lf", &((*record).xyz[i][2]));
sscanf(line+40, "%lf", &((*record).xyz[i][3]));
sscanf(line+51, "%c",   &((*record).segid[i][0]));
sscanf(line+52, "%c",   &((*record).segid[i][1]));
sscanf(line+53, "%c",   &((*record).segid[i][2]));
sscanf(line+54, "%c",   &((*record).segid[i][3]));
(*record).segid[i][4]='\0';
sscanf(line+56, "%d",  &((*record).res_no2[i]));
sscanf(line+60, "%lf",&((*record).variable[i]));
}
/*..............................................................*/
/* The procedure prints a CRD-Atom to a file.			*/
/*								*/
/* Syntax: fprint_CRDatom(fptr,&record)				*/
/*								*/
void fprint_CRDatom(FILE *fptr, CRD_atom *record, int i)
{
fprintf(fptr, "%5d %4d %c%c%c%c %c%c%c%c%10.5f%10.5f%10.5f %c%c%c%c %-4d%10.5f\n", 
		(*record).atomno[i],
		(*record).res_no[i],
		(*record).residue[i][0],
		(*record).residue[i][1],
		(*record).residue[i][2],
		(*record).residue[i][3],
		(*record).atomid[i][0],
		(*record).atomid[i][1],
		(*record).atomid[i][2],
		(*record).atomid[i][3],
		(*record).xyz[i][1],
		(*record).xyz[i][2],
		(*record).xyz[i][3],
		(*record).segid[i][0],
		(*record).segid[i][1],
		(*record).segid[i][2],
		(*record).segid[i][3],
		(*record).res_no2[i],
		(*record).variable[i]);
}

/*..............................................................*/
/* allocate an crd-protein 					*/

void crd_alloc(CRD_atom *crd)
{
(*crd).atomid=charmatrix(1,(*crd).number_of_atoms,0,4); 	/*[5][N]*/
(*crd).residue=charmatrix(1,(*crd).number_of_atoms,0,4); 	/*[5][N]*/
(*crd).segid=charmatrix(1,(*crd).number_of_atoms,0,4); 	/*[5][N]*/

(*crd).variable	=dvector(1,(*crd).number_of_atoms);	   

(*crd).atomno=ivector(1,(*crd).number_of_atoms);   
(*crd).res_no2=ivector(1,(*crd).number_of_atoms);   
(*crd).res_no=ivector(1,(*crd).number_of_atoms);   
(*crd).xyz=dmatrix(1,(*crd).number_of_atoms,1,3);   		/*[4][N]*/		  
}

/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void read_PSFatom(PSF_atom *record, int i, char *line)
{
sscanf(line,"%d %s %d %s %s %d %lf %lf %d",   
		&((*record).atomno[i]),
		((*record).segid[i]), 
		&((*record).res_no[i]),
		((*record).residue[i]), 
		((*record).atomid[i]),
		&((*record).number[i]),  
		&((*record).charge[i]), 
		&((*record).mass[i]), 
		&((*record).res_no2[i]));
}

/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Das Format hier ist nicht im Standard-PSF-Format.		*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void fprint_PSFatom(FILE *fptr, PSF_atom *record, int i)
{
fprintf(fptr,"%d %s %d %s %s %d %f %f %d\n",   
		(*record).atomno[i],
		(*record).segid[i], 
		(*record).res_no[i],
		(*record).residue[i], 
		(*record).atomid[i], 
		(*record).number[i],  
		(*record).charge[i], 
		(*record).mass[i], 
		(*record).res_no2[i]);
}
/*..............................................................*/
/* allocate an psf-protein 					*/

void psf_alloc(PSF_atom *psf)
{
(*psf).atomid=charmatrix(1,(*psf).number_of_atoms,0,4); 	/*[5][N]*/
(*psf).atom_type=charmatrix(1,(*psf).number_of_atoms,0,4); 	/*[5][N]*/
(*psf).residue=charmatrix(1,(*psf).number_of_atoms,0,4); 	/*[5][N]*/
(*psf).segid=charmatrix(1,(*psf).number_of_atoms,0,4); 	/*[5][N]*/

(*psf).charge	=dvector(1,(*psf).number_of_atoms);	   
(*psf).mass	=dvector(1,(*psf).number_of_atoms);	   

(*psf).atomno=ivector(1,(*psf).number_of_atoms);   
(*psf).res_no2=ivector(1,(*psf).number_of_atoms);   
(*psf).res_no=ivector(1,(*psf).number_of_atoms);   
(*psf).number=ivector(1,(*psf).number_of_atoms);   
}
/*..............................................................*/
/* Reading the PSF-File						*/
void read_PSF(char *filename, PSF_atom *psf)
{
FILE *fppsf;
char buff[LINELENGTH], help1[LINELENGTH], help2[LINELENGTH];
int i;

fppsf=file_open(filename,"r");
while (fgets(buff,LINELENGTH,fppsf)!=NULL)
      {sscanf(buff,"%d %s",&(*psf).number_of_atoms, help1);
       if (strcmp(help1,"!NATOM")==0) break;
      }
fprintf(stderr,"Number of Atoms in PSF: %d\n",(*psf).number_of_atoms);
psf_alloc(psf); /* Allocates not the bond etc. matrices */
for (i=1; i<=(*psf).number_of_atoms; i++)
	     {fgets(buff,LINELENGTH,fppsf); 
	      read_PSFatom(psf,i,buff);
	     }
/* After one blank line the number of bonds follows */
while (fgets(buff,LINELENGTH,fppsf)!=NULL)
      {sscanf(buff,"%d %s %s",&(*psf).number_of_bonds, help1, help2);
       if (strcmp(help2,"bonds")==0) break;
      }
fprintf(stderr,"Number of Bonds in PSF: %d\n",(*psf).number_of_bonds);
(*psf).bond=imatrix(1,(*psf).number_of_bonds,1,2);
for (i=1; i<=(*psf).number_of_bonds; i++)
    {fscanf(fppsf,"%d %d", &(*psf).bond[i][1], &(*psf).bond[i][2]);
    }
/* After some blank line the number of angles follows */
while (fgets(buff,LINELENGTH,fppsf)!=NULL)
      {sscanf(buff,"%d %s %s",&(*psf).number_of_angles, help1, help2);
       if (strcmp(help2,"angles")==0) break;
      }
fprintf(stderr,"Number of Angles in PSF: %d\n",(*psf).number_of_angles);
(*psf).angle=imatrix(1,(*psf).number_of_angles,1,3);
for (i=1; i<=(*psf).number_of_angles; i++)
    {fscanf(fppsf,"%d %d %d", &(*psf).angle[i][1], 
			      &(*psf).angle[i][2], 
			      &(*psf).angle[i][3]);
    }
/* After some blank line the number of dihedrals follows */
while (fgets(buff,LINELENGTH,fppsf)!=NULL)
      {sscanf(buff,"%d %s %s",&(*psf).number_of_torsions, help1, help2);
       if (strcmp(help2,"dihedrals")==0) break;
      }
fprintf(stderr,"Number of Torsions in PSF: %d\n",(*psf).number_of_torsions);
(*psf).torsion=imatrix(1,(*psf).number_of_torsions,1,4);
for (i=1; i<=(*psf).number_of_torsions; i++)
    {fscanf(fppsf,"%d %d %d %d", &(*psf).torsion[i][1], 
			         &(*psf).torsion[i][2], 
			         &(*psf).torsion[i][3], 
			         &(*psf).torsion[i][4]);
    }

/* After some blank line the number of improper torsions follows */
while (fgets(buff,LINELENGTH,fppsf)!=NULL)
      {sscanf(buff,"%d %s %s",&(*psf).number_of_improp, help1, help2);
       if (strcmp(help2,"impropers")==0) break;
      }
fprintf(stderr,"Number of Improper Torsions in PSF: %d\n",(*psf).number_of_improp);
(*psf).improp=imatrix(1,(*psf).number_of_improp,1,4);
for (i=1; i<=(*psf).number_of_improp; i++)
    {fscanf(fppsf,"%d %d %d %d", &(*psf).improp[i][1], 
			         &(*psf).improp[i][2], 
			         &(*psf).improp[i][3], 
			         &(*psf).improp[i][4]);
    }
      
fclose(fppsf);
}
/*..............................................................*/
/* Reading the CRD-Files 					*/
void read_CRD(char *filename, CRD_atom *crd)
{
FILE *fpcrd;
char buff[LINELENGTH];
int i;

fpcrd=file_open(filename,"r");
fgets(buff,LINELENGTH,fpcrd);
while (buff[0]=='*') {fgets(buff,LINELENGTH,fpcrd);}
sscanf(buff,"%d",&(*crd).number_of_atoms);
fprintf(stderr,"Number of Atoms in CRD: %d\n",(*crd).number_of_atoms);
crd_alloc(crd);
for (i=1; i<=(*crd).number_of_atoms; i++)
	     {fgets(buff,LINELENGTH,fpcrd); 
	      read_CRDatom(crd,i,buff);
	     }
fclose(fpcrd);
}
