/* Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* .....................................................*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "constants.h"
#include "mathematik.h"
#include "random.h"
#include "util_c.h"
#include "nrutil.h"
#include "energy.h"
#include "random.h"
#include "sc-torsion.h"


/****************************************************************/

/* ............................................................ */
/* energy test							*/

void energy_test(double *energy, char **topo, MUS_atom *mus, NB_Para *nb,
	         iListGrid *grid, iList *list)
{
double ener[E_LENGTH+1];
int i;

/* print out the current energy */
fprintf(STDERR,"cuurent energy .\n");
fprint_energy(STDERR,energy);

/* calculate the energy new without non-bonded list */
for (i=1; i<=E_LENGTH; i++) {ener[i]=0.0; }
fprintf(STDERR,"Complete energy calculation.\n");
energy_MUS1(ener, mus, nb, topo);
fprint_energy(STDERR,ener);

/* calculate the enery new with the old non-bonded list */
fprintf(STDERR,"calculate the energy new with the old non-bonded list.\n");
for (i=1; i<=E_LENGTH; i++) {ener[i]=0.0; }
energy_MUS3(ener, mus, nb, topo, grid);
fprint_energy(STDERR,ener);

/* calculate the enery new with the new non-bonded list and new internal coodinates*/
for (i=1; i<=E_LENGTH; i++) {ener[i]=0.0; }
fprintf(STDERR,"calculate the energy new with the new non-bonded list and new internal coordinates.\n");
for (i=1; i<=(*mus).number_of_atoms; i++)
	 {move_in_list(grid, &list[i], (*mus).xyz[i]);
         }
calculate_ic(mus);
energy_MUS3(ener, mus, nb, topo, grid);
fprint_energy(STDERR,ener);
}
/****************************************************************/
/* Some IO-Stuff						*/
/* ............................................................ */
/* procedure prints an energy vector				*/
/*								*/
void fprint_energy(FILE *fpout, double *energy)
{
double ener;
int i;

fprintf(fpout,"Bond  	 	%f\n",energy[E_BOND]);
fprintf(fpout,"Angle 	 	%f\n",energy[E_ANGL]);
fprintf(fpout,"Torsion          %f\n",energy[E_TORS]);
fprintf(fpout,"Improp 	 	%f\n",energy[E_IMPR]);
fprintf(fpout,"Coulomb  	%f\n",energy[E_COUL]);
fprintf(fpout,"LJ	 	%f\n",energy[E_LEJO]);

fprintf(fpout,"SOLV	 	%f\n",energy[E_SOLV]);
fprintf(fpout,"POTN	 	%f\n",energy[E_POTN]);
fprintf(fpout,"KINT	 	%f\n",energy[E_KINT]);
fprintf(fpout,"HBON	 	%f\n",energy[E_HBON]);
fprintf(fpout,"USER	 	%f\n",energy[E_USER]);

ener=0.0;
for (i=1; i<=E_LENGTH-1; i++)
    {ener=ener+energy[i];
    }
energy[E_TOTL]=ener;
fprintf(fpout,"Total	 %f\n",energy[E_TOTL]);
}
/****************************************************************/
/* Solvation energy						*/
/*..............................................................*/
/* Procedure calculates the solvation energy of a molecule with */
/* the simple model:						*/
/* 	E_solv= SUM sigma_i*A_i					*/
/* where sigma_i is a atom specific solvation parameter and	*/
/* A_i is the area of atom i which belongs to the surface of the*/
/* molecule.							*/

double solvation_energy(double *sigma, double *area, int natom)
{
int i;
double energy=0.0;

for (i=1; i<=natom;i++)
	energy = energy + sigma[i]*area[i];

return energy;
}

/****************************************************************/
/* Topology Matrix						*/
/*..............................................................*/
/* Functions sets up an topology matrix.			*/
/* The topology matrix is an integer matrix. If there is no	*/
/* bonded interaction between these two atoms, the integer	*/
/* is for instance set to zero.					*/
/* The integer values used for the particular interactions are	*/
/* defined in "mus_atom.h" via "#define"-statements		*/
/* ATTENTION!							*/
/* If the procedure should be used in a docking context, care 	*/
/* must be taken, that the "number_of_.."-elements have the	*/
/* right size, i.e. are the sum of both MUS.			*/
/* The MUS must be read in via "read_two_MUS"			*/
/* The complex structure must be given to this function		*/
/* !!!!!!!!!!!!!!!!!!!!!!!!!					*/
/* ATTENTION!							*/
/* The procedure assumes that the atom number and the field 	*/
/* index are identical. A renumbering of the atoms records 	*/
/* might be necessary. Care must be taken, that the numbering	*/
/* int BOND-, ANGLE- etc. records is also done in a proper way. */
/* For instance recount_MUS() can be used.			*/
/* !!!!!!!!!!!!!!!!!!!!!!!!!!					*/
/* Here the topology matrix is symmetric			*/
/*								*/
char **topology(MUS_atom *mus)
{
char **topo; /* topology matrix */
int i,j;

topo=charmatrix(1,(*mus).number_of_atoms, 1,(*mus).number_of_atoms);
fprintf(stderr,"My topology matrix need %8.3f kb.\n",
	(double)(sizeof(char)*(*mus).number_of_atoms*(*mus).number_of_atoms)
			/(1024));

/* Initialize with nonbonded interaction */
for (i=1; i<=(*mus).number_of_atoms; i++)
    for (j=1; j<=(*mus).number_of_atoms; j++)
	{topo[j][i]=NON;
	}

/* Assign the value self to diagonal fields */
for (i=1; i<=(*mus).number_of_atoms; i++)
	{topo[i][i]=SELF;
	}

/* Now assigning the values to the triangle matrix */
/* Bonds */
for (i=1; i<=(*mus).number_of_bonds; i++)
    {if ((*mus).bond[i][1]==(*mus).bond[i][2])
	{fprintf (STDERR,"Bond %d\n",i);
	 muerror("Numbers of bonded atoms are identical 1.");
	}
     topo[(*mus).bond[i][1]][(*mus).bond[i][2]]=BOND;
     topo[(*mus).bond[i][2]][(*mus).bond[i][1]]=BOND;
    }

/* Angles */
for (i=1; i<=(*mus).number_of_angles; i++)
    {if ((*mus).angle[i][1]==(*mus).angle[i][3])
	{muerror("Numbers of bonded atoms are identical 2.");
	}
     topo[(*mus).angle[i][1]][(*mus).angle[i][3]]=ANGLE;
     topo[(*mus).angle[i][3]][(*mus).angle[i][1]]=ANGLE;
    }

/* Torsions */
for (i=1; i<=(*mus).number_of_torsions; i++)
    {if ((*mus).torsion[i][1]==(*mus).torsion[i][4])
	{muerror("Numbers of bonded atoms are identical 3.");
	}
     topo[(*mus).torsion[i][1]][(*mus).torsion[i][4]]=TORSION;
     topo[(*mus).torsion[i][4]][(*mus).torsion[i][1]]=TORSION;
    }

return topo;
}


/****************************************************************/
/* bonded energy term						*/
/* The bonded energy terms are defined via Makros (in energy.h) */
/* as well as via functions. Makros are faster!			*/
/*..............................................................*/
double bond_energy(double bond_length, double K_bond, double bond_0)
{
return (K_bond*(bond_length-bond_0)*(bond_length-bond_0));
}

/* .....................................................*/
double angle_energy(double bond_angle, double K_angle, double angle_0)
{
return (K_angle*(bond_angle-angle_0)*(bond_angle-angle_0));
}

/* .....................................................*/
double torsion_energy(double torsion_angle, double K_torsion, double delta, int n)
{
return (K_torsion*(1.0+cos(n*torsion_angle-delta)));
}

/* .....................................................*/
double improp_energy(double improp_angle, double K_improp, double improp_0)
{
return (K_improp*(improp_angle-improp_0)*(improp_angle-improp_0));
}

/****************************************************************/
/* .....................................................*/
double lennard_jones_energy(double e,double s,double r)	
{
double elj;

elj=s/r;
elj=elj*elj;
elj=elj*elj*elj;
elj=e*((elj*elj)-elj);

return elj;
}		
/* .....................................................*/
double switchfkt(double x, double xon, double xoff)
/* schneller, wenn 3*xon und (xoff-xon)^3 als konstanten gehalten werden!!!*/
{
	if ( x <= xon )
		return 1.0;
	if ( x > xoff )
		return 0.0;
	return ((xoff-x)*(xoff-x)*(xoff+2.0*x-3.0*xon))/((xoff-xon)*(xoff-xon)*(xoff-xon));
}
/*..............................................................*/
/* Procedure assignes a number to each atom type		*/
void assign_typeno(MUS_atom *mus, NB_Para *nb_para)
{
char types[MAX_ATOMTYPES][5];
double Emin[MAX_ATOMTYPES];
double radius[MAX_ATOMTYPES];
double Emin1_4[MAX_ATOMTYPES];
double radius1_4[MAX_ATOMTYPES];
int no_type=0;
int i,j, found;

if ((*mus).number_of_atoms==0) 
    muerror("No atoms in MUS in assign_typeno().");

/* assign first atom type */
no_type++;
strcpy(types[no_type], (*mus).atom_type[1]);
radius[no_type]=(*mus).lj_radius[1];
Emin[no_type]=(*mus).E_min[1];
radius1_4[no_type]=(*mus).lj_radius1_4[1];
Emin1_4[no_type]=(*mus).E_min1_4[1];
(*mus).type_no[1]=no_type;

for (i=2; i<=(*mus).number_of_atoms; i++)
    {found=0;
     for (j=1; j<=no_type; j++)
	 {if (strcmp((*mus).atom_type[i],types[j])==0)
	     {found=1;
	      (*mus).type_no[i]=j;
	      if (  (Emin[j]!=(*mus).E_min[i])   
                 || (Emin1_4[j]  !=(*mus).E_min1_4[i])
		 || (radius[j]!=(*mus).lj_radius[i])
                 || (radius1_4[j]!=(*mus).lj_radius1_4[i]))
		 {muwarn("need new Atom type for (in assign_typeno())");
		  fprint_PDBatom(STDERR,mus, i);
		  fprintf(STDERR,"\n\n\n");
		 }
	      break;
	     }
	 }
     if (found==0)
	{no_type++;
	 if (no_type>MAX_ATOMTYPES) muerror("Too many Atomtypes. Abort!");
	 /* then Recompile it with new MAX_ATOMTYPE in mus_atom.h */
	 strcpy(types[no_type], (*mus).atom_type[i]);
	 radius[no_type]=(*mus).lj_radius[i];
	 Emin[no_type]=(*mus).E_min[i];
	 radius1_4[no_type]=(*mus).lj_radius1_4[i];
	 Emin1_4[no_type]=(*mus).E_min1_4[i];
         (*mus).type_no[i]=no_type;
	}
    }
(*nb_para).types=no_type;
fprintf(stderr, "Number of found Atomtypes %d \n", (*nb_para).types);

/* allocating the matrices in the structure */
(*nb_para).atom_type=charmatrix(1,(*nb_para).types,0,5);
(*nb_para).Emin     =dvector(1,(*nb_para).types);
(*nb_para).radius   =dvector(1,(*nb_para).types);
(*nb_para).Emin1_4  =dvector(1,(*nb_para).types);
(*nb_para).radius1_4=dvector(1,(*nb_para).types);
(*nb_para).Eab      =dmatrix(1,(*nb_para).types,1,(*nb_para).types);
(*nb_para).sigma    =dmatrix(1,(*nb_para).types,1,(*nb_para).types);
(*nb_para).Eab1_4   =dmatrix(1,(*nb_para).types,1,(*nb_para).types);
(*nb_para).sigma1_4 =dmatrix(1,(*nb_para).types,1,(*nb_para).types);

/* copy the atom type name, Emin and Rmin in the structure */
for (i=1; i<=(*nb_para).types; i++)
    {strcpy((*nb_para).atom_type[i],types[i]);
     (*nb_para).radius[i]=radius[i];
     (*nb_para).Emin[i]=Emin[i];
     (*nb_para).radius1_4[i]=radius1_4[i];
     (*nb_para).Emin1_4[i]=Emin1_4[i];
      /*fprintf(STDERR,"%4d  %-5s  %8.3lf  %8.3lf\n",i,(*nb_para).atom_type[i],
				      (*nb_para).radius[i],(*nb_para).Emin[i]);*/
    }

}

/*..............................................................*/
/* Procedure assignes a E_ab and a sigma_ab matrix for the 	*/
/* different atom types.					*/
/* it also looks in the NBFIX records and changes the entries,	*/
/* if necessary.						*/
/*								*/
/* Setting up symmetric matrices. 				*/
/* The code is not optimal, but it is called only once		*/
/* Lennard-Jones-Interaction Matrix				*/

void LJ_ia_matrix(MUS_atom *mus, NB_Para *nb_para)
{
int i,j,n;
double help;

help=1.0/pow(2.0,1.0/6.0);
/* Standard calculation with CHARMM formulas */
for (i=1; i<=(*nb_para).types; i++)
    for (j=1; j<=(*nb_para).types; j++)
	{(*nb_para).Eab[i][j]  =sqrt((*nb_para).Emin[i]*(*nb_para).Emin[j]);
	 (*nb_para).sigma[i][j]=((*nb_para).radius[i]+(*nb_para).radius[j])*help;
	 (*nb_para).Eab1_4[i][j]  =sqrt((*nb_para).Emin1_4[i]*(*nb_para).Emin1_4[j]);
	 (*nb_para).sigma1_4[i][j]=((*nb_para).radius1_4[i]+(*nb_para).radius1_4[j])*help;

	}
	 
/* Now looking if there are any NBFIX records, which must be used */
/* Assign the new values if necessary				  */
for (n=1; n<=(*mus).number_of_nbfix; n++)
    for (i=1; i<=(*nb_para).types; i++)
        for (j=1; j<=(*nb_para).types; j++)
	    {if ((strcmp((*mus).fixatom1[n], (*nb_para).atom_type[i])==0)
	      && (strcmp((*mus).fixatom2[n], (*nb_para).atom_type[j])==0))
		{(*nb_para).Eab[i][j]     = -(*mus).E_minFix[n];
		 (*nb_para).Eab[j][i]     = (*nb_para).Eab[i][j];
		 (*nb_para).Eab1_4[i][j]  = -(*mus).E_minFix[n];
		 (*nb_para).Eab1_4[j][i]  = (*nb_para).Eab1_4[i][j];

		 (*nb_para).sigma[i][j]   = (*mus).radiusFix[n]*help;
		 (*nb_para).sigma[j][i]   = (*nb_para).sigma[i][j];
		 (*nb_para).sigma1_4[i][j]= (*mus).radiusFix[n]*help;
		 (*nb_para).sigma1_4[j][i]= (*nb_para).sigma1_4[i][j];
		}
	    }
/* Free the allocated memory of unneeded structure elementes */
free_charmatrix((*nb_para).atom_type, 1, (*nb_para).types,0,5);
free_dvector((*nb_para).Emin1_4,   1, (*nb_para).types);
free_dvector((*nb_para).radius,    1, (*nb_para).types);
free_dvector((*nb_para).radius1_4, 1, (*nb_para).types);
}
/*..............................................................*/
/* calculates the total energy of the given structure		*/
/* ATTENTION							*/
/* It is assumed, that the counting in the atomno entry is the	*/
/* same as the one obtained by simple counting.			*/
/*								*/
/* It is assumed, that the internal coordinates are calculated	*/
/* before this function is called				*/
/*								*/
/* char topology matrix						*/
/* Switch function is used					*/
/* The 1-4 nonbonded interaction is weighted by the factor	*/
/* fact1_4 (standard CHARMM: (*nb).fact1_4=0.5)			*/
/*								*/


double energy_MUS1(double *energy, MUS_atom *mus, NB_Para *nb, char **topo)
{
int i,j;
double dist,ener=0.0;
double bond=0.0,    angle=0.0, 
       torsion=0.0, improp=0.0,
       coulomb=0.0, LJ_energy=0.0;
double *p1, *p2;
double ener_LJ; /* for Lennard-Jonnes    */
double sw_fact; /* Switch function value */

/* bonded energy */
for (i=1; i<= (*mus).number_of_bonds;i++)
    {bond = bond
		  +BondEnergy((*mus).bond_length[i],(*mus).K_bond[i],(*mus).bond_0[i]);
    }
for (i=1; i<= (*mus).number_of_angles;i++)
    {angle = angle
		   +AngleEnergy((*mus).bond_angle[i],(*mus).K_angle[i],(*mus).angle_0[i]);
    }
for (i=1; i<= (*mus).number_of_torsions;i++)
    {torsion = torsion
		     +TorsionEnergy((*mus).torsion_angle[i],(*mus).K_torsion[i],(*mus).delta[i], (*mus).n[i]);
    }
for (i=1; i<= (*mus).number_of_improp;i++)
    {improp = improp
		    +ImpropEnergy((*mus).improp_angle[i],(*mus).K_improp[i],(*mus).improp_0[i]);
    }

/* nonbonded energy */
for (i=1; i<= (*mus).number_of_atoms;i++)
  {p1=(*mus).xyz[i];
   for (j=1; j<i;j++)
    {switch (topo[j][i])
       {case NON : p2=(*mus).xyz[j];
	     	   dist=Distance(p1,p2);
	     	   if (dist!=0.0)
	             {Switch(sw_fact, dist, (*nb).xon, (*nb).xoff);
		      coulomb = coulomb
			       +sw_fact*CoulombEnergy((*mus).charge[i], (*mus).charge[j], dist);
	              LennardJonesEnergy(ener_LJ,(*nb).Eab[(*mus).type_no[i]][(*mus).type_no[j]],\
					 (*nb).sigma[(*mus).type_no[i]][(*mus).type_no[j]],dist);
	              LJ_energy = LJ_energy+ sw_fact*ener_LJ;
		     }
                   else {muwarn("Distance is zero in energy_MUS(). 1");}
		   break;
	 case TORSION : p2=(*mus).xyz[j];
	     		dist=Distance(p1,p2);
	     		if (dist!=0.0)
	     		   {Switch(sw_fact, dist, (*nb).xon, (*nb).xoff);
		            coulomb = coulomb
			           +sw_fact*((*nb).fact1_4*CoulombEnergy((*mus).charge[i], (*mus).charge[j], dist));
	                    LennardJonesEnergy(ener_LJ,(*nb).Eab1_4[(*mus).type_no[i]][(*mus).type_no[j]],\
					       (*nb).sigma1_4[(*mus).type_no[i]][(*mus).type_no[j]],dist);
	     		    LJ_energy = LJ_energy+sw_fact*(ener_LJ);
	 		   }
                        else {muwarn("Distance is zero in energy_MUS(). 2");}
			break;
	 case BOND: break;
	 case ANGLE: break;
	 case SELF: break;
	 default: muwarn("Number not recognized in switch in energy_MUS().");
	}
     }
   }
/* OUTPUT - scaling!*/
LJ_energy=4.0*LJ_energy;
coulomb=CONV*(*nb).inv_eps*coulomb;

energy[E_BOND]=bond;
energy[E_ANGL]=angle;
energy[E_TORS]=torsion;
energy[E_IMPR]=improp;
energy[E_LEJO]=LJ_energy;
energy[E_COUL]=coulomb;

ener = 	 bond+angle+torsion+improp
	+coulomb+LJ_energy;

energy[E_TOTL]=ener;
return ener;
}


/*..............................................................*/
/* calculates the total energy of the given structure		*/
/* ATTENTION							*/
/* It is assumed, that the counting in the atomno entry is the	*/
/* same as the one obtained by simple counting.			*/
/*								*/
/* It is assumed, that the internal coordinates are calculated	*/
/* before this function is called				*/
/*								*/
/* The 1-4 nonbonded interaction is weighted by the factor	*/
/* fact1_4 (standard CHARMM: (*nb).fact1_4=0.5)			*/
/*								*/
/* A grid-cell-based nonbonded list is used.			*/
/* Though the use of the non-bonded list, the each energy term	*/
/* is counted twice, so it is less efficient than energy_MUS1.	*/
/* However, it is added for consistancy check.			*/
/*								*/
/* It returns 1 if calculate_interaction returns 1, in other 	*/
/* cases it returns 0.						*/


int energy_MUS3(double *energy, MUS_atom *mus, NB_Para *nb, 
		   char **topo, iListGrid *grid)
{
int i, back;
double bond=0.0,    angle=0.0, 
       torsion=0.0, improp=0.0,
       ener_LJ=0.0, elect=0.0; /* for Lennard-Jonnes and Electrostatic */
 
/* bonded energy */
for (i=1; i<= (*mus).number_of_bonds;i++)
    {bond = bond
		  +BondEnergy((*mus).bond_length[i],(*mus).K_bond[i],(*mus).bond_0[i]);
    }
for (i=1; i<= (*mus).number_of_angles;i++)
    {angle = angle
		   +AngleEnergy((*mus).bond_angle[i],(*mus).K_angle[i],(*mus).angle_0[i]);
    }
for (i=1; i<= (*mus).number_of_torsions;i++)
    {torsion = torsion
		     +TorsionEnergy((*mus).torsion_angle[i],(*mus).K_torsion[i],(*mus).delta[i], (*mus).n[i]);
    }
for (i=1; i<= (*mus).number_of_improp;i++)
    {improp = improp
		    +ImpropEnergy((*mus).improp_angle[i],(*mus).K_improp[i],(*mus).improp_0[i]);
    }

/* nonbonded energy */
for (i=1; i<= (*mus).number_of_atoms;i++)
 {back=calculate_interaction(&ener_LJ, &elect, i, (*mus).xyz, (*mus).charge, 
			(*mus).type_no, nb, topo, grid);
  if (back==1) {return 1;
	       }
 }

/* Results - scaling!*/
/* The non-bonded interaction must be multiplied with 0.5, since each pair is	*/
/* touched twice in the calculation - but it can be avoided CHANGE		*/
energy[E_BOND]=bond;
energy[E_ANGL]=angle;
energy[E_TORS]=torsion;
energy[E_IMPR]=improp;
energy[E_LEJO]=0.5*4.0*ener_LJ;
energy[E_COUL]=0.5*CONV*(*nb).inv_eps*elect;

return 0;
}

/*..............................................................*/
/* The procedure calulates the interaction between a atom i	*/
/* and all the atom in the boxes which are within the cut-off	*/
/* distance.							*/
/*								*/
/* If ia_atom_box returns 1, the calculation is aborted and 1 	*/
/* is returned, in other cases zero is returned			*/
/*								*/
/* ATTENTION							*/
/* The function returns NOT proper scaled non-bonded		*/
/* interactions. In the calling function must do this!!!	*/
/* (for instance multiplying the LJ-term by 4.0 and the 	*/
/* electrostatic term by CONV)					*/
/*								*/
int calculate_interaction(double *ener_LJ, double *elect, int n, 
			  double **x, double *q, int *type, NB_Para *nb, 
			  char **topo, iListGrid *grid)
{
int ix,iy,iz;
int i,j,k, ih,jh,kh;
int back;
iList *help;

ix=(int)rint(x[n][1]*(*grid).inv_spacing);
iy=(int)rint(x[n][2]*(*grid).inv_spacing);
iz=(int)rint(x[n][3]*(*grid).inv_spacing);
for (i=-(*grid).look; i<=(*grid).look; i++)
  {ih=ix+i;
   if(!((ih>(*grid).imax[1])||(ih<(*grid).imin[1]))) 
     {for (j=-(*grid).look; j<=+(*grid).look; j++)
        {jh=iy+j;
	 if (!((jh>(*grid).imax[2])||(jh<(*grid).imin[2]))) 
	   {for (k=-(*grid).look; k<=+(*grid).look; k++)
	       {kh=iz+k;
	        if (!((kh>(*grid).imax[3])||(kh<(*grid).imin[3])))
	          {help=&(*grid).root[ih][jh][kh];
		   if(NULL!=(*help).next)
		       {back=ia_atom_box(ener_LJ,elect,n,x,q,type,nb,
			      topo,&(*grid).root[ih][jh][kh]);
	                if (back==1) {return 1;}
		       }
	          }
	     	}
	   }
        }
     }
  }
return 0;
}
/*..............................................................*/
/* calculates the non bonded interaction between the atom i	*/
/* and all atoms which are within a given box.			*/
/* The iList-pointer given to the procedure is a pointer to the	*/
/* root-iList, i.e. it contains no data, the calculation should	*/
/* start with (*list).next					*/
/*								*/
/* It is tested whether the interaction between is less than	*/
/* (*nb).r_too_small. If this is the case, 1 is returned else	*/
/* 0 is returned						*/
/*								*/
/* ATTENTION							*/
/* The function returns NOT proper proper scaled non-bonded	*/
/* interactions. In the calling function must do this!!!	*/
/* (for instance multiplying the LJ-term by 4.0 and the 	*/
/* electrostatic term by CONV					*/
/*								*/
int ia_atom_box(double *ener_LeJo, double *elect, int i,
	        double **x, double *q, int *type, NB_Para *nb, 
		char **topo, iList *list)
{
iList *help;
int j;
double *p1, *p2, sw_fact, dist;
double coulomb=0.0, LJ_energy=0.0, ener_LJ=0.0; 

p1=x[i];
help=(*list).next;
/* run through the list */
while (help!=NULL)
   {j=(*help).number;
    help=(*help).next;
    switch (topo[i][j])
       {case NON : p2=x[j];
	     	   dist=Distance(p1,p2);
	     	   if (dist>=(*nb).r_too_small)
	             {Switch(sw_fact, dist, (*nb).xon, (*nb).xoff);
		      coulomb = coulomb
			       +sw_fact*CoulombEnergy(q[i], q[j], dist);
	              LennardJonesEnergy(ener_LJ,(*nb).Eab[type[i]][type[j]],\
					 (*nb).sigma[type[i]][type[j]],dist);
	              LJ_energy = LJ_energy+sw_fact*ener_LJ;
		     }
                   else {/*fprintf(STDERR,"Error: %d %d\n", i,j);*/
			 return 1;
			}
		   break;
	 case TORSION : p2=x[j];
	     		dist=Distance(p1,p2);
	     		if (dist>=(*nb).r_too_small)
	     		   {Switch(sw_fact, dist, (*nb).xon, (*nb).xoff);
		      	    coulomb = coulomb
			           +sw_fact*((*nb).fact1_4*CoulombEnergy(q[i], q[j], dist));
	                    LennardJonesEnergy(ener_LJ,(*nb).Eab1_4[type[i]][type[j]],\
					       (*nb).sigma1_4[type[i]][type[j]],dist);
	     		    LJ_energy = LJ_energy+sw_fact*(ener_LJ);
	 		   }
                   	else {/*fprintf(STDERR,"Error: %d %d\n", i,j);*/
			      return 1;
			     }
			break;
	 case BOND:  break;
	 case ANGLE: break;
	 case SELF:  break;
	 default: muwarn("Number not recognized in switch in energy_MUS().");
	};
     }
*ener_LeJo= *ener_LeJo + LJ_energy;
*elect= *elect + coulomb;
/*fprintf(stdout,"%8.3lf  %8.3lf  -  %8.3lf  %8.3lf\n",coulomb, LJ_energy,
					      *ener_LeJo, *elect); CHANGE */
return 0;
}

/*..............................................................*/
/* Procedure calculates the non-bonded interaction energy 	*/
/* between two specified molecules. 				*/
/* The calculation is aborted, if the distance between two	*/
/* atoms is less than r_too_small and 1 is returned. In other	*/
/* cases 0 is returned						*/
/* The topology matrix is not tested, since it is not assumed 	*/
/* that their are any bonded interation between these two 	*/
/* molecules.							*/
/* CHANGE							*/
/* Beside the energy calculation, a test flexibility indicating	*/
/* vector is created.						*/
/*								*/
/* flexible: flexibility flag vector				*/
/* d2:	     square of the interface criterion distance		*/
/* aa1:	     aminoacid id of protein1				*/
/* aa2:	     aminoacid id of protein2				*/
/*								*/
int ia_energy(double *elect, double *lejo_ener, 
	      double **x1, double *charge1, int *type1, int atoms1,
	      double **x2, double *charge2, int *type2, int atoms2,
	      NB_Para *nb,
	      char *flexible, double d2, int *aa1, int *aa2)
{
int i,j;
int t1, t2;
double *p1, *p2;
double q1, q2;
double lejo=0.0, coulomb=0.0, ener_LJ,sw_fact;
double xoff2, dist;

xoff2=(*nb).xoff*(*nb).xoff;

for (i=1; i<=atoms1; i++)
    {p1=x1[i];
     t1=type1[i];
     q1=charge1[i];
     for (j=1; j<=atoms2; j++)
	 {p2=x2[j];
          dist=SqrDistance(p1,p2);
	  if (dist<=d2) /* Test the atom is in the interface */
	     {flexible[aa1[i]]=FLEX;
	      flexible[aa2[j]]=FLEX;
	     }
	  if (dist<=xoff2) /* do the energy calculation */
	     {t2=type2[j];
              q2=charge2[j];
	      dist=sqrt(dist);
	      if (dist>=(*nb).r_too_small)
	             {Switch(sw_fact, dist, (*nb).xon, (*nb).xoff);
		      coulomb = coulomb
			       +sw_fact*CoulombEnergy(q1, q2, dist);
	              LennardJonesEnergy(ener_LJ,(*nb).Eab[t1][t2],\
					 (*nb).sigma[t1][t2],dist);
	              lejo = lejo+sw_fact*ener_LJ;
		     }
               else {return 1;}
	      }
	 }
    }
*lejo_ener=4.0*lejo;
*elect=CONV*(*nb).inv_eps*coulomb;
return 0;
}

/*..............................................................*/
/* The function calculates the total Coulomb-electrostatic	*/
/* energy of a given coordinate set with corresbonding charges. */
/* NOTE: in contrast to CHARMM the full coulombic energy 	*/
/* between all atoms is calculated				*/
/* ATTENTION:							*/
/* It returns the energy in the units, given to the routine.	*/
/*								*/
double coulomb(int no_atoms, double **xyz, double *charge, double eps)
{
double energy=0.0,r;
int i,j;


for (i=1; i<=no_atoms; i++)
    {for (j=1; j<i; j++)
	{r=Distance(xyz[i],xyz[j]);
	 if (r>0.0)
	    {energy=energy+CoulombEnergy(charge[i],charge[j],r);
	    }
	 else {muwarn("Distance less or equal zero.");
	      }
	}
    }
return energy/eps;
}
