/* Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* Some functions for grid handling			*/
/* .....................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "nrutil.h"
#include "util_c.h"
#include "constants.h"
#include "mathematik.h"
#include "grid.h"

/***************************************************************/
/* Initializing						       */
/* ............................................................*/
/* Initialzies a dGrid with the double 'value'		       */
void Init_dGrid(dGrid *phi, double value)
{
int i,j,k;

for (i=(*phi).imin[1];i<=(*phi).imax[1]; i++)
    for (j=(*phi).imin[2];j<=(*phi).imax[2]; j++)
	for (k=(*phi).imin[3];k<=(*phi).imax[3]; k++)
	    (*phi).grid[i][j][k]=value;
}
/* ............................................................*/
/* Initialzies a iGrid with the double 'value'		       */
void Init_iGrid(iGrid *phi, int value)
{
int i,j,k;

for (i=(*phi).imin[1];i<=(*phi).imax[1]; i++)
    for (j=(*phi).imin[2];j<=(*phi).imax[2]; j++)
	for (k=(*phi).imin[3];k<=(*phi).imax[3]; k++)
	    (*phi).grid[i][j][k]=value;
}

/***************************************************************/
/* Infos about grid parameter				       */
/* ............................................................*/
/* prints infos about the potential 			       */
void cGrid_info(FILE *fpout, cGrid *excl)
{
fprintf(fpout,"MIN: "); PRINT_iVEC(fpout,(*excl).imin);
fprintf(fpout,"MAX: "); PRINT_iVEC(fpout,(*excl).imax);
fprintf(fpout,"Spacing  : %8.3f\n", (*excl).spacing);
fprintf(fpout,"Inverse Spacing  : %8.3f\n", (*excl).inv_spacing);
}

/* ............................................................*/
/* prints infos about the potential 			       */
void iGrid_info(FILE *fpout, iGrid *excl)
{
fprintf(fpout,"MIN: "); PRINT_iVEC(fpout,(*excl).imin);
fprintf(fpout,"MAX: "); PRINT_iVEC(fpout,(*excl).imax);
fprintf(fpout,"Spacing  : %8.3f\n", (*excl).spacing);
fprintf(fpout,"Inverse Spacing  : %8.3f\n", (*excl).inv_spacing);
}

/*..............................................................*/
/* prints infos about the potential 				*/
void dGrid_info(FILE *fpout,dGrid *phi)
{
fprintf(fpout,"MIN: "); PRINT_iVEC(fpout,(*phi).imin);
fprintf(fpout,"MAX: "); PRINT_iVEC(fpout,(*phi).imax);
fprintf(fpout,"Spacing: %8.3f\n", (*phi).spacing);
fprintf(fpout,"Inverse Spacing: %8.3f\n", (*phi).inv_spacing);
}

/*..............................................................*/
/* prints infos about the potential 				*/
void d4Grid_info(FILE *fpout,d4Grid *phi)
{
fprintf(fpout,"MIN: "); PRINT_iVEC(fpout,(*phi).imin);
fprintf(fpout,"MAX: "); PRINT_iVEC(fpout,(*phi).imax);
fprintf(fpout,"Spacing: %8.3f\n", (*phi).spacing);
fprintf(fpout,"Inverse Spacing: %8.3f\n", (*phi).inv_spacing);
}

/****************************************************************/
/* reading and writing of grids from/to binary files		*/
/*..............................................................*/
/* REAL Grids							*/
/*..............................................................*/
/* prints the dGrid to a file in binary mode			*/
void fprint_field_binary(FILE *fpout,dGrid *phi)
{
int i,j,k;

SAVE_fwrite(&((*phi).imax[1]), sizeof(int), 3, fpout);
SAVE_fwrite(&((*phi).imin[1]), sizeof(int), 3, fpout);
SAVE_fwrite(&((*phi).spacing), sizeof(double), 1, fpout);

for (i=(*phi).imin[1];i<=(*phi).imax[1]; i++)
    for (j=(*phi).imin[2];j<=(*phi).imax[2]; j++)
	for (k=(*phi).imin[3];k<=(*phi).imax[3]; k++)
	    SAVE_fwrite(&((*phi).grid[i][j][k]), sizeof(REAL), 1, fpout);
}

/*..............................................................*/
/* reads and allocates a dGrid from a file in binary mode	*/
void fread_field_binary(FILE *fpin, dGrid *phi)
{
int i,j,k;

SAVE_fread(&((*phi).imax[1]), sizeof(int), 3, fpin);
SAVE_fread(&((*phi).imin[1]), sizeof(int), 3, fpin);
SAVE_fread(&((*phi).spacing), sizeof(double), 1, fpin);
(*phi).inv_spacing=1.0/(*phi).spacing;

/* allocation of the tensor */
(*phi).grid=REAL3TENSOR((*phi).imin[1], (*phi).imax[1], 
		        (*phi).imin[2], (*phi).imax[2], 
		        (*phi).imin[3], (*phi).imax[3]);

for (i=(*phi).imin[1];i<=(*phi).imax[1]; i++)
    for (j=(*phi).imin[2];j<=(*phi).imax[2]; j++)
	for (k=(*phi).imin[3];k<=(*phi).imax[3]; k++)
	    SAVE_fread(&((*phi).grid[i][j][k]), sizeof(REAL), 1, fpin);
}


/*..............................................................*/
/* integer Grids						*/
/*..............................................................*/
/* prints the integer grid to a file in binary mode		*/
void fwrite_iGrid(FILE *fpout,iGrid *excl)
{
int i,j,k;

SAVE_fwrite(&((*excl).imax[1]), sizeof(int), 3, fpout);
SAVE_fwrite(&((*excl).imin[1]), sizeof(int), 3, fpout);
SAVE_fwrite(&((*excl).spacing), sizeof(double), 1, fpout);

for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
    for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
	for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
	    SAVE_fwrite(&((*excl).grid[i][j][k]), sizeof(int), 1, fpout);
}
/*..............................................................*/
/* reads and allocates a integer grid from a file in binary mode*/
void fread_iGrid(FILE *fpin, iGrid *excl)
{
int i,j,k;

SAVE_fread(&((*excl).imax[1]), sizeof(int), 3, fpin);
SAVE_fread(&((*excl).imin[1]), sizeof(int), 3, fpin);
SAVE_fread(&((*excl).spacing), sizeof(double), 1, fpin);
(*excl).inv_spacing=1.0/(*excl).spacing;

/* allocation of the tensor */
(*excl).grid=i3tensor((*excl).imin[1], (*excl).imax[1], 
		      (*excl).imin[2], (*excl).imax[2], 
		      (*excl).imin[3], (*excl).imax[3]);

for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
    for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
	for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
	    SAVE_fread(&((*excl).grid[i][j][k]), sizeof(int), 1, fpin);
}

/****************************************************************/
/* Some Utilities						*/
/*..............................................................*/
/* interpolates linearly between 8 grid corner pointes and 	*/
/* returns the value represented at the grid in phi at a given 	*/
/* coordinate xyz.						*/
/* weighted averaging						*/
/* If a atoms falls outside the grid, its energy contribution   */
/* is assumed to be zero, which is a good guess since the 	*/
/* potential at this region should be zero			*/

double interpolate(dGrid *phi, double *xyz, int *warn)
{
double phi_n;
double a,b,c,d,e,f;
int i,j,k,l,m,n;

i=rint(xyz[1]*(*phi).inv_spacing); 
j=rint(xyz[2]*(*phi).inv_spacing); 
k=rint(xyz[3]*(*phi).inv_spacing); 

l=i+1; m=j+1; n=k+1;

#define WARNING {*warn++; phi_n=0.0; goto end;}
if ((i<(*phi).imin[1])||(i>(*phi).imax[1])) WARNING;
if ((j<(*phi).imin[2])||(j>(*phi).imax[2])) WARNING;
if ((k<(*phi).imin[3])||(k>(*phi).imax[3])) WARNING;
if ((l<(*phi).imin[1])||(l>(*phi).imax[1])) WARNING;
if ((m<(*phi).imin[2])||(m>(*phi).imax[2])) WARNING;
if ((n<(*phi).imin[3])||(n>(*phi).imax[3])) WARNING;
#undef WARNING
     
a=(xyz[1]-i*(*phi).spacing)*(*phi).inv_spacing;
b=(xyz[2]-j*(*phi).spacing)*(*phi).inv_spacing;
c=(xyz[3]-k*(*phi).spacing)*(*phi).inv_spacing;

d=1.0-a; e=1.0-b; f=1.0-c;

phi_n=   d*e*f*(*phi).grid[i][j][k]
	+a*e*f*(*phi).grid[l][j][k]+d*b*f*(*phi).grid[i][m][k]+d*e*c*(*phi).grid[i][j][n]
	+a*b*f*(*phi).grid[l][m][k]+a*e*c*(*phi).grid[l][j][n]+d*b*c*(*phi).grid[i][m][n]
	+a*b*c*(*phi).grid[l][m][n];

end: /* Label */
return phi_n;

}
/*..............................................................*/
/* Function calculates the distance between a grid point 	*/
/* [i][j][k] (with the spacing "spacing") to a cartesian 	*/
/* coordinates co.						*/

double dist(int i, int j, int k, double spacing, double *co)
{
double vec[4];

vec[1]=i*spacing;
vec[2]=j*spacing;
vec[3]=k*spacing;

return Distance(co, vec);
}

/*..............................................................*/
/* function returns the corners of a box which completely	*/
/* enclose a given coordinate set xyz.				*/

void min_max_coord(double *min, double *max, double **xyz, int atoms)
{
int i;

max[1]=xyz[1][1]; max[2]=xyz[1][2]; max[3]=xyz[1][3];
min[1]=xyz[1][1]; min[2]=xyz[1][2]; min[3]=xyz[1][3];

for (i=2; i<=atoms; i++)
	{if (max[1]<xyz[i][1]) {max[1]=xyz[i][1];}
	 if (max[2]<xyz[i][2]) {max[2]=xyz[i][2];}
	 if (max[3]<xyz[i][3]) {max[3]=xyz[i][3];}
	 if (min[1]>xyz[i][1]) {min[1]=xyz[i][1];}
	 if (min[2]>xyz[i][2]) {min[2]=xyz[i][2];}
	 if (min[3]>xyz[i][3]) {min[3]=xyz[i][3];}
	}
}

/*..............................................................*/
/* Function copies grid <phi> to <phi2>				*/
/* only the grid entries, the dimensions of the grid are assumed*/
/* to be the same. It is not tested in this function		*/

void copy_grid(dGrid *phi2, dGrid *phi)
{
int i,j,k;

for (i=(*phi).imin[1]+1; i<=(*phi).imax[1]-1;i++)
    for (j=(*phi).imin[2]+1; j<=(*phi).imax[2]-1;j++)
	for (k=(*phi).imin[3]+1; k<=(*phi).imax[3]-1;k++)
	    {(*phi2).grid[i][j][k]=(*phi).grid[i][j][k];
	    }
}


/****************************************************************/
/* Make the grid dimensions to odd integer if they aren't	*/

/*..............................................................*/
/* make the integers bigger					*/
/*								*/

void odding_up(int *ivec, int len)
{
int i;

for (i=1; i<=len; i++)
    {if ((ceil((double)ivec[i]/2.0))!=(floor((double)ivec[i]/2.0)))
	{ivec[i]++;
	}
    }
}
/*..............................................................*/
/* make the integers smaller					*/
/*								*/

void odding_down(int *ivec, int len)
{
int i;

for (i=1; i<=len; i++)
    {if ((ceil((double)ivec[i]/2.0))!=(floor((double)ivec[i]/2.0)))
	{ivec[i]--;
	}
    }
}
