/* Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* .....................................................*/

/* dGrid = float-grid					*/
/* for mapping spatial properties on a grid		*/
#ifndef FLOAT_GRID
#define REAL 		float
#define REAL3TENSOR     f3tensor
#define REAL4TENSOR     f4tensor
typedef struct {REAL  ****grid;         /* lattice      	*/
		int    imin[5];	 	/* extension    	*/
		int    imax[5];   	/* extension 	  	*/
		double spacing;   	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }d4Grid;

typedef struct {REAL  ***grid;    	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];   	/* extension 	  	*/
		double spacing;   	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }dGrid;
#define FLOAT_GRID
#endif

/* iGrid = integer-grid 				*/
/* for instance for exclusion grids			*/
#ifndef INT_GRID
typedef struct {int    ***grid;   	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];  	/* extension 	  	*/
		double spacing;        	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }iGrid;
#define INT_GRID
#endif

/* cGrid = char-grid 					*/
/* for instance for exclusion grids			*/
#ifndef CHAR_GRID
typedef struct {char   ***grid;   	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];  	/* extension 	  	*/
		double spacing;        	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }cGrid;
#define CHAR_GRID
#endif

/*------------------------------------------------------*/
/*	Functions					*/
/*------------------------------------------------------*/
/* IO-Routines						*/
void fprint_field_binary(FILE *fpout,dGrid *phi);
void fread_field_binary(FILE *fpin, dGrid *phi);
void fwrite_iGrid(FILE *fpout,iGrid *excl);
void fread_iGrid(FILE *fpin, iGrid *excl);


/* Grid initialization					*/
void Init_dGrid(dGrid *phi, double value);
void Init_iGrid(iGrid *phi, int value);

/* Infos about grid parameters				*/
void dGrid_info(FILE *fpout, dGrid *phi);
void d4Grid_info(FILE *fpout,d4Grid *phi);
void iGrid_info(FILE *fpout, iGrid *excl);
void cGrid_info(FILE *fpout, cGrid *excl);

/* Utilities						*/
double interpolate(dGrid *phi, double *xyz, int *warn);
double dist(int i, int j, int k, double spacing, double *co);
void min_max_coord(double *min, double *max, double **xyz, int atoms);
void copy_grid(dGrid *phi2, dGrid *phi);

/* Odding		*/
void odding_up(int *ivec, int len);
void odding_down(int *ivec, int len);

