/* Matthias Ullmann, Bayreuth Jan 2006                          */
/* some routines for matrix calculations.                       */
/*                                                              */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#include "util_c.h"
#include "mathematik.h"

/* .....................................................*/
/*                                                      */
double MaxElement(double **c, int n, int m)
{
    double maxele=0.0;
    int i,j;

    for (i=1; i<=n; i++)
        for (j=1; j<=m; j++)
            if (maxele<c[i][j]) maxele=c[i][j];

    return maxele;
}
/* .....................................................*/
/*                                                      */
void PrintMatrix(FILE *fp, double **c, int n, int m)
{

    int i,j;

   for (i=1; i<=n; i++)
       {for (j=1; j<=m; j++)
           {fprintf(fp,"%10.3e  ",c[i][j]);
           }
        fprintf(fp,"\n");
       }
}
/*............................................................. */
/* Scalarproduct of two vectors                                 */
/*  sum = a[na]*b[nb]                                           */
/*                                                              */
double ScalarProd(double *a, int na, double *b, int nb)
{
    int i;
    double sum=0.0;

    if (na!=nb) muerror("na!=nb in ScalarProd()");

    for (i=1; i<=na; i++)
        {sum=sum+(a[i]*b[i]);
        }
    return sum;
}

/*............................................................. */
/* multiplication of two matrices:                              */
/*         C[nc][mc] = A[ma][na]*B[mb][nb]                      */
/*                                                              */

void MultMatrix(double **C, int mc, int nc, 
                double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j, k;

    /* test if the formats are ok */
    if (ma!=mc) muerror("ma!=mc in MultMatrix()");
    if (na!=mb) muerror("na!=mb in MultMatrix()");
    if (nb!=nc) muerror("nb!=nc in MultMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = 0.0;
             for (k=1; k<=na;  k++)
                 C[i][j] = C[i][j] + A[i][k]*B[k][j];
            }
        }

}

/*............................................................. */
/* substraction of two matrices:                                */
/*         C[nc][mc] = A[ma][na]-B[mb][nb]                      */
/*                                                              */

void SubstractMatrix(double **C, int mc, int nc, 
                double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) muerror("ma!=mc in SubstractMatrix()");
    if (ma!=mb) muerror("ma!=mb in SubstractMatrix()");
    if (na!=nc) muerror("na!=nc in SubstractMatrix()");
    if (na!=nb) muerror("na!=nb in SubstractMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = A[i][j] - B[i][j];
            }
        }

}
/*............................................................. */
/* Addition of two matrices:                                    */
/*         C[nc][mc] = A[ma][na]+B[mb][nb]                      */
/*                                                              */

void AddMatrix(double **C, int mc, int nc, 
               double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) muerror("ma!=mc in AddMatrix()");
    if (ma!=mb) muerror("ma!=mb in AddMatrix()");
    if (na!=nc) muerror("na!=nc in AddMatrix()");
    if (na!=nb) muerror("na!=nb in AddMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = A[i][j] + B[i][j];
            }
        }
}
/*............................................................. */
/* Multiplication of a  matrix  with a scalar                   */
/*         C[nc][mc] = s*A[ma][na]                              */
/*                                                              */

void ScalarMatrix(double **C, int mc, int nc, 
                  double s, double **A, int ma, int na)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) muerror("ma!=mc in ScalarMatrix()");
    if (na!=nc) muerror("na!=nc in ScalarMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = s*A[i][j];
            }
        }
}
/*............................................................. */
/* Multiplication of a  matrix  with a scalar                   */
/*         c[nc] = s*A[ma][na]                              */
/*                                                              */

void ScalarVector(double *c, int nc, 
                  double s, double *a, int na)
{
    int i;

    /* test if the formats are ok */
    if (na!=nc) muerror("na!=nc in ScalarVector()");
        
    for (i=1; i<=nc; i++)
        {c[i] = s*a[i];
        }
}
/*............................................................. */
/* Transpose a  matrix                                          */
/*        A[ma][na] ---> C[nc][mc]                              */
/*                                                              */

void TransposeMatrix(double **C, int mc, int nc, 
                     double **A, int ma, int na)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=nc) muerror("ma!=nc in TransposeMatrix()");
    if (na!=mc) muerror("na!=mc in TransposeMatrix()");
        
    for (i=1; i<=ma; i++)
        {for (j=1; j<=na; j++)
            {C[j][i] = A[i][j];
            }
        }
}
/*............................................................. */
/* multiplication of a matrix with a vector                     */
/*        c[mc] = A[ma][na] * b[nb]                         */
/*                                                              */

void MatrixVectMult(double *c, int mc, 
                    double **A, int ma, int na, double *b, int nb)
{
    int i, k;

    /* test if the formats are ok */
    if (ma!=mc) muerror("ma!=mc in MultMatrix()");
    if (nb!=mc) muerror("nb!=nc in MultMatrix()");
        
    for (i=1; i<=mc; i++)
        {c[i] = 0.0;
         for (k=1; k<=na;  k++)
             c[i] = c[i] + A[i][k]*b[k];
        }
}
