/* Matthias Ullmann                                     */
/* Berlin, 01.10.1995                   */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "surface.h"
#include "energy.h"
#include "grid.h"
#include "nb-list.h"
#include "mathematik.h"
#include "constants.h"
#include "nrutil.h"
#include "util_c.h"
#include "mus_atom.h"
#include "properties.h"

/* ------------------------------------------------------------ */
/* Procedure allocates the single vectors and matricies in a    */
/* MUS structure. The structure component "number_of_atoms" must*/
/* be assigned, befor calling this procedure.           */
/*                              */
/* ------------------------------------------------------------ */

/*
a[i] is a pointer to a[i][0]
*/

void MUS_alloc(MUS_atom *mus_atom)
{
/* Test */
if ((*mus_atom).number_of_atoms==0)
    {fprintf(STDERR, "number_of_atoms=0 - Abort in MUS_alloc.\n");
         exit(EXIT_FAILURE);
        }
/* Allocation */        /* rows */      /* columns */
(*mus_atom).identifier=charmatrix(1,(*mus_atom).number_of_atoms,0,6);   /*[7][N]*/
(*mus_atom).atomid=charmatrix(1,(*mus_atom).number_of_atoms,0,4);   /*[5][N]*/
(*mus_atom).atom_type=charmatrix(1,(*mus_atom).number_of_atoms,0,5);    /*[5][N]*/
(*mus_atom).residue=charmatrix(1,(*mus_atom).number_of_atoms,0,3);  /*[4][N]*/
(*mus_atom).xyz=dmatrix(1,(*mus_atom).number_of_atoms,1,3);         /*[4][N]*/

(*mus_atom).alternate=charvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).chain=charvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).insert=charvector(1,(*mus_atom).number_of_atoms);

(*mus_atom).hbond=charvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).ion=charvector(1,(*mus_atom).number_of_atoms);

(*mus_atom).res_no=ivector(1,(*mus_atom).number_of_atoms);
(*mus_atom).atomno=ivector(1,(*mus_atom).number_of_atoms);
(*mus_atom).type_no=ivector(1,(*mus_atom).number_of_atoms);
(*mus_atom).aa_no=ivector(1,(*mus_atom).number_of_atoms);           /* for counting */

(*mus_atom).charge  =dvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).formal_charge=dvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).mass    =dvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).radius  =dvector(1,(*mus_atom).number_of_atoms);

(*mus_atom).lj_radius   =dvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).E_min   =dvector(1,(*mus_atom).number_of_atoms);    /* VdW E_min            */
(*mus_atom).lj_radius1_4=dvector(1,(*mus_atom).number_of_atoms);
(*mus_atom).E_min1_4    =dvector(1,(*mus_atom).number_of_atoms);    /* VdW E_min            */
(*mus_atom).surface =dvector(1,(*mus_atom).number_of_atoms);    /* surface accessiblility   */
(*mus_atom).coupling    =dvector(1,(*mus_atom).number_of_atoms);    /* electronic coupling      */
(*mus_atom).temperature =dvector(1,(*mus_atom).number_of_atoms);    /* temperature factor       */
(*mus_atom).dist_cm     =dvector(1,(*mus_atom).number_of_atoms);    /* distance from the center of mass*/
(*mus_atom).sigma   =dvector(1,(*mus_atom).number_of_atoms);    /* sigma for solvent accessibiliy */

if ((*mus_atom).number_of_bonds !=0)
   {(*mus_atom).bond    =imatrix(1,(*mus_atom).number_of_bonds,1,2);
    (*mus_atom).K_bond  =dvector(1,(*mus_atom).number_of_bonds);
    (*mus_atom).bond_0  =dvector(1,(*mus_atom).number_of_bonds);
    (*mus_atom).bond_length=dvector(1,(*mus_atom).number_of_bonds);
   }

if ((*mus_atom).number_of_angles !=0)
   {(*mus_atom).angle   =imatrix(1,(*mus_atom).number_of_angles,1,3);
    (*mus_atom).bond_angle=dvector(1,(*mus_atom).number_of_angles);
    (*mus_atom).K_angle =dvector(1,(*mus_atom).number_of_angles);
    (*mus_atom).angle_0 =dvector(1,(*mus_atom).number_of_angles);
    (*mus_atom).K_ub    =dvector(1,(*mus_atom).number_of_angles);
    (*mus_atom).dist_ub_0 =dvector(1,(*mus_atom).number_of_angles);
   }

if ((*mus_atom).number_of_torsions !=0)
   {(*mus_atom).torsion =imatrix(1,(*mus_atom).number_of_torsions,1,4);
    (*mus_atom).torsion_angle=dvector(1,(*mus_atom).number_of_torsions);
    (*mus_atom).K_torsion=dvector(1,(*mus_atom).number_of_torsions);
    (*mus_atom).delta   =dvector(1,(*mus_atom).number_of_torsions);
    (*mus_atom).n   =ivector(1,(*mus_atom).number_of_torsions);
   }

if ((*mus_atom).number_of_improp !=0)
   {(*mus_atom).improp  =imatrix(1,(*mus_atom).number_of_improp,1,4);
    (*mus_atom).improp_angle=dvector(1,(*mus_atom).number_of_improp);
    (*mus_atom).K_improp=dvector(1,(*mus_atom).number_of_improp);
    (*mus_atom).improp_0=dvector(1,(*mus_atom).number_of_improp);
   }
if ((*mus_atom).number_of_nbfix !=0)
   {(*mus_atom).fixatom1=charmatrix(1,(*mus_atom).number_of_nbfix,0,4);
    (*mus_atom).fixatom2=charmatrix(1,(*mus_atom).number_of_nbfix,0,4);
    (*mus_atom).E_minFix=dvector(1,(*mus_atom).number_of_nbfix);
    (*mus_atom).radiusFix=dvector(1,(*mus_atom).number_of_nbfix);
   }

(*mus_atom).cm=dvector(1,3);            /* center of mass     */
(*mus_atom).save_cm=dvector(1,3);       /* saveing of center of mass     */
(*mus_atom).new_cm=dvector(1,3);        /* new center of mass */
(*mus_atom).dipolvec=dvector(1,3);      /* dipol vector       */
(*mus_atom).diff_matrix=dmatrix(1,6,1,6);
}

/* ------------------------------------------------------------ */
/* Procedure frees the single vectors and matricies in a    */
/* MUS structure.                       */
/*                              */
/* ------------------------------------------------------------ */
void MUS_free(MUS_atom *mus_atom)
{
/* Test */
if ((*mus_atom).number_of_atoms==0)
    {fprintf(STDERR, "number_of_atoms=0 - Abort in MUS_free.\n");
         exit(EXIT_FAILURE);
        }
/* Allocation */        /* rows */      /* columns */
free_charmatrix((*mus_atom).identifier,1,(*mus_atom).number_of_atoms,0,6);  /*[7][N]*/
free_charmatrix((*mus_atom).atomid,1,(*mus_atom).number_of_atoms,0,4);  /*[5][N]*/
free_charmatrix((*mus_atom).atom_type,1,(*mus_atom).number_of_atoms,0,4);   /*[5][N]*/
free_charmatrix((*mus_atom).residue,1,(*mus_atom).number_of_atoms,0,3); /*[4][N]*/

free_dmatrix((*mus_atom).xyz,1,(*mus_atom).number_of_atoms,1,3);        /*[4][N]*/
free_dmatrix((*mus_atom).save_xyz,1,(*mus_atom).number_of_atoms,1,3);       /*[4][N]*/  /* initial coordinates */
free_dmatrix((*mus_atom).new_xyz,1,(*mus_atom).number_of_atoms,1,3);    /*[4][N]*/

free_charvector((*mus_atom).alternate,1,(*mus_atom).number_of_atoms);
free_charvector((*mus_atom).chain,1,(*mus_atom).number_of_atoms);
free_charvector((*mus_atom).insert,1,(*mus_atom).number_of_atoms);

free_charvector((*mus_atom).hbond,1,(*mus_atom).number_of_atoms);
free_charvector((*mus_atom).ion,1,(*mus_atom).number_of_atoms);

free_ivector((*mus_atom).res_no,1,(*mus_atom).number_of_atoms);
free_ivector((*mus_atom).atomno,1,(*mus_atom).number_of_atoms);
free_ivector((*mus_atom).type_no,1,(*mus_atom).number_of_atoms);

free_dvector((*mus_atom).charge,1,(*mus_atom).number_of_atoms);
free_dvector((*mus_atom).formal_charge,1,(*mus_atom).number_of_atoms);
free_dvector((*mus_atom).mass,1,(*mus_atom).number_of_atoms);
free_dvector((*mus_atom).radius,1,(*mus_atom).number_of_atoms);

free_dvector((*mus_atom).lj_radius,1,(*mus_atom).number_of_atoms);
free_dvector((*mus_atom).E_min,1,(*mus_atom).number_of_atoms);  /* VdW E_min            */
free_dvector((*mus_atom).lj_radius1_4,1,(*mus_atom).number_of_atoms);
free_dvector((*mus_atom).E_min1_4,1,(*mus_atom).number_of_atoms);   /* VdW E_min            */
free_dvector((*mus_atom).surface,1,(*mus_atom).number_of_atoms);    /* surface accessiblility   */
free_dvector((*mus_atom).coupling,1,(*mus_atom).number_of_atoms);   /* electronic coupling      */
free_dvector((*mus_atom).temperature,1,(*mus_atom).number_of_atoms);    /* temperature factor       */
free_dvector((*mus_atom).dist_cm,1,(*mus_atom).number_of_atoms);    /* distance from the center of mass*/
free_dvector((*mus_atom).sigma,1,(*mus_atom).number_of_atoms);  /* sigma for solvent accessibiliy */

if ((*mus_atom).number_of_bonds !=0)
   {free_imatrix((*mus_atom).bond,1,(*mus_atom).number_of_bonds,1,2);
    free_dvector((*mus_atom).K_bond,1,(*mus_atom).number_of_bonds);
    free_dvector((*mus_atom).bond_0,1,(*mus_atom).number_of_bonds);
    free_dvector((*mus_atom).bond_length,1,(*mus_atom).number_of_bonds);
   }

if ((*mus_atom).number_of_angles !=0)
   {free_imatrix((*mus_atom).angle,1,(*mus_atom).number_of_angles,1,3);
    free_dvector((*mus_atom).bond_angle,1,(*mus_atom).number_of_angles);
    free_dvector((*mus_atom).K_angle,1,(*mus_atom).number_of_angles);
    free_dvector((*mus_atom).angle_0,1,(*mus_atom).number_of_angles);
    free_dvector((*mus_atom).K_ub,1,(*mus_atom).number_of_angles);
    free_dvector((*mus_atom).dist_ub_0,1,(*mus_atom).number_of_angles);
   }

if ((*mus_atom).number_of_torsions !=0)
   {free_imatrix((*mus_atom).torsion,1,(*mus_atom).number_of_torsions,1,4);
    free_dvector((*mus_atom).torsion_angle,1,(*mus_atom).number_of_torsions);
    free_dvector((*mus_atom).K_torsion,1,(*mus_atom).number_of_torsions);
    free_dvector((*mus_atom).delta,1,(*mus_atom).number_of_torsions);
    free_ivector((*mus_atom).n,1,(*mus_atom).number_of_torsions);
   }

if ((*mus_atom).number_of_improp !=0)
   {free_imatrix((*mus_atom).improp,1,(*mus_atom).number_of_improp,1,4);
    free_dvector((*mus_atom).improp_angle,1,(*mus_atom).number_of_improp);
    free_dvector((*mus_atom).K_improp,1,(*mus_atom).number_of_improp);
    free_dvector((*mus_atom).improp_0,1,(*mus_atom).number_of_improp);
   }

if ((*mus_atom).number_of_nbfix !=0)
   {free_charmatrix((*mus_atom).fixatom1,1,(*mus_atom).number_of_nbfix,1,2);
    free_charmatrix((*mus_atom).fixatom2,1,(*mus_atom).number_of_nbfix,1,2);
    free_dvector((*mus_atom).E_minFix,1,(*mus_atom).number_of_nbfix);
    free_dvector((*mus_atom).radiusFix,1,(*mus_atom).number_of_nbfix);
   }

free_dvector((*mus_atom).cm,1,3);           /* center of mass     */
free_dvector((*mus_atom).save_cm,1,3);      /* saveing of center of mass     */
free_dvector((*mus_atom).new_cm,1,3);           /* new center of mass */
free_dvector((*mus_atom).dipolvec,1,3);         /* dipol vector       */
free_dmatrix((*mus_atom).diff_matrix,1,6,1,6);      /*[7][7]*/
}

/* ------------------------------------------------------------ */
/* Procedure assignes aminoacids                */
/* It assumes that different residues have different residues   */
/* numbers. Numbering via insert is not allowed         */
/* ------------------------------------------------------------ */
void assign_aminoacid(MUS_atom *protein)
{
int i, j, aa, count=0;

/* Counting of the aminoacids */
j=1;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {if ((*protein).res_no[i] != (*protein).res_no[i-1]) j++;
    }
(*protein).aminoacids=j;

/* fprintf(STDERR,"AA: %d\n",(*protein).aminoacids); */

/* Allocation */
(*protein).aa=imatrix(1,(*protein).aminoacids,1,2);

/* Assignment */
j=1; aa=1;
(*protein).aa[aa][1]=1;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {if ((*protein).res_no[i] != (*protein).res_no[i-1])
    {(*protein).aa[aa][2]=j;
     j=1;aa++;
     (*protein).aa[aa][1]=i;
    }
     else j++;
    }
(*protein).aa[aa][2]=j;
/* assign the value to aa_no */
for (i=1;  i<=(*protein).aminoacids; i++)
    {for (j=0; j<(*protein).aa[i][2]; j++)
     {count++;
      (*protein).aa_no[count]=i;
     }
    }
}
/* ------------------------------------------------------------ */
/* prints an aminoacid with a given number          */
/* Output is in PDB-Format                  */
/* ------------------------------------------------------------ */
void fprint_aminoacid(FILE *fpout, MUS_atom *protein, int aa_no)
{
int i;

if (aa_no>(*protein).aminoacids)
   {muerror("Aminoacid number not existent. Error in fprint_aminoacid().");
   }
for (i=0; i<(*protein).aa[aa_no][2];i++)
   {fprint_PDBatom(fpout, protein, (*protein).aa[aa_no][1]+i);
   }

}

/* ------------------------------------------------------------ */
/* Procedure reads in a MUS                     */
/* A lot of things done here are not nessesarry in all      */
/* application however, it is not very time consuming       */
/* and this function is only called a few times during a run    */
/* ------------------------------------------------------------ */

void read_MUS(char *filename, MUS_atom *mus_atom)
{

read_MUS2(filename, mus_atom);
properties(mus_atom);
(*mus_atom).new_cm[1]= (*mus_atom).cm[1];
(*mus_atom).new_cm[2]= (*mus_atom).cm[2];
(*mus_atom).new_cm[3]= (*mus_atom).cm[3];
on_surf(mus_atom);
save_initial_coordinates(mus_atom);
search_double_entries(mus_atom);
}
/* ------------------------------------------------------------ */
/* Procedure reads in a MUS                     */
/* A lot of things done here are not nessesarry in all      */
/* application however, it is not very time consuming       */
/* and this function is only called a few times during a run    */
/* no search for double entries                 */
/* ------------------------------------------------------------ */

void read_MUS2(char *filename, MUS_atom *mus_atom)
{
FILE *fpin;
char line[LINELENGTH];
int atom=0, bond=0, angle=0, torsion=0, improp=0, nbfix=0;


fpin=file_open(filename,"r");

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
       if  (!strncmp(line,"BOND  ",6)) bond++;
       if  (!strncmp(line,"ANGLE ",6)) angle++;
       if  (!strncmp(line,"DIHE  ",6)) torsion++;
       if  (!strncmp(line,"IMPROP",6)) improp++;
       if  (!strncmp(line,"NBFIX ",6)) nbfix++;
      }
(*mus_atom).number_of_atoms=atom;
(*mus_atom).number_of_bonds=bond;
(*mus_atom).number_of_angles=angle;
(*mus_atom).number_of_torsions=torsion;
(*mus_atom).number_of_improp=improp;
(*mus_atom).number_of_nbfix=nbfix;

/*fprintf(STDERR,"Number of Atoms: %d\n",(*mus_atom).number_of_atoms);
fprintf(STDERR,"Number of Torsions: %d\n",(*mus_atom).number_of_torsions);*/

MUS_alloc(mus_atom);
rewind(fpin);

atom=0; bond=0; angle=0; torsion=0; improp=0; nbfix=0;

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
      {atom++; read_MUSatom(mus_atom, atom, line);}
       if (!strncmp(line,"BOND  ",6))
      {bond++; read_MUSbond(mus_atom, bond, line);}
       if (!strncmp(line,"ANGLE ",6))
      {angle++; read_MUSangle(mus_atom, angle, line);}
       if (!strncmp(line,"DIHE  ",6))
      {torsion++; read_MUStorsion(mus_atom, torsion, line);}
       if (!strncmp(line,"IMPROP",6))
      {improp++; read_MUSimprop(mus_atom, improp, line);}
       if (!strncmp(line,"NBFIX ",6))
      {nbfix++; read_MUSnbfix(mus_atom, nbfix, line);}
      }
/* same additional allocation */
(*mus_atom).save_xyz=dmatrix(1,(*mus_atom).number_of_atoms,1,3);
(*mus_atom).new_xyz=dmatrix(1,(*mus_atom).number_of_atoms,1,3);

assign_aminoacid(mus_atom);
fclose(fpin);
}
/* ------------------------------------------------------------ */
/* Procedure reads in a MUS                     */
/* A lot of things done here are not nessesarry in all      */
/* application however, it is not very time consuming       */
/* and this function is only called a few times during a run    */
/*                              */
/* The allocation is performed in a way that both proteins are  */
/* neightbors in the memory, so that a loop over both protein   */
/* atom can be done in a very simple way:           */
/* for (i=1; i<= (mus.atoms); i++)              */
/*     {mus[1]....}                     */
/* The main structure is mus!                   */
/* i.e. mus==complex; mus1&&mus2==components            */
/*                              */
/* It will give advantage when surfaces, dipols etc. are    */
/* calculated.                          */
/* ATTENTION -  it is assumed, that the nbfix entries in both   */
/* files are the same.                          */
/*                              */
/* The save coordinates and the new coordinates vectors were    */
/* not assigned.                        */
/*                              */
/* CHANGE: not fully tested yet, but some tests have been done  */
/* It seems to work.                        */
/* ------------------------------------------------------------ */

void read_two_MUS(char *filename1, char *filename2,
          MUS_atom *mus ,MUS_atom *mus1, MUS_atom *mus2)
{
FILE *fpin1, *fpin2;
char line[LINELENGTH];
int atom=0, bond=0, angle=0, torsion=0, improp=0, nbfix=0;


fpin1=file_open(filename1,"r");
fpin2=file_open(filename2,"r");

while (fgets(line,LINELENGTH,fpin1)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
       if  (!strncmp(line,"BOND  ",6)) bond++;
       if  (!strncmp(line,"ANGLE ",6)) angle++;
       if  (!strncmp(line,"DIHE  ",6)) torsion++;
       if  (!strncmp(line,"IMPROP",6)) improp++;
       if  (!strncmp(line,"NBFIX ",6))  nbfix++;
      }
(*mus1).number_of_atoms=atom;
(*mus1).number_of_bonds=bond;
(*mus1).number_of_angles=angle;
(*mus1).number_of_torsions=torsion;
(*mus1).number_of_improp=improp;
(*mus1).number_of_nbfix=nbfix;

atom=0; bond=0; angle=0; torsion=0; improp=0; nbfix=0;
while (fgets(line,LINELENGTH,fpin2)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
       if  (!strncmp(line,"BOND  ",6)) bond++;
       if  (!strncmp(line,"ANGLE ",6)) angle++;
       if  (!strncmp(line,"DIHE  ",6)) torsion++;
       if  (!strncmp(line,"IMPROP",6)) improp++;
       if  (!strncmp(line,"NBFIX ",6)) nbfix++;
      }
(*mus2).number_of_atoms=atom;
(*mus2).number_of_bonds=bond;
(*mus2).number_of_angles=angle;
(*mus2).number_of_torsions=torsion;
(*mus2).number_of_improp=improp;
(*mus2).number_of_nbfix=nbfix;

if ((*mus1).number_of_nbfix!=(*mus2).number_of_nbfix)
   {muwarn("The number of NBFIX-Entries is not the same!!!");
    fprintf(STDERR,"%d  -  %d\n",(*mus1).number_of_nbfix,(*mus2).number_of_nbfix);
   }
/* ATTENTION -  it is assumed, that the nbfix entries in both files are the same */
(*mus).number_of_atoms   =(*mus1).number_of_atoms   +(*mus2).number_of_atoms;
(*mus).number_of_bonds   =(*mus1).number_of_bonds   +(*mus2).number_of_bonds;
(*mus).number_of_angles  =(*mus1).number_of_angles  +(*mus2).number_of_angles;
(*mus).number_of_torsions=(*mus1).number_of_torsions+(*mus2).number_of_torsions;
(*mus).number_of_improp  =(*mus1).number_of_improp  +(*mus2).number_of_improp;
(*mus).number_of_nbfix   =(*mus1).number_of_nbfix;

/*fprintf(STDERR,"Number of Atoms: %d\n",(*mus_atom).number_of_atoms);*/

MUS_alloc(mus);

/*--------------------------------------------------------------------*/
/* reset the pointers to the structure elements in mus1 and mus2 to   */
/* the vectors in mus                             */
if ((*mus1).number_of_atoms==0)
    {muerror("number_of_atoms=0 - Abort in read_two_MUS.");
        }
(*mus1).identifier  =(*mus).identifier;
(*mus1).atomid      =(*mus).atomid;
(*mus1).atom_type   =(*mus).atom_type;
(*mus1).residue     =(*mus).residue;

(*mus1).xyz     =(*mus).xyz;

(*mus1).alternate   =(*mus).alternate;
(*mus1).chain       =(*mus).chain;
(*mus1).insert      =(*mus).insert;

(*mus1).hbond       =(*mus).hbond;
(*mus1).ion     =(*mus).ion;

(*mus1).res_no      =(*mus).res_no;
(*mus1).atomno      =(*mus).atomno;
(*mus1).type_no     =(*mus).type_no;
(*mus1).aa_no       =(*mus).aa_no;

(*mus1).charge      =(*mus).charge;
(*mus1).formal_charge   =(*mus).formal_charge;
(*mus1).mass        =(*mus).mass;
(*mus1).radius      =(*mus).radius;
(*mus1).lj_radius       =(*mus).lj_radius;
(*mus1).E_min       =(*mus).E_min;
(*mus1).lj_radius1_4    =(*mus).lj_radius1_4;
(*mus1).E_min1_4    =(*mus).E_min1_4;
(*mus1).surface     =(*mus).surface;
(*mus1).coupling    =(*mus).coupling;
(*mus1).temperature     =(*mus).temperature;
(*mus1).dist_cm         =(*mus).dist_cm;
(*mus1).sigma       =(*mus).sigma;

if ((*mus1).number_of_bonds !=0)
   {(*mus1).bond    =(*mus).bond;
    (*mus1).K_bond  =(*mus).K_bond;
    (*mus1).bond_0  =(*mus).bond_0;
    (*mus1).bond_length =(*mus).bond_length;
   }

if ((*mus1).number_of_angles !=0)
   {(*mus1).angle   =(*mus).angle;
    (*mus1).bond_angle  =(*mus).bond_angle;
    (*mus1).K_angle =(*mus).K_angle;
    (*mus1).angle_0 =(*mus).angle_0;
    (*mus1).K_ub    =(*mus).K_ub;
    (*mus1).dist_ub_0   =(*mus).dist_ub_0;


   }

if ((*mus1).number_of_torsions !=0)
   {(*mus1).torsion =(*mus).torsion;
    (*mus1).torsion_angle=(*mus).torsion_angle;
    (*mus1).K_torsion   =(*mus).K_torsion;
    (*mus1).delta   =(*mus).delta;
    (*mus1).n       =(*mus).n;
   }

if ((*mus1).number_of_improp !=0)
   {(*mus1).improp  =(*mus).improp;
    (*mus1).improp_angle=(*mus).improp_angle;
    (*mus1).K_improp    =(*mus).K_improp;
    (*mus1).improp_0    =(*mus).improp_0;
   }
/* It is assumed. that the NBFIX enetries in both files are the same */
if ((*mus1).number_of_nbfix !=0)
   {(*mus1).fixatom1    =(*mus).fixatom1;
    (*mus1).fixatom2    =(*mus).fixatom2;
    (*mus1).E_minFix    =(*mus).E_minFix;
    (*mus1).radiusFix   =(*mus).radiusFix;
   }
(*mus1).cm=dvector(1,3);            /* center of mass     */
(*mus1).new_cm=dvector(1,3);                /* new center of mass */
(*mus1).dipolvec=dvector(1,3);          /* dipol vector       */

/*--------------------------------------------------------------------*/
/* reset the pointers to the structure elements in mus1 and mus2 to   */
/* the vectors in mus                             */
if ((*mus2).number_of_atoms==0)
    {muerror("number_of_atoms=0 - Abort in read_two_MUS.");
        }
(*mus2).identifier  =(*mus).identifier  +((*mus1).number_of_atoms);
(*mus2).atomid      =(*mus).atomid      +((*mus1).number_of_atoms);
(*mus2).atom_type   =(*mus).atom_type   +((*mus1).number_of_atoms);
(*mus2).residue     =(*mus).residue     +((*mus1).number_of_atoms);

(*mus2).xyz     =(*mus).xyz     +((*mus1).number_of_atoms);

(*mus2).alternate   =(*mus).alternate   +((*mus1).number_of_atoms);
(*mus2).chain       =(*mus).chain       +((*mus1).number_of_atoms);
(*mus2).insert      =(*mus).insert      +((*mus1).number_of_atoms);

(*mus2).hbond       =(*mus).hbond       +((*mus1).number_of_atoms);
(*mus2).ion     =(*mus).ion     +((*mus1).number_of_atoms);

(*mus2).res_no      =(*mus).res_no      +((*mus1).number_of_atoms);
(*mus2).atomno      =(*mus).atomno      +((*mus1).number_of_atoms);
(*mus2).type_no     =(*mus).type_no     +((*mus1).number_of_atoms);
(*mus2).aa_no       =(*mus).aa_no       +((*mus1).number_of_atoms);

(*mus2).charge      =(*mus).charge      +((*mus1).number_of_atoms);
(*mus2).formal_charge   =(*mus).formal_charge   +((*mus1).number_of_atoms);
(*mus2).mass        =(*mus).mass        +((*mus1).number_of_atoms);
(*mus2).radius      =(*mus).radius      +((*mus1).number_of_atoms);
(*mus2).lj_radius       =(*mus).lj_radius   +((*mus1).number_of_atoms);
(*mus2).E_min       =(*mus).E_min       +((*mus1).number_of_atoms);
(*mus2).lj_radius1_4    =(*mus).lj_radius1_4    +((*mus1).number_of_atoms);
(*mus2).E_min1_4    =(*mus).E_min1_4    +((*mus1).number_of_atoms);
(*mus2).surface     =(*mus).surface     +((*mus1).number_of_atoms);
(*mus2).coupling    =(*mus).coupling    +((*mus1).number_of_atoms);
(*mus2).temperature     =(*mus).temperature +((*mus1).number_of_atoms);
(*mus2).dist_cm         =(*mus).dist_cm     +((*mus1).number_of_atoms);
(*mus2).sigma       =(*mus).sigma       +((*mus1).number_of_atoms);

if ((*mus2).number_of_bonds !=0)
   {(*mus2).bond    =(*mus).bond        +((*mus1).number_of_bonds);
    (*mus2).K_bond  =(*mus).K_bond      +((*mus1).number_of_bonds);
    (*mus2).bond_0  =(*mus).bond_0      +((*mus1).number_of_bonds);
    (*mus2).bond_length =(*mus).bond_length +((*mus1).number_of_bonds);
   }

if ((*mus2).number_of_angles !=0)
   {(*mus2).angle   =(*mus).angle       +((*mus1).number_of_angles);
    (*mus2).bond_angle  =(*mus).bond_angle  +((*mus1).number_of_angles);
    (*mus2).K_angle =(*mus).K_angle     +((*mus1).number_of_angles);
    (*mus2).angle_0 =(*mus).angle_0     +((*mus1).number_of_angles);
    (*mus2).K_ub    =(*mus).K_ub        +((*mus1).number_of_angles);
    (*mus2).dist_ub_0   =(*mus).dist_ub_0       +((*mus1).number_of_angles);
   }

if ((*mus2).number_of_torsions !=0)
   {(*mus2).torsion =(*mus).torsion     +((*mus1).number_of_torsions);
    (*mus2).torsion_angle=(*mus).torsion_angle  +((*mus1).number_of_torsions);
    (*mus2).K_torsion   =(*mus).K_torsion   +((*mus1).number_of_torsions);
    (*mus2).delta   =(*mus).delta       +((*mus1).number_of_torsions);
    (*mus2).n       =(*mus).n       +((*mus1).number_of_torsions);
   }

if ((*mus2).number_of_improp !=0)
   {(*mus2).improp  =(*mus).improp      +((*mus1).number_of_improp);
    (*mus2).improp_angle=(*mus).improp_angle    +((*mus1).number_of_improp);
    (*mus2).K_improp    =(*mus).K_improp    +((*mus1).number_of_improp);
    (*mus2).improp_0    =(*mus).improp_0    +((*mus1).number_of_improp);
   }

/* It is assumed. that the NBFIX enetries in both files are the same */
if ((*mus2).number_of_nbfix !=0)
   {(*mus2).fixatom1    =(*mus).fixatom1;
    (*mus2).fixatom2    =(*mus).fixatom2;
    (*mus2).E_minFix    =(*mus).E_minFix;
    (*mus2).radiusFix   =(*mus).radiusFix;
   }

(*mus2).cm=dvector(1,3);        /* center of mass     */
(*mus2).new_cm=dvector(1,3);        /* new center of mass */
(*mus2).dipolvec=dvector(1,3);      /* dipol vector       */

/*--------------------------------------------------------------------*/
/* Reading of the structures                          */
rewind(fpin1);
rewind(fpin2);

atom=0; bond=0; angle=0; torsion=0; improp=0; nbfix=0;
while (fgets(line,LINELENGTH,fpin1)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
      {atom++; read_MUSatom(mus1, atom, line);}
       if (!strncmp(line,"BOND  ",6))
      {bond++; read_MUSbond(mus1, bond, line);}
       if (!strncmp(line,"ANGLE ",6))
      {angle++; read_MUSangle(mus1, angle, line);}
       if (!strncmp(line,"DIHE  ",6))
      {torsion++; read_MUStorsion(mus1, torsion, line);}
       if (!strncmp(line,"IMPROP",6))
      {improp++; read_MUSimprop(mus1, improp, line);}
       if (!strncmp(line,"NBFIX ",6))
      {nbfix++; read_MUSnbfix(mus1, nbfix, line);}
      }
atom=0; bond=0; angle=0; torsion=0; improp=0; nbfix=0;
while (fgets(line,LINELENGTH,fpin2)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
      {atom++; read_MUSatom(mus2, atom, line);}
       if (!strncmp(line,"BOND  ",6))
      {bond++; read_MUSbond(mus2, bond, line);}
       if (!strncmp(line,"ANGLE ",6))
      {angle++; read_MUSangle(mus2, angle, line);}
       if (!strncmp(line,"DIHE  ",6))
      {torsion++; read_MUStorsion(mus2, torsion, line);}
       if (!strncmp(line,"IMPROP",6))
      {improp++; read_MUSimprop(mus2, improp, line);}
       if (!strncmp(line,"NBFIX ",6))
      {nbfix++; read_MUSnbfix(mus2, nbfix, line);}
      }

assign_aminoacid(mus);

/* calculation of the centers of mass */
properties(mus1);
(*mus1).new_cm[1]= (*mus1).cm[1];
(*mus1).new_cm[2]= (*mus1).cm[2];
(*mus1).new_cm[3]= (*mus1).cm[3];

properties(mus2);
(*mus2).new_cm[1]= (*mus2).cm[1];
(*mus2).new_cm[2]= (*mus2).cm[2];
(*mus2).new_cm[3]= (*mus2).cm[3];

fclose(fpin1);
fclose(fpin2);
}
/* ------------------------------------------------------------ */
/* Reads the given line - BOND                  */
/* ------------------------------------------------------------ */
void read_MUSbond(MUS_atom *mus_atom, int i, char *line)
{
sscanf(line+ 6 , "%d %d %lf %lf", &((*mus_atom).bond[i][1]),
                      &((*mus_atom).bond[i][2]),
                      &((*mus_atom).bond_0[i]),
                      &((*mus_atom).K_bond[i]));
}
/* ------------------------------------------------------------ */
/* Reads the given line - ANGLE                 */
/* ------------------------------------------------------------ */
void read_MUSangle(MUS_atom *mus_atom, int i, char *line)
{
sscanf(line+ 6 , "%d %d %d %lf %lf %lf %lf", &((*mus_atom).angle[i][1]),
                         &((*mus_atom).angle[i][2]),
                         &((*mus_atom).angle[i][3]),
                         &((*mus_atom).angle_0[i]),
                         &((*mus_atom).K_angle[i]),
                         &((*mus_atom).dist_ub_0[i]),
                         &((*mus_atom).K_ub[i]));

(*mus_atom).angle_0[i]=TORAD((*mus_atom).angle_0[i]);
}
/* ------------------------------------------------------------ */
/* Reads the given line - DIHE                  */
/* ------------------------------------------------------------ */
void read_MUStorsion(MUS_atom *mus_atom, int i, char *line)
{
sscanf(line+ 6 , "%d %d %d %d %lf %lf %d", &((*mus_atom).torsion[i][1]),
                               &((*mus_atom).torsion[i][2]),
                               &((*mus_atom).torsion[i][3]),
                               &((*mus_atom).torsion[i][4]),
                               &((*mus_atom).delta[i]),
                               &((*mus_atom).K_torsion[i]),
                               &((*mus_atom).n[i]));
(*mus_atom).delta[i]= TORAD((*mus_atom).delta[i]);
}
/* ------------------------------------------------------------ */
/* Reads the given line - IMPROP                */
/* ------------------------------------------------------------ */
void read_MUSimprop(MUS_atom *mus_atom, int i, char *line)
{
sscanf(line+ 6 , "%d %d %d %d %lf %lf", &((*mus_atom).improp[i][1]),
                        &((*mus_atom).improp[i][2]),
                        &((*mus_atom).improp[i][3]),
                        &((*mus_atom).improp[i][4]),
                        &((*mus_atom).improp_0[i]),
                        &((*mus_atom).K_improp[i]));
(*mus_atom).improp_0[i]= TORAD((*mus_atom).improp_0[i]);
}
/* ------------------------------------------------------------ */
/* Reads the given line - NBFIX                 */
/* ------------------------------------------------------------ */

void read_MUSnbfix(MUS_atom *mus_atom, int i, char *line)
{
sscanf(line+ 6 , "%s %s %lf %lf", (*mus_atom).fixatom1[i],
                  (*mus_atom).fixatom2[i],
                  &((*mus_atom).E_minFix[i]),
                  &((*mus_atom).radiusFix[i]));
}

/* ------------------------------------------------------------ */
/* Reads the given line - ATOM or HETATM            */
/* ------------------------------------------------------------ */
void read_MUSatom(MUS_atom *mus_atom, int i, char *line)
{
/*printf("%s",line);*/
sscanf(line   , "%c",  &((*mus_atom).identifier[i][0]));
sscanf(line+1 , "%c",  &((*mus_atom).identifier[i][1]));
sscanf(line+2 , "%c",  &((*mus_atom).identifier[i][2]));
sscanf(line+3 , "%c",  &((*mus_atom).identifier[i][3]));
sscanf(line+4 , "%c",  &((*mus_atom).identifier[i][4]));
sscanf(line+5 , "%c",  &((*mus_atom).identifier[i][5]));
(*mus_atom).identifier[i][6]='\0';
sscanf(line+ 6 , "%5d", &((*mus_atom).atomno[i]));
sscanf(line+ 12, "%c",  &((*mus_atom).atomid[i][0]));
sscanf(line+ 13, "%c",  &((*mus_atom).atomid[i][1]));
sscanf(line+ 14, "%c",  &((*mus_atom).atomid[i][2]));
sscanf(line+ 15, "%c",  &((*mus_atom).atomid[i][3]));
(*mus_atom).atomid[i][4]='\0';
sscanf(line+ 16, "%c",  &((*mus_atom).alternate[i]));
sscanf(line+ 17, "%c",  &((*mus_atom).residue[i][0]));
sscanf(line+ 18, "%c",  &((*mus_atom).residue[i][1]));
sscanf(line+ 19, "%c",  &((*mus_atom).residue[i][2]));
(*mus_atom).residue[i][3]='\0';
sscanf(line+ 21, "%c",  & ((*mus_atom).chain[i]));
sscanf(line+ 22, "%4d", &((*mus_atom).res_no[i]));
sscanf(line+ 26, "%c",  &((*mus_atom).insert[i]));
sscanf(line+ 30, "%8lf",&(((*mus_atom).xyz[i][1])));
sscanf(line+ 38, "%8lf",&(((*mus_atom).xyz[i][2])));
sscanf(line+ 46, "%8lf",&(((*mus_atom).xyz[i][3])));
sscanf(line+ 54, "%11lf",&((*mus_atom).charge[i]));
sscanf(line+ 65, "%8lf",&((*mus_atom).radius[i]));
sscanf(line+ 73, "%8lf",&((*mus_atom).sigma[i]));

sscanf(line+ 81, "%8lf",&((*mus_atom).mass[i]));

sscanf(line+ 89, "%8lf",&((*mus_atom).lj_radius[i]));
sscanf(line+ 97, "%8lf",&((*mus_atom).E_min[i]));
sscanf(line+105, "%8lf",&((*mus_atom).lj_radius1_4[i]));
sscanf(line+113, "%8lf",&((*mus_atom).E_min1_4[i]));
#
sscanf(line+121, "%s %d",(*mus_atom).atom_type[i], &(*mus_atom).type_no[i]);
}
/* ------------------------------------------------------------ */
/* Procedure prints a single atom of a MUS to a specified file  */
/* ------------------------------------------------------------ */
void fprint_MUSatom(FILE *fpout, MUS_atom *mus_atom, int number)
{
fprintf(fpout,
       "%c%c%c%c%c%c%5d %c%c%c%c%c%c%c%c %c%4d%c   %8.3f%8.3f%8.3f%11.6f%8.3f%8.3f%8.4f%8.4f%8.4f%8.4f%8.4f  %5s %4d\n",
          (*mus_atom).identifier[number][0],
          (*mus_atom).identifier[number][1],
          (*mus_atom).identifier[number][2],
          (*mus_atom).identifier[number][3],
          (*mus_atom).identifier[number][4],
          (*mus_atom).identifier[number][5],
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][0],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          (*mus_atom).alternate[number],
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          (*mus_atom).chain[number],
          (*mus_atom).res_no[number],  /*(*mus_atom).aa_no[number],*/
          (*mus_atom).insert[number],
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).charge[number],
          (*mus_atom).radius[number],
      (*mus_atom).sigma[number],

          (*mus_atom).mass[number],

          (*mus_atom).lj_radius[number],
          (*mus_atom).E_min[number],
          (*mus_atom).lj_radius1_4[number],
          (*mus_atom).E_min1_4[number],
      (*mus_atom).atom_type[number],

      (*mus_atom).type_no[number]);
}
/* ------------------------------------------------------------ */
/* Procedure prints a single atom of a PDB to a specified file  */
/* The procedure prints the radius in the occupancy and the     */
/* charge in the B-value column.                */
/* ------------------------------------------------------------ */
void fprint_PDBatom(FILE *fpout, MUS_atom *mus_atom, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %c%4d%c   %8.3f%8.3f%8.3f%6.2f%7.3f\n",
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][0],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          (*mus_atom).alternate[number],
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          (*mus_atom).chain[number],
          (*mus_atom).res_no[number],
          (*mus_atom).insert[number],
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).radius[number],
          (*mus_atom).charge[number]);
}
/* ------------------------------------------------------------ */
/* Procedure prints a single atom of a MUS to a specified file  */
/* in pqr-Format (MEAD)                     */
/* ------------------------------------------------------------ */
void fprint_PQRatom(FILE *fpout, MUS_atom *mus_atom, int number)
{
  /*fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %c%4d%c   %8.3lf%8.3lf%8.3lf%10.5lf%10.5lf\n",
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][0],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          ' ',
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          ' ',
          (*mus_atom).res_no[number],
          ' ',
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).charge[number],
          (*mus_atom).radius[number]);*/
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c   %8.3f%8.3f%8.3f%11.6f%8.3f\n",
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][0],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          ' ',
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          (*mus_atom).res_no[number],
          ' ',
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).charge[number],
          (*mus_atom).radius[number]);
}
/* ------------------------------------------------------------ */
void fprint_PQRatom2(FILE *fpout, MUS_atom *mus_atom, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c   %8.3f%8.3f%8.3f%11.6f%8.3f\n",
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          (*mus_atom).atomid[number][0],
          ' ',
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          (*mus_atom).res_no[number],
          ' ',
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).charge[number],
          (*mus_atom).radius[number]);
}

/* ------------------------------------------------------------ */
/* Procedure prints a single atom of a MUS to a specified file  */
/* in pqr-Format in Beroza-style(PEP)               */
/* ------------------------------------------------------------ */
void fprint_PQRatom_beroza(FILE *fpout, MUS_atom *mus_atom, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %c%4d%c   %8.3f%8.3f%8.3f%10.5f%10.5f\n",
          (*mus_atom).atomno[number],
          (*mus_atom).atomid[number][0],
          (*mus_atom).atomid[number][1],
          (*mus_atom).atomid[number][2],
          (*mus_atom).atomid[number][3],
          ' ',
          (*mus_atom).residue[number][0],
          (*mus_atom).residue[number][1],
          (*mus_atom).residue[number][2],
          ' ',
          (*mus_atom).res_no[number],
          ' ',
          (*mus_atom).xyz[number][1],
          (*mus_atom).xyz[number][2],
          (*mus_atom).xyz[number][3],
          (*mus_atom).charge[number],
          (*mus_atom).radius[number]);
}

/* ------------------------------------------------------------ */
/* Procedure prints a single bond of a MUS to a specified file  */
/* ------------------------------------------------------------ */
void fprint_MUSbond(FILE *fpout, MUS_atom *mus_atom, int i)
{
fprintf(fpout,"BOND     %5d %5d  %8.3f  %8.3f \n",
            (*mus_atom).bond[i][1],
            (*mus_atom).bond[i][2],
            (*mus_atom).bond_0[i],
            (*mus_atom).K_bond[i]);

}

/* ------------------------------------------------------------ */
/* Procedure prints a single angle of a MUS to a specified file */
/* ------------------------------------------------------------ */
void fprint_MUSangle(FILE *fpout, MUS_atom *mus_atom, int i)
{
fprintf(fpout,"ANGLE    %5d %5d %5d  %8.3f %8.3f %8.3f %8.3f\n",
            (*mus_atom).angle[i][1],
            (*mus_atom).angle[i][2],
            (*mus_atom).angle[i][3],
            TOGRD((*mus_atom).angle_0[i]),
            (*mus_atom).K_angle[i],
            (*mus_atom).dist_ub_0[i],
            (*mus_atom).K_ub[i]);
}

/* ------------------------------------------------------------- */
/* Procedure prints a single improp of a MUS to a specified file */
/* ------------------------------------------------------------- */
void fprint_MUSimprop(FILE *fpout, MUS_atom *mus_atom, int i)
{
fprintf(fpout,"IMPROP   %5d %5d %5d %5d %8.3f %8.3f\n",
            (*mus_atom).improp[i][1],
            (*mus_atom).improp[i][2],
            (*mus_atom).improp[i][3],
            (*mus_atom).improp[i][4],
            TOGRD((*mus_atom).improp_0[i]),
            (*mus_atom).K_improp[i]);
}

/* -------------------------------------------------------------- */
/* Procedure prints a single torsion of a MUS to a specified file */
/* -------------------------------------------------------------- */
void fprint_MUStorsion(FILE *fpout, MUS_atom *mus_atom, int i)
{
fprintf(fpout,"DIHE     %5d %5d %5d %5d %8.3f %8.3f %3d\n",
            (*mus_atom).torsion[i][1],
            (*mus_atom).torsion[i][2],
            (*mus_atom).torsion[i][3],
            (*mus_atom).torsion[i][4],
            TOGRD((*mus_atom).delta[i]),
            (*mus_atom).K_torsion[i],
            (*mus_atom).n[i]);
}


/* ------------------------------------------------------------ */
/* Reads the given line - NBFIX                 */
/* ------------------------------------------------------------ */

void fprint_MUSnbfix(FILE *fpout, MUS_atom *mus_atom, int i)
{
fprintf(fpout,"NBFIX    %-5s %-5s %11.6f %11.6f\n",
                        (*mus_atom).fixatom1[i],
                        (*mus_atom).fixatom2[i],
                            (*mus_atom).E_minFix[i],
                        (*mus_atom).radiusFix[i]);
}

/* ------------------------------------------------------------ */
/* Procedure prints a Dummy atom                */
/* ------------------------------------------------------------ */
void fprint_DUMMYatom(FILE *fpout, int number, char *atomid,
              double *xyz)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c DUM          %8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f  NN\n",
          number,
          atomid[0],
          atomid[1],
          atomid[2],
          atomid[3],
          xyz[1],
          xyz[2],
          xyz[3],
          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
}

/* ------------------------------------------------------------ */
/* Procedure prints a Dummy atom                */
/* The value in bval is writen to the temperature factor    */
/* so that is can be used to color atoms            */
/* ------------------------------------------------------------ */
void fprint_DUMMYatom2(FILE *fpout, int number, char *atomid,
               double *xyz, double bval)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c DUM          %8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f  NN\n",
          number,
          atomid[0],
          atomid[1],
          atomid[2],
          atomid[3],
          xyz[1],
          xyz[2],
          xyz[3],
          0.0,
      bval,
      0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
}

/* ------------------------------------------------------------ */
/* Procedure prints a Dummy atom                */
/* The value in bval is writen to the temperature factor    */
/* so that is can be used to color atoms            */
/* ------------------------------------------------------------ */
void fprint_DUMMYatomPQR(FILE *fpout, int number, char *atomid,
               double *xyz, double bval)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c DUM %5d    %8.3f%8.3f%8.3f%8.3f%8.3f\n",
          number, atomid[0], atomid[1], atomid[2], atomid[3],
          number, xyz[1], xyz[2], xyz[3], 0.0, bval);
}

/* ------------------------------------------------------------ */
/* Procedure prints a MUS to a specified file           */
/* ------------------------------------------------------------ */
void fprint_MUS(FILE *fpout, MUS_atom *mus_atom)
{
int i;

for (i=1; i<=(*mus_atom).number_of_atoms; i++)
    {fprint_MUSatom(fpout, mus_atom, i);
    }
fprintf(fpout,"TER\n");
for (i=1; i<=(*mus_atom).number_of_bonds; i++)
    {fprint_MUSbond(fpout, mus_atom, i);
    }
for (i=1; i<=(*mus_atom).number_of_angles; i++)
    {fprint_MUSangle(fpout, mus_atom, i);
    }
for (i=1; i<=(*mus_atom).number_of_torsions; i++)
    {fprint_MUStorsion(fpout, mus_atom, i);
    }
for (i=1; i<=(*mus_atom).number_of_improp; i++)
    {fprint_MUSimprop(fpout, mus_atom, i);
    }
for (i=1; i<=(*mus_atom).number_of_nbfix; i++)
    {fprint_MUSnbfix(fpout, mus_atom, i);
    }
fprintf(fpout,"END\n");
}

/* ------------------------------------------------------------ */
/* Procedure prints a MUS to a specified file in PQR-Format */
/* ------------------------------------------------------------ */
void fprint_PQR(FILE *fpout, MUS_atom *mus_atom)
{
int i;

for (i=1; i<=(*mus_atom).number_of_atoms; i++)
    {if ((*mus_atom).atomid[i][0]==' ')
        {fprint_PQRatom(fpout, mus_atom, i);
    }
     else {fprint_PQRatom2(fpout, mus_atom, i);
          }
    }
fprintf(fpout,"TER\n");
}

/* ------------------------------------------------------------ */
/* Procedure prints a MUS to a specified file in PDB-Format */
/* ------------------------------------------------------------ */
void fprint_PDB(FILE *fpout, MUS_atom *mus_atom)
{
int i;

for (i=1; i<=(*mus_atom).number_of_atoms; i++)
    {fprint_PDBatom(fpout, mus_atom, i);
    }
fprintf(fpout,"TER\n");
}

/* ------------------------------------------------------------ */
/* procedure calculates the properties of molecules and assigns */
/* the rest of the structure.                   */
/* ATTENTION                            */
/* diffusion coefficients are at present not assigned.      */
/* ------------------------------------------------------------ */
void properties(MUS_atom *mus)
{
(*mus).total_mass  =center_of_mass((*mus).xyz, (*mus).mass, (*mus).number_of_atoms,
                   (*mus).cm);
(*mus).total_charge=charge((*mus).charge, (*mus).number_of_atoms);
(*mus).dipol       =dipol((*mus).xyz, (*mus).charge, (*mus).number_of_atoms,
              (*mus).cm, (*mus).dipolvec);
(*mus).area    = 0.0; /*surface(mus, 600, 1.4); CHANGE */
(*mus).volume      =volume((*mus).xyz, (*mus).radius, (*mus).number_of_atoms);
(*mus).gyrationradius = gyrationradius((*mus).xyz, (*mus).number_of_atoms, (*mus).cm);
(*mus).max_cm_dist = max_dist((*mus).xyz, (*mus).number_of_atoms, (*mus).cm, (*mus).dist_cm);
}


/* ------------------------------------------------------------ */
/* procedure prints properties                  */
/* ATTENTION                            */
/* diffusion coefficients are inactive and so are at present    */
/* not printed.                         */
/* ------------------------------------------------------------ */
void fprint_properties(FILE *fpout, MUS_atom *mus)
{
fprintf(fpout,"Number of atoms:     %d\n",(*mus).number_of_atoms);
fprintf(fpout,"Number of bonds:     %d\n",(*mus).number_of_bonds);
fprintf(fpout,"Number of angles:        %d\n",(*mus).number_of_angles);
fprintf(fpout,"Number of torsions:      %d\n",(*mus).number_of_torsions);
fprintf(fpout,"Number of improper torsions: %d\n",(*mus).number_of_improp);

fprintf(fpout,"Mass:        %8.3f\n",(*mus).total_mass);
fprintf(fpout,"Center of Mass:  %8.3f  %8.3f  %8.3f\n",(*mus).cm[1],(*mus).cm[2],(*mus).cm[3]);
fprintf(fpout,"Radius of Gyration:      %8.3f\n",(*mus).gyrationradius);
fprintf(fpout,"Surface:     %8.3f\n",(*mus).area);
fprintf(fpout,"Volume:      %8.3f\n",(*mus).volume);
fprintf(fpout,"Charge:      %8.3f\n",(*mus).total_charge);
fprintf(fpout,"Dipol:       %8.3f  = %8.3f Debye\n",(*mus).dipol,(*mus).dipol*4.802);
fprintf(fpout,"Dipolvector-KM:  %8.3f  %8.3f  %8.3f\n",
    (*mus).dipolvec[1],(*mus).dipolvec[2],(*mus).dipolvec[3]);
fprintf(fpout,"Maximal Distance from the center of mass:   %8.3f\n",(*mus).max_cm_dist);

/* Diffusion coefficients */
/*
fprintf(fpout,"Translational Diffusion Coefficient: %8.3lf  \n",(*mus).diff_trans);
fprintf(fpout,"Rotational Diffusion Coefficient:    %8.3lf  \n",(*mus).diff_rot);
fprintf(fpout,"Diffusion Matrix:\n");
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[1][1],(*mus).diff_matrix[1][2],(*mus).diff_matrix[1][3],
    (*mus).diff_matrix[1][4],(*mus).diff_matrix[1][5],(*mus).diff_matrix[1][6]);
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[2][1],(*mus).diff_matrix[2][2],(*mus).diff_matrix[2][3],
    (*mus).diff_matrix[2][4],(*mus).diff_matrix[2][5],(*mus).diff_matrix[2][6]);
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[3][1],(*mus).diff_matrix[3][2],(*mus).diff_matrix[3][3],
    (*mus).diff_matrix[3][4],(*mus).diff_matrix[3][5],(*mus).diff_matrix[3][6]);
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[4][1],(*mus).diff_matrix[4][2],(*mus).diff_matrix[4][3],
    (*mus).diff_matrix[4][4],(*mus).diff_matrix[4][5],(*mus).diff_matrix[4][6]);
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[5][1],(*mus).diff_matrix[5][2],(*mus).diff_matrix[5][3],
    (*mus).diff_matrix[5][4],(*mus).diff_matrix[5][5],(*mus).diff_matrix[5][6]);
fprintf(fpout,"%8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf\n",
    (*mus).diff_matrix[6][1],(*mus).diff_matrix[6][2],(*mus).diff_matrix[6][3],
    (*mus).diff_matrix[6][4],(*mus).diff_matrix[6][5],(*mus).diff_matrix[6][6]);
*/
}

/* ------------------------------------------------------------ */
/* Procedure shifts the center of mass to a given point     */

void shift_cm(MUS_atom *protein, double *point)
{
double trans[4];
int i;

trans[1]= point[1]-(*protein).cm[1];
trans[2]= point[2]-(*protein).cm[2];
trans[3]= point[3]-(*protein).cm[3];

for (i=1; i<= (*protein).number_of_atoms;i++)
    {(*protein).xyz[i][1] = (*protein).xyz[i][1]+trans[1];
     (*protein).xyz[i][2] = (*protein).xyz[i][2]+trans[2];
     (*protein).xyz[i][3] = (*protein).xyz[i][3]+trans[3];
    }
/*
fprintf(STDERR," Vorher: %8.3lf%8.3lf%8.3lf\n",(*protein).cm[1],(*protein).cm[2],(*protein).cm[3]);
properties(protein);
fprintf(STDERR,"Nachher: %8.3lf%8.3lf%8.3lf\n",(*protein).cm[1],(*protein).cm[2],(*protein).cm[3]);
*/

}
/* ------------------------------------------------------------ */
/* procedure copies all xyz-coordinates to save_xyz     */
/* and cm to save_cm                        */

void save_initial_coordinates(MUS_atom *protein)
{
int i;

(*protein).save_cm[1]=(*protein).cm[1];
(*protein).save_cm[2]=(*protein).cm[2];
(*protein).save_cm[3]=(*protein).cm[3];

for (i=1; i<=(*protein).number_of_atoms; i++)
    {(*protein).save_xyz[i][1]=(*protein).xyz[i][1];
     (*protein).save_xyz[i][2]=(*protein).xyz[i][2];
     (*protein).save_xyz[i][3]=(*protein).xyz[i][3];
    }
}

/* ------------------------------------------------------------ */
void RestoreCoor(MUS_atom *protein)
{
int i;

if ((*protein).save_xyz==NULL)
    muerror("save_xyz not allocated. Error in RestoreCoor().");

(*protein).cm[1]=(*protein).save_cm[1];
(*protein).cm[2]=(*protein).save_cm[2];
(*protein).cm[3]=(*protein).save_cm[3];

(*protein).new_cm[1]=(*protein).save_cm[1];
(*protein).new_cm[2]=(*protein).save_cm[2];
(*protein).new_cm[3]=(*protein).save_cm[3];

for (i=1; i<=(*protein).number_of_atoms; i++)
    {(*protein).xyz[i][1]=(*protein).save_xyz[i][1];
     (*protein).xyz[i][2]=(*protein).save_xyz[i][2];
     (*protein).xyz[i][3]=(*protein).save_xyz[i][3];
    }


}
/* ---------------------------------------------------- */
/* procedure prooves which atoms lie on the surface */
/* of the protein                   */
/* ATTENTION - Surface must be calculated prior calling */
/* tihs function.                   */

void on_surf(MUS_atom *protein)
{
int i, j=0;

/* looking for the number of surface atoms */
for (i = 1; i <= (*protein).number_of_atoms; i++)
    if ((*protein).surface[i]!=0.0) j++;

(*protein).surf_atom=j;

/* allocation */
(*protein).surf=ivector(1,(*protein).surf_atom);

/* assigning */
j=0;
for (i = 1; i <= (*protein).number_of_atoms; i++)
    if ((*protein).surface[i]!=0.0)
        {j++;
         (*protein).surf[j]=i;
         /*fprint_MUSatom(stdout,protein,(*protein).surf[j]);*/
        }
}
/*..............................................................*/
/* Calcultate Internal coordinates              */
/* ATTENTION! The procedure assumes, that the atomnoumber and   */
/* the counting number are identical. This might not generally  */
/* be true, especially in the case of complex structures, where */
/* two proteins are the member of one stucture (read_two_MUS()) */
/* In these cases the complex structure should be given to the  */
/* procedure rather than the single proteins.           */
/* A looking for the correct atoms is to time consuming.    */
void calculate_ic(MUS_atom *mus)
{
int i;
double *p1, *p2, *p3, *p4;

/* Bond length */
for (i=1; i<=(*mus).number_of_bonds; i++)
    {p1=(*mus).xyz[(*mus).bond[i][1]];
     p2=(*mus).xyz[(*mus).bond[i][2]];
     (*mus).bond_length[i]=Distance(p1,p2);
    }

/* Bond angles */
for (i=1; i<=(*mus).number_of_angles; i++)
    {p1=(*mus).xyz[(*mus).angle[i][1]];
     p2=(*mus).xyz[(*mus).angle[i][2]];
     p3=(*mus).xyz[(*mus).angle[i][3]];
     (*mus).bond_angle[i]=bond_angle(p1,p2,p3);
    }

/* Torsion angels */
for (i=1; i<=(*mus).number_of_torsions; i++)
    {p1=(*mus).xyz[(*mus).torsion[i][1]];
     p2=(*mus).xyz[(*mus).torsion[i][2]];
     p3=(*mus).xyz[(*mus).torsion[i][3]];
     p4=(*mus).xyz[(*mus).torsion[i][4]];
     (*mus).torsion_angle[i]=dihedral(p1,p2,p3,p4);
    }

/* Improper Torsion angels */
for (i=1; i<=(*mus).number_of_improp; i++)
    {p1=(*mus).xyz[(*mus).improp[i][1]];
     p2=(*mus).xyz[(*mus).improp[i][2]];
     p3=(*mus).xyz[(*mus).improp[i][3]];
     p4=(*mus).xyz[(*mus).improp[i][4]];
     (*mus).improp_angle[i]=dihedral(p1,p2,p3,p4);
    }

}
/*..............................................................*/
/* Procedure recounts atom in a MUS_record. This is done in a   */
/* proper way so that also the ANGLE-, BOND- etc. Records are   */
/* updated                          */
void recount_MUS(MUS_atom *mus,MUS_atom *mus1,MUS_atom *mus2)
{
int i, j, k;
int *help;
/* allocating a help integer vector */
help=ivector(1,(*mus).number_of_atoms);

/* saveing the old numbering and recount the atomno-vector*/
for (i=1; i<=(*mus).number_of_atoms; i++)
    {help[i]=(*mus).atomno[i];
     (*mus).atomno[i]=i;
    }

/* Now dating up the other records for the first protein in the complex */
for (j=1; j<=(*mus1).number_of_atoms; j++)
   {/* Bond records */
    for (i=1; i<=(*mus1).number_of_bonds; i++)
        {if ((*mus1).bond[i][1]==help[j]) (*mus1).bond[i][1]=(*mus1).atomno[j];
     if ((*mus1).bond[i][2]==help[j]) (*mus1).bond[i][2]=(*mus1).atomno[j];
    }
    /* Angle records */
    for (i=1; i<=(*mus1).number_of_angles; i++)
        {if ((*mus1).angle[i][1]==help[j]) (*mus1).angle[i][1]=(*mus1).atomno[j];
     if ((*mus1).angle[i][2]==help[j]) (*mus1).angle[i][2]=(*mus1).atomno[j];
     if ((*mus1).angle[i][3]==help[j]) (*mus1).angle[i][3]=(*mus1).atomno[j];
    }
    /* Torsion records */
    for (i=1; i<=(*mus1).number_of_torsions; i++)
        {if ((*mus1).torsion[i][1]==help[j]) (*mus1).torsion[i][1]=(*mus1).atomno[j];
     if ((*mus1).torsion[i][2]==help[j]) (*mus1).torsion[i][2]=(*mus1).atomno[j];
     if ((*mus1).torsion[i][3]==help[j]) (*mus1).torsion[i][3]=(*mus1).atomno[j];
     if ((*mus1).torsion[i][4]==help[j]) (*mus1).torsion[i][4]=(*mus1).atomno[j];
    }
    /* Improper Torsion records */
    for (i=1; i<=(*mus1).number_of_improp; i++)
        {if ((*mus1).improp[i][1]==help[j]) (*mus1).improp[i][1]=(*mus1).atomno[j];
     if ((*mus1).improp[i][2]==help[j]) (*mus1).improp[i][2]=(*mus1).atomno[j];
     if ((*mus1).improp[i][3]==help[j]) (*mus1).improp[i][3]=(*mus1).atomno[j];
     if ((*mus1).improp[i][4]==help[j]) (*mus1).improp[i][4]=(*mus1).atomno[j];
    }
   }

/* Now dating up the other records for the second protein in the complex */
for (j=(*mus1).number_of_atoms; j<=((*mus1).number_of_atoms+(*mus2).number_of_atoms); j++)
   {/* Bond records */
    k=j-(*mus1).number_of_atoms;
    for (i=1; i<=(*mus2).number_of_bonds; i++)
        {if ((*mus2).bond[i][1]==help[j]) (*mus2).bond[i][1]=(*mus2).atomno[k];
     if ((*mus2).bond[i][2]==help[j]) (*mus2).bond[i][2]=(*mus2).atomno[k];
    }
    /* Angle records */
    for (i=1; i<=(*mus2).number_of_angles; i++)
        {if ((*mus2).angle[i][1]==help[j]) (*mus2).angle[i][1]=(*mus2).atomno[k];
     if ((*mus2).angle[i][2]==help[j]) (*mus2).angle[i][2]=(*mus2).atomno[k];
     if ((*mus2).angle[i][3]==help[j]) (*mus2).angle[i][3]=(*mus2).atomno[k];
    }
    /* Torsion records */
    for (i=1; i<=(*mus2).number_of_torsions; i++)
        {if ((*mus2).torsion[i][1]==help[j]) (*mus2).torsion[i][1]=(*mus2).atomno[k];
     if ((*mus2).torsion[i][2]==help[j]) (*mus2).torsion[i][2]=(*mus2).atomno[k];
     if ((*mus2).torsion[i][3]==help[j]) (*mus2).torsion[i][3]=(*mus2).atomno[k];
     if ((*mus2).torsion[i][4]==help[j]) (*mus2).torsion[i][4]=(*mus2).atomno[k];
    }
    /* Improper Torsion records */
    for (i=1; i<=(*mus2).number_of_improp; i++)
        {if ((*mus2).improp[i][1]==help[j]) (*mus2).improp[i][1]=(*mus2).atomno[k];
     if ((*mus2).improp[i][2]==help[j]) (*mus2).improp[i][2]=(*mus2).atomno[k];
     if ((*mus2).improp[i][3]==help[j]) (*mus2).improp[i][3]=(*mus2).atomno[k];
     if ((*mus2).improp[i][4]==help[j]) (*mus2).improp[i][4]=(*mus2).atomno[k];
    }
   }
}
/*..............................................................*/
/* This function sorts the atoms in the BOND-, ANGLE- etc.  */
/* Records in a way that the left integer is smaller than the   */
/* right integer. This is nessessary for the triangluar     */
/* interaction and topology etc. matrices.          */

void sorting_MUS(MUS_atom *mus)
{
int help,i;
/* Bonds */
for (i=1; i<=(*mus).number_of_bonds; i++)
    {/* sorting partner in a way that bond[i][1]<bond[i][2] */
     if ((*mus).bond[i][1]>(*mus).bond[i][2])
    {help=(*mus).bond[i][1];
     (*mus).bond[i][1]=(*mus).bond[i][2];
     (*mus).bond[i][2]=help;
    }
    }

/* Angles */
for (i=1; i<=(*mus).number_of_angles; i++)
    {/* sorting partner in a way that angle[i][1]<angle[i][3] */
     if ((*mus).angle[i][1]>(*mus).angle[i][3])
    {help=(*mus).angle[i][1];
     (*mus).angle[i][1]=(*mus).angle[i][3];
     (*mus).angle[i][3]=help;
    }
    }

/* Torsions */
for (i=1; i<=(*mus).number_of_torsions; i++)
    {/* sorting partner in a way that torsion[i][1]<torsion[i][4] */
     if ((*mus).torsion[i][1]>(*mus).torsion[i][4])
    {help=(*mus).torsion[i][1];
     (*mus).torsion[i][1]=(*mus).torsion[i][4];
     (*mus).torsion[i][4]=help;
     help=(*mus).torsion[i][2];
     (*mus).torsion[i][2]=(*mus).torsion[i][3];
     (*mus).torsion[i][3]=help;
    }
    }

}
/*..............................................................*/
/* searches for double entries in the BOND, ANGLE, TORSION  */
/* and IMPR-records. If double entries occur, an error message  */
/* is issued                            */
/*                              */
void search_double_entries(MUS_atom *mus)
{
int i,j;

/* look in the BOND records */
for (i=1; i<=(*mus).number_of_bonds;i++)
    {for (j=1; j<i;j++)
     {if (i!=j)
         {if ( (((*mus).bond[i][1]==(*mus).bond[j][1])
          &&((*mus).bond[i][2]==(*mus).bond[j][2]))
         ||(((*mus).bond[i][2]==(*mus).bond[j][1])
          &&((*mus).bond[i][1]==(*mus).bond[j][2])))
        {fprintf(stderr,"WARNING:\n");
            fprintf(stderr,"Double entries in the MUS File (search_double_entries())\n");
         fprintf(stderr,"i=%d, j=%d \n",i,j);
         fprint_MUSbond(stderr,mus,i);
         fprint_MUSbond(stderr,mus,j);
        }
          }
      }
    }
/* look in the ANGLE records */
for (i=1; i<=(*mus).number_of_angles;i++)
    {for (j=1; j<i;j++)
     {if (i!=j)
         {if ( (((*mus).angle[i][1]==(*mus).angle[j][1])
          &&((*mus).angle[i][2]==(*mus).angle[j][2])
          &&((*mus).angle[i][3]==(*mus).angle[j][3]))
         ||(((*mus).angle[i][3]==(*mus).angle[j][1])
          &&((*mus).angle[i][2]==(*mus).angle[j][2])
          &&((*mus).angle[i][1]==(*mus).angle[j][3])))
        {fprintf(stderr,"WARNING:\n");
            fprintf(stderr,"Double entries in the MUS File (search_double_entries())\n");
         fprintf(stderr,"i=%d, j=%d \n",i,j);
         fprint_MUSangle(stderr,mus,i);
         fprint_MUSangle(stderr,mus,j);
        }
          }
      }
    }
/* look in the TORSION records */
for (i=1; i<=(*mus).number_of_torsions;i++)
    {for (j=1; j<i;j++)
     {if (i!=j)
         {if ( (((*mus).torsion[i][1]==(*mus).torsion[j][1])
          &&((*mus).torsion[i][2]==(*mus).torsion[j][2])
          &&((*mus).torsion[i][3]==(*mus).torsion[j][3])
          &&((*mus).torsion[i][4]==(*mus).torsion[j][4]))
         ||(((*mus).torsion[i][4]==(*mus).torsion[j][1])
          &&((*mus).torsion[i][3]==(*mus).torsion[j][2])
          &&((*mus).torsion[i][2]==(*mus).torsion[j][3])
          &&((*mus).torsion[i][1]==(*mus).torsion[j][4])))
        {if (((*mus).delta[i]    ==(*mus).delta[j])
           &&((*mus).K_torsion[i]==(*mus).K_torsion[j])
           &&((*mus).n[i]        ==(*mus).n[j]))
           {fprintf(stderr,"WARNING:\n");
            fprintf(stderr,"Double entries in the MUS File (search_double_entries())\n");
            fprintf(stderr,"i=%d, j=%d \n",i,j);
            fprint_MUStorsion(stderr,mus,i);
            fprint_MUStorsion(stderr,mus,j);
           }
        }
          }
      }
    }
/* look in the IMPROP records */
/* CHANGE muss noch mal richtig programmiert werden */
/* mehr Variationen!!                   */
for (i=1; i<=(*mus).number_of_improp;i++)
    {for (j=1; j<i;j++)
     {if (i!=j)
         {if ( (((*mus).improp[i][1]==(*mus).improp[j][1])
          &&((*mus).improp[i][2]==(*mus).improp[j][2])
          &&((*mus).improp[i][3]==(*mus).improp[j][3])
          &&((*mus).improp[i][4]==(*mus).improp[j][4]))
         ||(((*mus).improp[i][4]==(*mus).improp[j][1])
          &&((*mus).improp[i][3]==(*mus).improp[j][2])
          &&((*mus).improp[i][2]==(*mus).improp[j][3])
          &&((*mus).improp[i][1]==(*mus).improp[j][4])))
        {fprintf(stderr,"WARNING:\n");
            fprintf(stderr,"Double entries in the MUS File (search_double_entries())\n");
         fprintf(stderr,"i=%d, j=%d \n",i,j);
         fprint_MUSimprop(stderr,mus,i);
         fprint_MUSimprop(stderr,mus,j);
        }
          }
      }
    }
}

/* .................................................... */
/* procedure creates PDB-styled CONECT-Record from MUS- */
/* BOND-entries                     */
/*                          */

void create_conect(int **conect, MUS_atom *mus)
{
int i,j,n;

/* initializing  */
for (i=1; i<=(*mus).number_of_atoms; i++)
    {conect[i][1]=i;
     for (j=2; j<=MAX_CONECT; j++)
    {conect[i][j]=0;
    }
    }

/* assigning - loop throu all bond record - very uneffective !! */
for (i=1; i<=(*mus).number_of_atoms; i++)
    {n=1;
     if (n>MAX_CONECT)
        {fprintf(STDERR,"%d\n",n);
     muerror("Array too small in fprint_conect().");
        }
     for (j=1; j<=(*mus).number_of_bonds; j++)
     {if ((*mus).bond[j][1]==i)
         {conect[i][n+1]=(*mus).bond[j][2];
          n++;
         }
      if ((*mus).bond[j][2]==i)
         {conect[i][n+1]=(*mus).bond[j][1];
          n++;
         }
          if (n>MAX_CONECT)
             {fprintf(STDERR,"%d\n",n);
          muerror("Array too small in fprint_conect().");
             }
     }
    }
}
/* .................................................... */
/* prints the incorrect PDB-styled CONECT-records to    */
/* a file                       */
/*                          */

void fprint_conect1(FILE *fpout, int **conect, int len)
{
int i,j;

for (i=1; i<=len; i++)
    {fprintf(fpout,"CONECT");
     for (j=1; j<=MAX_CONECT;j++)
       {fprintf(fpout," %4d",conect[i][j]);
        if (0==conect[i][j+1])
           break;
       }
     fprintf(fpout,"\n");
    }

}

/* .................................................... */
/* prints the correct PDB-styled CONECT-records to a    */
/* file                         */
/*                          */

void fprint_conect2(FILE *fpout, int **conect, int len)
{
int i,j;

for (i=1; i<=len; i++)
    {fprintf(fpout,"CONECT");
     for (j=1; j<=MAX_CONECT;j++)
       {fprintf(fpout,"%5d",conect[i][j]);
        if (0==conect[i][j+1])
           break;
       }
     fprintf(fpout,"\n");
    }

}

/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION                        */
/* Care must be taken, that two neightboured amino- */
/* acids does not have the same id number!!!!       */
/*                          */
void recount_aminoacids3(MUS_atom *protein)
{
int i,aa, ihelp, save;
char save_chain, save_insert,save_name[5];

/* Assignment */
aa=1;
aa=(*protein).res_no[1];
i=1;
save=(*protein).res_no[i];
save_insert=(*protein).insert[i];
save_chain=(*protein).chain[i];
strcpy(save_name,(*protein).residue[i]);
(*protein).res_no[i]=aa;
(*protein).atomno[i]=i;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {(*protein).atomno[i]=i;
     if (  ((*protein).res_no[i] != save)
     ||(0!=strncmp((*protein).residue[i],save_name,3))
     ||(save_insert!=(*protein).insert[i])
     ||(save_chain!=(*protein).chain[i]))
    {if (save_chain!=(*protein).chain[i])
        {ihelp=1000-aa;
         while (ihelp<0)
           {ihelp=ihelp+1000;
           }
           aa=aa+ihelp;
        }
     aa++;
         save_insert=(*protein).insert[i];
     save=(*protein).res_no[i];
         strcpy(save_name,(*protein).residue[i]);
         save_chain=(*protein).chain[i];
     (*protein).res_no[i]=aa;
    }
     else (*protein).res_no[i]=aa;
    }
}
/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION                        */
/* Care must be taken, that two neightboured amino- */
/* acids does not have the same id number!!!!       */
/*                          */
void recount_aminoacids2(MUS_atom *protein)
{
int i,aa, ihelp, save;
char save_chain, save_insert,save_name[5];

/* Assignment */
ihelp=0;
aa=(*protein).res_no[1];
i=1;
save=(*protein).res_no[i];
save_insert=(*protein).insert[i];
save_chain=(*protein).chain[i];
strcpy(save_name,(*protein).residue[i]);
(*protein).res_no[i]=aa;
(*protein).atomno[i]=i;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {(*protein).atomno[i]=i;
     if (  ((*protein).res_no[i] != save)
     ||(0!=strncmp((*protein).residue[i],save_name,3))
     ||(save_insert!=(*protein).insert[i])
     ||(save_chain!=(*protein).chain[i]))
    {if (save_chain!=(*protein).chain[i])
        {ihelp++;
             aa=(*protein).res_no[i]+ihelp*1000;
        }
     else {aa=(*protein).res_no[i]+ihelp*1000;
          }
     save_insert=(*protein).insert[i];
     save=(*protein).res_no[i];
         strcpy(save_name,(*protein).residue[i]);
         save_chain=(*protein).chain[i];
     (*protein).res_no[i]=aa;
    }
     else (*protein).res_no[i]=aa;
    }
}
/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION                        */
/* Care must be taken, that two neightboured amino- */
/* acids does not have the same id number!!!!       */
/*                          */
void recount_aminoacids2a(MUS_atom *protein)
{
int i,aa, ihelp, save;
char save_chain, save_insert,save_name[5];

/* Assignment */
ihelp=1;
aa=(*protein).res_no[1];
i=1;
save=(*protein).res_no[i];
save_insert=(*protein).insert[i];
save_chain=(*protein).chain[i];
strcpy(save_name,(*protein).residue[i]);
(*protein).res_no[i]=aa;
(*protein).atomno[i]=i;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {(*protein).atomno[i]=i;
     if (  ((*protein).res_no[i] != save)
     ||(0!=strncmp((*protein).residue[i],save_name,3))
     ||(save_insert!=(*protein).insert[i])
     ||(save_chain!=(*protein).chain[i]))
    {if (save_chain!=(*protein).chain[i])
        {aa=(*protein).res_no[i]+ihelp*1000-1;
         ihelp++;
            }

     aa++;
         save_insert=(*protein).insert[i];
     save=(*protein).res_no[i];
         strcpy(save_name,(*protein).residue[i]);
         save_chain=(*protein).chain[i];
     (*protein).res_no[i]=aa;
    }
     else (*protein).res_no[i]=aa;
    }
}
/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION                        */
/* Care must be taken, that two neightboured amino- */
/* acids does not have the same id number!!!!       */
/*                          */
void recount_aminoacids(MUS_atom *protein)
{
int i,aa, save;

/* Assignment */
aa=1;
i=1;
save=(*protein).res_no[i];
(*protein).res_no[i]=aa;
(*protein).atomno[i]=i;
for (i=2; i<=(*protein).number_of_atoms; i++)
    {(*protein).atomno[i]=i;
     if ((*protein).res_no[i] != save)
    {aa++;
     save=(*protein).res_no[i];
     (*protein).res_no[i]=aa;
    }
     else (*protein).res_no[i]=aa;
    }
}
/* ------------------------------------------------------------ */
/* Procedure reads in a PDB                     */
/* A lot of things done here are not nessesarry in all      */
/* application however, it is not very time consuming       */
/* and this function is only called a few times during a run    */
/* ------------------------------------------------------------ */

void read_PDB(char *filename, MUS_atom *pdb)
{
FILE *fpin;
char line[LINELENGTH];
int atom=0;


fpin=file_open(filename,"r");
if (fpin==NULL) return;

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
      }
(*pdb).number_of_atoms=atom;
(*pdb).number_of_bonds=0;
(*pdb).number_of_angles=0;
(*pdb).number_of_torsions=0;
(*pdb).number_of_improp=0;
(*pdb).number_of_nbfix=0;

/*fprintf(STDERR,"Number of Atoms: %d\n",(*pdb).number_of_atoms);
fprintf(STDERR,"Number of Torsions: %d\n",(*pdb).number_of_torsions);*/

MUS_alloc(pdb);
rewind(fpin);

atom=0;

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
      {atom++; read_PDBatom(pdb, atom, line);}
      }
/* same additional allocation */
(*pdb).save_xyz=dmatrix(1,(*pdb).number_of_atoms,1,3);
(*pdb).new_xyz=dmatrix(1,(*pdb).number_of_atoms,1,3);

assign_aminoacid(pdb);
fclose(fpin);
}

/* ------------------------------------------------------------ */
/* Reads the given line - ATOM or HETATM            */
/* in Grasp and Delphi style                    */
/* ------------------------------------------------------------ */
void read_PDBatom(MUS_atom *pdb, int i, char *line)
{
/*printf("%s",line);*/
sscanf(line   , "%c",  &((*pdb).identifier[i][0]));
sscanf(line+1 , "%c",  &((*pdb).identifier[i][1]));
sscanf(line+2 , "%c",  &((*pdb).identifier[i][2]));
sscanf(line+3 , "%c",  &((*pdb).identifier[i][3]));
sscanf(line+4 , "%c",  &((*pdb).identifier[i][4]));
sscanf(line+5 , "%c",  &((*pdb).identifier[i][5]));
(*pdb).identifier[i][6]='\0';
sscanf(line+ 6 , "%5d", &((*pdb).atomno[i]));
sscanf(line+ 12, "%c",  &((*pdb).atomid[i][0]));
sscanf(line+ 13, "%c",  &((*pdb).atomid[i][1]));
sscanf(line+ 14, "%c",  &((*pdb).atomid[i][2]));
sscanf(line+ 15, "%c",  &((*pdb).atomid[i][3]));
(*pdb).atomid[i][4]='\0';
sscanf(line+ 16, "%c",  &((*pdb).alternate[i]));
sscanf(line+ 17, "%c",  &((*pdb).residue[i][0]));
sscanf(line+ 18, "%c",  &((*pdb).residue[i][1]));
sscanf(line+ 19, "%c",  &((*pdb).residue[i][2]));
(*pdb).residue[i][3]='\0';
sscanf(line+ 21, "%c",  & ((*pdb).chain[i]));
sscanf(line+ 22, "%4d", &((*pdb).res_no[i]));
sscanf(line+ 26, "%c",  &((*pdb).insert[i]));
sscanf(line+ 30, "%8lf",&(((*pdb).xyz[i][1])));
sscanf(line+ 38, "%8lf",&(((*pdb).xyz[i][2])));
sscanf(line+ 46, "%8lf",&(((*pdb).xyz[i][3])));
/*
sscanf(line+ 54, "%6lf",&((*pdb).radius[i]));
sscanf(line+ 60, "%7lf",&((*pdb).charge[i]));
*/
sscanf(line+ 54, "%lf %lf",&((*pdb).charge[i]),&((*pdb).radius[i]));
}

/* ------------------------------------------------------------ */
/* Procedure reads in a POR                     */
/* A lot of things done here are not nessesarry in all      */
/* application however, it is not very time consuming       */
/* and this function is only called a few times during a run    */
/* ------------------------------------------------------------ */

void read_PQR(char *filename, MUS_atom *pdb)
{
FILE *fpin;
char line[LINELENGTH];
int atom=0;


fpin=file_open(filename,"r");

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
      }
(*pdb).number_of_atoms=atom;
(*pdb).number_of_bonds=0;
(*pdb).number_of_angles=0;
(*pdb).number_of_torsions=0;
(*pdb).number_of_improp=0;
(*pdb).number_of_nbfix=0;

/*fprintf(STDERR,"Number of Atoms: %d\n",(*pdb).number_of_atoms);
fprintf(STDERR,"Number of Torsions: %d\n",(*pdb).number_of_torsions);*/

MUS_alloc(pdb);
rewind(fpin);

atom=0;

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
      {atom++;
           read_PQRatom(pdb, atom, line);
           /*fprint_PQRatom(stdout, pdb, atom);TEST;*/
          }
      }
/* same additional allocation */
(*pdb).save_xyz=dmatrix(1,(*pdb).number_of_atoms,1,3);
(*pdb).new_xyz=dmatrix(1,(*pdb).number_of_atoms,1,3);

assign_aminoacid(pdb);
fclose(fpin);
}

/* ------------------------------------------------------------ */
/* Reads the given line - ATOM or HETATM            */
/* in MEAD style                        */
/* .............                                                */
/* Change 6 April                                               */
/* MEAD does not real read the PDB-Format and a pattern matching*/
/* is better for conversions                                    */
/* ------------------------------------------------------------ */
void read_PQRatom(MUS_atom *pdb, int i, char *line)
{
char help[6];/*printf("%s",line);*/
/*
sscanf(line   , "%c",  &((*pdb).identifier[i][0]));
sscanf(line+1 , "%c",  &((*pdb).identifier[i][1]));
sscanf(line+2 , "%c",  &((*pdb).identifier[i][2]));
sscanf(line+3 , "%c",  &((*pdb).identifier[i][3]));
sscanf(line+4 , "%c",  &((*pdb).identifier[i][4]));
sscanf(line+5 , "%c",  &((*pdb).identifier[i][5]));
(*pdb).identifier[i][6]='\0';
sscanf(line+ 6 , "%5d", &((*pdb).atomno[i]));
sscanf(line+ 12, "%c",  &((*pdb).atomid[i][0]));
sscanf(line+ 13, "%c",  &((*pdb).atomid[i][1]));
sscanf(line+ 14, "%c",  &((*pdb).atomid[i][2]));
sscanf(line+ 15, "%c",  &((*pdb).atomid[i][3]));
(*pdb).atomid[i][4]='\0';
sscanf(line+ 16, "%c",  &((*pdb).alternate[i]));
sscanf(line+ 17, "%c",  &((*pdb).residue[i][0]));
sscanf(line+ 18, "%c",  &((*pdb).residue[i][1]));
sscanf(line+ 19, "%c",  &((*pdb).residue[i][2]));
(*pdb).residue[i][3]='\0';
sscanf(line+ 21, "%c",  & ((*pdb).chain[i]));
sscanf(line+ 22, "%4d", &((*pdb).res_no[i]));
sscanf(line+ 26, "%c",  &((*pdb).insert[i]));
sscanf(line+ 30, "%8lf",&(((*pdb).xyz[i][1])));
sscanf(line+ 38, "%8lf",&(((*pdb).xyz[i][2])));
sscanf(line+ 46, "%8lf",&(((*pdb).xyz[i][3])));
sscanf(line+ 56, "%10lf",&((*pdb).charge[i]));
sscanf(line+ 66, "%10lf",&((*pdb).radius[i]));
*/
strcpy((*pdb).atomid[i],"");
sscanf(line,
       "%s %d %s %s %d %lf %lf %lf %lf %lf\n",
       (*pdb).identifier[i], &(*pdb).atomno[i],
       (*pdb).atomid[i], (*pdb).residue[i],  &(*pdb).res_no[i],
       &(*pdb).xyz[i][1], &(*pdb).xyz[i][2], &(*pdb).xyz[i][3],
       &(*pdb).charge[i], &(*pdb).radius[i]);
//       printf("%s\n", (*pdb).atomid[i]);

help[0]=(*pdb).atomid[i][0]; help[1]=(*pdb).atomid[i][1];
help[2]=(*pdb).atomid[i][2]; help[3]=(*pdb).atomid[i][3];
if (strlen((*pdb).atomid[i])==4)
  {(*pdb).atomid[i][0]=help[0]; (*pdb).atomid[i][1]=help[1];
   (*pdb).atomid[i][2]=help[2]; (*pdb).atomid[i][3]=help[3];
  }

if (strlen((*pdb).atomid[i])==3)
  {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
   (*pdb).atomid[i][2]=help[1]; (*pdb).atomid[i][3]=help[2];
  }
if (strlen((*pdb).atomid[i])==2)
  {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
   (*pdb).atomid[i][2]=help[1]; (*pdb).atomid[i][3]=' ';
  }
if (strlen((*pdb).atomid[i])==1)
  {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
   (*pdb).atomid[i][2]=' ';     (*pdb).atomid[i][3]=' ';
  }
if (strlen((*pdb).residue[i])==2)
  {(*pdb).residue[i][2]=' ';
  }
}

/*......................................................*/
/* copies a mus atom                    */
void copy_mus_atom(MUS_atom *copy, int icopy, MUS_atom *ori, int iori)
{
strncpy((*copy).identifier[icopy],(*ori).identifier[iori],7);
strncpy((*copy).atomid[icopy],    (*ori).atomid[iori],5);
strncpy((*copy).residue[icopy],   (*ori).residue[iori],4);

(*copy).xyz[icopy][1]=(*ori).xyz[iori][1];
(*copy).xyz[icopy][2]=(*ori).xyz[iori][2];
(*copy).xyz[icopy][3]=(*ori).xyz[iori][3];

(*copy).alternate[icopy]=(*ori).alternate[iori];
(*copy).chain[icopy]    =(*ori).chain[iori];
(*copy).insert[icopy]   =(*ori).insert[iori];
(*copy).res_no[icopy]   =(*ori).res_no[iori];
(*copy).atomno[icopy]   =(*ori).atomno[iori];
(*copy).aa_no[icopy]    =(*ori).aa_no[iori];

(*copy).charge[icopy]   =(*ori).charge[iori];
(*copy).mass[icopy] =(*ori).mass[iori];
(*copy).radius[icopy]   =(*ori).radius[iori];

(*copy).lj_radius[icopy]    =(*ori).lj_radius[iori];
(*copy).E_min[icopy]    =(*ori).E_min[iori];
(*copy).surface[icopy]  =(*ori).surface[iori];
(*copy).lj_radius1_4[icopy]=(*ori).lj_radius1_4[iori];
(*copy).E_min1_4[icopy] =(*ori).E_min1_4[iori];
(*copy).dist_cm[icopy]  =(*ori).dist_cm[iori];
(*copy).sigma[icopy]    =(*ori).sigma[iori];
(*copy).hbond[icopy]    =(*ori).hbond[iori];
(*copy).ion[icopy]  =(*ori).ion[iori];
(*copy).type_no[icopy]  =(*ori).type_no[iori];

strcpy((*copy).atom_type[icopy],(*ori).atom_type[iori]);
}
