/* Matthias Ullmann, 02.10.1995				*/
/* Matthias Ullmann Structure - MUS			*/
/*......................................................*/

#ifndef MUS_ATOM
#define MUS_ATOM



/* declaration of structures */
/* The aminoacid i is assigned by the two numbers	*/
/* 1. Its starting point - aa[i][1]			*/
/* 2. Its length - aa[i][2]				*/
/* Internally, all Emin are stored as sqrt(-Emin)	*/
/* The conversion is done in the IO-Rountines		*/
typedef struct {int  number_of_atoms; /* N */

		char **identifier; 	/*[N][7]*/
	     	int  *atomno;  
	     	char **atomid; 	   	/*[N][5]*/
	     	char **atom_type;       /*[N][5]*/
		int  *type_no;
	     	char *alternate;	    
	     	char **residue;	   	/*[N][4]*/	    
	     	char *chain;	  
	     	int  *res_no;   
	     	char *insert;	  
	     	double **xyz;   	/*[N][4]*/		  

	     	double *save_cm;   	/*[4]*/		  
	     	double **save_xyz;   	/*[N][4]*/		  
	     	double **new_xyz;   	/*[N][4]*/		  

	     	double *mass;
	     	double *surface;	/* surface accessiblility		*/
		double *coupling;  	/* electronic coupling			*/
		double *temperature;	/* temperature factor			*/
		double *dist_cm;	/* distance from the center of mass 	*/
		double *sigma;		/* sigma for solvent accessibility  	*/
                double *radius;         /* radius for PBE and surface           */

		int *surf;		/* atom numbers of the surface atoms	*/
		int surf_atom;		/* number of surface atoms	    	*/

		int  aminoacids;	/* number of aminoacids			*/
		int  **aa;		/* to find aminoacids - 		*/
					/* start atom number aminacid and length*/
		int  *aa_no;		/* aa number [1..atoms]			*/

		char *hbond;
		char *ion;

		/* energy parameters */
		/* non-bonded interactions */
                double *lj_radius;      /* Lennard-Jones-Radius */
		double *E_min;	  	/* VdW E_min */			
		double *lj_radius1_4;   /* Lennard-Jones-Radius */
		double *E_min1_4;	/* VdW E_min */			

	     	double *charge;	   
		double *formal_charge;
		
		/* NB Fix Terms */
		int    number_of_nbfix;
		char   **fixatom1;
		char   **fixatom2;
		double *E_minFix;
		double *radiusFix;

		/* Bond energy term */
		int    number_of_bonds;
		double *bond_length;
		int    **bond;		/* [N][2] */
		double *K_bond;
		double *bond_0;

		/* Angle energy term */
		int    number_of_angles;
		double *bond_angle;
		int    **angle;		/* [N][3] */
		double *K_angle;
		double *angle_0;
		double *K_ub;
		double *dist_ub_0;

		/* Torsion angle */
		int    number_of_torsions;
		double *torsion_angle;
		int    **torsion;		/* [N][4] */
		int    *n;
		double *K_torsion;
		double *delta;

		/* Improper Torsions */
		int    number_of_improp;
		double *improp_angle;
		int    **improp;		/* [N][4] */
		double *K_improp;
		double *improp_0;		

		/* properties */
		double diff_trans;
		double diff_rot;
		double **diff_matrix;
		double total_mass;
		double total_charge;
		double *cm;		/* center of mass 		*/
		double *new_cm;		/* new center of mass 		*/
		double *dipolvec;	/* dipolvector 			*/
		double dipol;		/* dipolmoment			*/
		double volume;		/* 				*/
		double area;		/* 				*/
		double gyrationradius;
		double max_cm_dist;	/* maximal distance from the 	*/
					/* center of mass 		*/
	
		/* CONECT records - only added if needed */
		int    **conect;
	       } MUS_atom;


#define MAX_CONECT 8
/*************************/
/* Allocation */
void MUS_alloc(MUS_atom *mus_atom);
void MUS_free(MUS_atom *mus_atom);


/* IO */
void assign_aminoacid(MUS_atom *protein);

void read_two_MUS(char *filename1, char *filename2, 
		  MUS_atom *mus ,MUS_atom *mus1, MUS_atom *mus2);
void read_MUS(char *filename, MUS_atom *mus_atom);
void read_MUS2(char *filename, MUS_atom *mus_atom);
void read_MUSatom(MUS_atom *mus_atom, int i, char *line);
void read_MUSbond(MUS_atom *mus_atom, int i, char *line);
void read_MUSangle(MUS_atom *mus_atom, int i, char *line);
void read_MUStorsion(MUS_atom *mus_atom, int i, char *line);
void read_MUSimprop(MUS_atom *mus_atom, int i, char *line);
void read_MUSnbfix(MUS_atom *mus_atom, int i, char *line);


void fprint_aminoacid(FILE *fpout, MUS_atom *protein, int aa_no);
void fprint_DUMMYatom(FILE *fpout, int number, char *atomid, double *xyz);
void fprint_DUMMYatom2(FILE *fpout, int number, char *atomid, double *xyz, double bval);
void fprint_DUMMYatomPQR(FILE *fpout, int number, char *atomid,
                         double *xyz, double bval);

void fprint_MUS(FILE *fpout, MUS_atom *mus_atom);
void fprint_MUSatom(FILE *fpout, MUS_atom *mus_atom, int number);
void fprint_MUSbond(FILE *fpout, MUS_atom *mus_atom, int i);
void fprint_MUSangle(FILE *fpout, MUS_atom *mus_atom, int i);
void fprint_MUSimprop(FILE *fpout, MUS_atom *mus_atom, int i);
void fprint_MUStorsion(FILE *fpout, MUS_atom *mus_atom, int i);
void fprint_MUSnbfix(FILE *fpout, MUS_atom *mus_atom, int i);

void read_PQR(char *filename, MUS_atom *pdb);
void read_PQRatom(MUS_atom *pdb, int i, char *line);
void fprint_PQR(FILE *fpout, MUS_atom *mus_atom);
void fprint_PQRatom(FILE *fpout, MUS_atom *mus_atom, int number);

void read_PDB(char *filename, MUS_atom *pdb);
void read_PDBatom(MUS_atom *pdb, int i, char *line);
void fprint_PDB(FILE *fpout, MUS_atom *mus_atom);
void fprint_PDBatom(FILE *fpout, MUS_atom *mus_atom, int number);
/* Utilities */
void properties(MUS_atom *mus);
void fprint_properties(FILE *fpout, MUS_atom *mus);
void save_initial_coordinates(MUS_atom *protein);
void shift_cm(MUS_atom *protein, double *point);
void on_surf(MUS_atom *protein);
void RestoreCoor(MUS_atom *protein);

void calculate_ic(MUS_atom *mus);
void recount_MUS(MUS_atom *mus,MUS_atom *mus1,MUS_atom *mus2);
void sorting_MUS(MUS_atom *mus);
void search_double_entries(MUS_atom *mus);


void create_conect(int **conect, MUS_atom *mus);
void fprint_conect1(FILE *fpout, int **conect, int len);
void fprint_conect2(FILE *fpout, int **conect, int len);
void recount_aminoacids(MUS_atom *protein);
void recount_aminoacids2(MUS_atom *protein);
void copy_mus_atom(MUS_atom *copy, int icopy, MUS_atom *ori, int iori);
#endif 
