/* Matthias Ullmann, Berlin 15.01.1995                  */
/*                                  */
/* routines for surface and volume calculation              */
/*                                  */
/* method by F. Eisenhaber:                     */
/* F.Eisenhaber, P.Lijnzaad, P.Argos, M.Scharf              */
/*    "The Double Cubic Lattice Method: Efficient Approaches to     */
/*    Numerical Integration of Surface Area and Volume and to Dot   */
/*    Surface Contouring of Molecular Assemblies"           */
/*    Journal of Computational Chemistry (1995) v.16, N3, pp.273-284    */
/*                                  */
/* .................................................................... */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "mathematik.h"
#include "constants.h"
#include "nrutil.h"
#include "util_c.h"
#include "grid.h"
#include "surface.h"

/************************************************************************/
/* creation of a regular dot distribution - tesselation         */
/*                                  */
/* The code is mainly taken from Frank Eisenhaber - NSC         */
/* It has been adopted to my array alloaction starting by 1 and ending  */
/* at n rather than starting at 0 and ending at n-1.            */
/* The resulting code is a little bit complex and hard to understand.   */
/* DON'T TOUCH IT, if you have the possibility to avoid it.     */
/* Since these function are called only once in a run, it is not a CPU- */
/* time critical part.                          */
/* .................................................................... */
/* function returns the typ of tesselation                  */
/*                                  */
int tessellation_type(int densit)
{
int i1=1, i2=1;

while (10*i1*i1+2 < densit) i1++;
while (30*i2*i2+2 < densit) i2++;
if (10*i1*i1-2 < 30*i2*i2-2)
     {return UNSP_ICO_ARC;}   /* i.e. class I  tessellation */
else {return UNSP_ICO_DOD;}   /* i.e. class II tessellation */
}

/* .................................................................... */
/* routine for dot distributions on the surface of the unit sphere      */
/*                                  */

void icosaeder_vertices(double **xus)
{
double rg, rh;

rh = sqrt(1.-2.*cos(TORAD(72.)))/(1.-cos(TORAD(72.)));
rg = cos(TORAD(72.))/(1.-cos(TORAD(72.)));
/* icosaeder vertices */
xus[ 1][1] = 0.;                  xus[ 1][2] = 0.;                  xus[ 1][3] = 1.;
xus[ 2][1] = rh*cos(TORAD(72.));  xus[ 2][2] = rh*sin(TORAD(72.));  xus[ 2][3] = rg;
xus[ 3][1] = rh*cos(TORAD(144.)); xus[ 3][2] = rh*sin(TORAD(144.)); xus[ 3][3] = rg;
xus[ 4][1] = rh*cos(TORAD(216.)); xus[ 4][2] = rh*sin(TORAD(216.)); xus[ 4][3] = rg;
xus[ 5][1] = rh*cos(TORAD(288.)); xus[ 5][2] = rh*sin(TORAD(288.)); xus[ 5][3] = rg;
xus[ 6][1] = rh;                  xus[ 6][2] = 0;                   xus[ 6][3] = rg;
xus[ 7][1] = rh*cos(TORAD(36.));  xus[ 7][2] = rh*sin(TORAD(36.));  xus[ 7][3] = -rg;
xus[ 8][1] = rh*cos(TORAD(108.)); xus[ 8][2] = rh*sin(TORAD(108.)); xus[ 8][3] = -rg;
xus[ 9][1] = -rh;                 xus[ 9][2] = 0;                   xus[ 9][3] = -rg;
xus[10][1] = rh*cos(TORAD(252.)); xus[10][2] = rh*sin(TORAD(252.)); xus[10][3] = -rg;
xus[11][1] = rh*cos(TORAD(324.)); xus[11][2] = rh*sin(TORAD(324.)); xus[11][3] = -rg;
xus[12][1] = 0.;                  xus[12][2] = 0.;                  xus[12][3] = -1.;
}

/* .................................................................... */
/* construct the class I tessellation                   */
/*                                  */
/* densit...required dots per unit sphere                   */
/* dot distribution on a unit sphere based on an icosaeder          */
/* great circle average refining of icosahedral face                */
/*                                  */
void ico_dot_arc(int densit, Surf *surf)
{
double x1[4], x2[4], x3[4], xij[4], xji[4], xki[4], xjk[4], xik[4], xkj[4];
double a, rh;
double x,y,z,d ;
int i, j, k, tess, tn, tl, tl2;

/* Code */
/* calculate the number of dots per unit sphere */
a = sqrt((((double) densit)-2.)/10.);
tess = (int) ceil(a);
(*surf).n_dot = 10*tess*tess+2;
if ((*surf).n_dot < densit)
    {fprintf(stderr,"WARNING: ico_dot_arc: error in formula for tessellation level (%d->%d, %d)\n", tess, (*surf).n_dot, densit);
    }

/* allocation of dot-matrix */
(*surf).xus =dmatrix(1,(*surf).n_dot, 1,3);
(*surf).help=dmatrix(1,(*surf).n_dot, 1,3);

/* calculate the first 12 dots */
icosaeder_vertices((*surf).xus);

if (tess > 1)
   {tn = 12;
    rh = sqrt(1.-2.*cos(TORAD(72.)))/(1.-cos(TORAD(72.)));
    a = rh*rh*2.*(1.-cos(TORAD(72.)));
    /* calculate tessellation of icosaeder edges */
    for (i=1; i<=11; i++)
    {for (j=i+1; j<=12; j++)
        {x = (*surf).xus[i][1]-(*surf).xus[j][1];
         y = (*surf).xus[i][2]-(*surf).xus[j][2];
         z = (*surf).xus[i][3]-(*surf).xus[j][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;
             for (tl=1; tl<tess; tl++)
         {if (tn >= (*surf).n_dot)
                     {fprintf(stderr,"ERROR: ico_dot: tn exceeds dimension of xus\n");}
                  tn++;
                  divarc((*surf).xus[i], (*surf).xus[j], tl, tess, (*surf).xus[tn]);
                 }
            }
        }
    /* calculate tessellation of icosaeder faces */
    for (i=1; i<=10; i++)
    {for (j=i+1; j<=11; j++)
        {x = (*surf).xus[i][1]-(*surf).xus[j][1];
             y = (*surf).xus[i][2]-(*surf).xus[j][2];
         z = (*surf).xus[i][3]-(*surf).xus[j][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;

         for (k=j+1; k<=12; k++)
        {x = (*surf).xus[i][1]-(*surf).xus[k][1];
             y = (*surf).xus[i][2]-(*surf).xus[k][2];
         z = (*surf).xus[i][3]-(*surf).xus[k][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;
             x = (*surf).xus[j][1]-(*surf).xus[k][1];
             y = (*surf).xus[j][2]-(*surf).xus[k][2];
         z = (*surf).xus[j][3]-(*surf).xus[k][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;

             for (tl=1; tl<tess-1; tl++)
             {divarc((*surf).xus[j], (*surf).xus[i], tl, tess, xji);
                      divarc((*surf).xus[k], (*surf).xus[i], tl, tess, xki);
              for (tl2=1; tl2<tess-tl; tl2++)
              {divarc((*surf).xus[i], (*surf).xus[j], tl2, tess, xij);
                       divarc((*surf).xus[k], (*surf).xus[j], tl2, tess, xkj);
                       divarc((*surf).xus[i], (*surf).xus[k], tess-tl-tl2, tess, xik);
                       divarc((*surf).xus[j], (*surf).xus[k], tess-tl-tl2, tess, xjk);
                       if (tn >= (*surf).n_dot) fprintf(stderr,"ERROR: tn exceeds dimension of xus");
                       divarc(xki, xji, tl2, tess-tl,  x1);
                       divarc(xkj, xij, tl,  tess-tl2, x2);
                       divarc(xjk, xik, tl,  tl+tl2,   x3);
                       x = x1[1]+x2[1]+x3[1];
               y = x1[2]+x2[2]+x3[2];
               z = x1[3]+x2[3]+x3[3];
                       d = sqrt(x*x+y*y+z*z);
                       tn++;
                       (*surf).xus[tn][1] = x/d;
                       (*surf).xus[tn][2] = y/d;
                       (*surf).xus[tn][3] = z/d;
                      } /* cycle tl2 */
                     }      /* cycle tl  */
             }      /* cycle k   */
             }          /* cycle j   */
        }           /* cycle i   */
    if ((*surf).n_dot != tn) fprintf(stderr,"ERROR: ico_dot: n_dot(%d) and tn(%d) differ", (*surf).n_dot, tn);
   }

        /* end of if (tess > 1) */
}

/* .................................................................... */
/* this procedure catches errors in asin.               */
/*                                  */
#define ASIN safe_asin
double safe_asin(double f)
{
if ( (fabs(f) < 1.00) ) return( asin(f) );
if ( (fabs(f) - 1.00)  <= DP_TOL )
    fprintf(stderr,"ERROR: ASIN : invalid argument %f", f);
return(M_PI_2);
}
/* .................................................................... */
/*                                  */
void divarc(double *x1, double *x2, int div1, int div2, double *xr)
{
double xd, yd, zd, dd, d1, d2, s, x, y, z;
double phi, sphi, cphi;

xd = x1[2]*x2[3]-x2[2]*x1[3];
yd = x1[3]*x2[1]-x2[3]*x1[1];
zd = x1[1]*x2[2]-x2[1]*x1[2];
dd = sqrt(xd*xd + yd*yd + zd*zd);

if (dd < DP_TOL) fprintf(stderr,"ERROR: divarc: rotation axis of length %f\n", dd);

d1 = x1[1]*x1[1] + x1[2]*x1[2] + x1[3]*x1[3];
if (d1 < 0.5) fprintf(stderr,"ERROR: divarc: vector 1 of sq.length %f\n", d1);
d2 = x2[1]*x2[1] + x2[2]*x2[2] + x2[3]*x2[3];
if (d2 < 0.5) fprintf(stderr,"ERROR: divarc: vector 2 of sq.length %f\n", d2);

phi = ASIN(dd/sqrt(d1*d2));
phi = phi*((double)div1)/((double)div2);
sphi = sin(phi);
cphi = cos(phi);
s  = (x1[1]*xd + x1[2]*yd + x1[3]*zd)/dd;

x = xd*s*(1.-cphi)/dd + x1[1] * cphi + (yd*x1[3]-x1[2]*zd)*sphi/dd;
y = yd*s*(1.-cphi)/dd + x1[2] * cphi + (zd*x1[1]-x1[3]*xd)*sphi/dd;
z = zd*s*(1.-cphi)/dd + x1[3] * cphi + (xd*x1[2]-x1[1]*yd)*sphi/dd;
dd = sqrt(x*x + y*y + z*z);

xr[1] = x/dd; xr[2] = y/dd; xr[3] = z/dd;
}

/* .................................................................... */
/* tessellation class II                        */
/*                                  */
/* densit...required dots per unit sphere                   */
/* dot distribution on a unit sphere based on an icosaeder          */
/* great circle average refining of icosahedral face                */
/*                                  */
void ico_dot_dod(int densit, Surf *surf)
{
int i, j, k, tl, tl2, tn, tess, j1, j2;
double rh;
double a, d, x, y, z, ai_d, adod;
double xki[4], xji[4], x1[4], xkj[4], xij[4], x2[4], xjk[4], xik[4], x3[4];

/* Code */
/* determine the number of dots */
a = sqrt((((double) densit)-2.)/30.);
tess = MAXIMUM((int) ceil(a), 1);
(*surf).n_dot = 30*tess*tess+2;
if ((*surf).n_dot < densit)
    {fprintf(stderr,"ERROR: ico_dot_dot: error in formula for tessellation level (%d->%d, %d)\n", tess, (*surf).n_dot, densit);
    }

/* allocation of a matrix */
(*surf).xus =dmatrix(1,(*surf).n_dot, 1,3);
(*surf).help=dmatrix(1,(*surf).n_dot, 1,3);

/* calculate the first 12 dots */
icosaeder_vertices((*surf).xus);

tn=13;

/* square of the edge of an icosaeder */
rh = sqrt(1.-2.*cos(TORAD(72.)))/(1.-cos(TORAD(72.)));
a = rh*rh*2.*(1.-cos(TORAD(72.)));
/* dodecaeder vertices */
for(i=1; i<=10; i++)
   {for(j=i+1; j<=11; j++)
       {x = (*surf).xus[i][1]-(*surf).xus[j][1];
        y = (*surf).xus[i][2]-(*surf).xus[j][2];
    z = (*surf).xus[i][3]-(*surf).xus[j][3];
        d = x*x+y*y+z*z;
        if (fabs(a-d) > DP_TOL) continue;
        for (k=j+1; k<=12; k++)
        {x = (*surf).xus[i][1]-(*surf).xus[k][1];
             y = (*surf).xus[i][2]-(*surf).xus[k][2];
         z = (*surf).xus[i][3]-(*surf).xus[k][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;
             x = (*surf).xus[j][1]-(*surf).xus[k][1];
             y = (*surf).xus[j][2]-(*surf).xus[k][2];
         z = (*surf).xus[j][3]-(*surf).xus[k][3];
             d = x*x+y*y+z*z;
             if (fabs(a-d) > DP_TOL) continue;
             x = (*surf).xus[i][1]+(*surf).xus[j][1]+(*surf).xus[k][1];
             y = (*surf).xus[i][2]+(*surf).xus[j][2]+(*surf).xus[k][2];
             z = (*surf).xus[i][3]+(*surf).xus[j][3]+(*surf).xus[k][3];
             d = sqrt(x*x+y*y+z*z);
             (*surf).xus[tn][1]=x/d;
         (*surf).xus[tn][2]=y/d;
         (*surf).xus[tn][3]=z/d;
             tn++;
            }
        }
   }

if (tess > 1)
   {tn = 33;
    /* square of the edge of an dodecaeder */
    adod = 4.*(cos(TORAD(108.))-cos(TORAD(120.)))/(1.-cos(TORAD(120.)));
    /* square of the distance of two adjacent vertices of ico- and dodecaeder */
    ai_d = 2.*(1.-sqrt(1.-a/3.));
    /* calculate tessellation of mixed edges */
    for(i=1; i<=31; i++)
       {j1 = 13; j2 = 33; a = ai_d;
        if (i>=13) { j1=i+1; a = adod; }
        for(j=j1; j<=j2; j++)
       {x = (*surf).xus[i][1]-(*surf).xus[j][1];
            y = (*surf).xus[i][2]-(*surf).xus[j][2];
        z = (*surf).xus[i][3]-(*surf).xus[j][3];
            d = x*x+y*y+z*z;
            if (fabs(a-d) > DP_TOL) continue;
            for(tl=1; tl<tess; tl++)
           {if(tn > (*surf).n_dot)
          {muerror("ico_dot: tn exceeds dimension of xus");}
                divarc((*surf).xus[i], (*surf).xus[j],tl, tess, (*surf).xus[tn]);
                tn++;
               }
           }
       }
    /* calculate tessellation of pentakisdodecahedron faces */
    for(i=1; i<=12; i++)
       {for(j=13; j<=31; j++)
       {x = (*surf).xus[i][1]-(*surf).xus[j][1];
            y = (*surf).xus[i][2]-(*surf).xus[j][2];
        z = (*surf).xus[i][3]-(*surf).xus[j][3];
            d = x*x+y*y+z*z;
            if (fabs(ai_d-d) > DP_TOL) continue;
            for (k=j+1; k<=32; k++)
        {x = (*surf).xus[i][1]-(*surf).xus[k][1];
                 y = (*surf).xus[i][2]-(*surf).xus[k][2];
         z = (*surf).xus[i][3]-(*surf).xus[k][3];
                 d = x*x+y*y+z*z;
                 if (fabs(ai_d-d) > DP_TOL) continue;
                 x = (*surf).xus[j][1]-(*surf).xus[k][1];
                 y = (*surf).xus[j][2]-(*surf).xus[k][2];
         z = (*surf).xus[j][3]-(*surf).xus[k][3];
                 d = x*x+y*y+z*z;
                 if (fabs(adod-d) > DP_TOL) continue;
                 for (tl=1; tl<tess-1; tl++)
             {divarc((*surf).xus[j], (*surf).xus[i], tl, tess, xji);
                      divarc((*surf).xus[k], (*surf).xus[i], tl, tess, xki);
                      for (tl2=1; tl2<tess-tl; tl2++)
              {divarc((*surf).xus[i], (*surf).xus[j], tl2, tess, xij);
                           divarc((*surf).xus[k], (*surf).xus[j], tl2, tess, xkj);
                       divarc((*surf).xus[i], (*surf).xus[k], tess-tl-tl2, tess, xik);
                       divarc((*surf).xus[j], (*surf).xus[k], tess-tl-tl2, tess, xjk);
                       if (tn > (*surf).n_dot)
                  {muerror("ico_dot: tn exceeds dimension of xus");
                          }
                       divarc(xki, xji, tl2, tess-tl, x1);
                       divarc(xkj, xij, tl, tess-tl2, x2);
                       divarc(xjk, xik, tl, tl+tl2,   x3);
                       x = x1[1]+x2[1]+x3[1];
               y = x1[2]+x2[2]+x3[2];
               z = x1[3]+x2[3]+x3[3];
                       d = sqrt(x*x+y*y+z*z);
                       (*surf).xus[tn][1] = x/d;
                       (*surf).xus[tn][2] = y/d;
                       (*surf).xus[tn][3] = z/d;
                       tn++;
                      }     /* cycle tl2 */
                     }      /* cycle tl */
                }       /* cycle k */
           }        /* cycle j */
      }         /* cycle i */
   if((*surf).n_dot != (tn-1))
     {fprintf(stderr,"WARNING: ico_dot: n_dot(%d) and tn(%d) differ\n", (*surf).n_dot, tn);
     }
  }     /* end of if (tess > 1) */
}       /* end of routine ico_dot_dod */

/************************************************************************/
/* .................................................................... */
/* Mapping the unit sphere to a grid using a linked list.       */
/* This grid looks a little bit different than the grid used for the    */
/* grid atom.Tthe reason is, that it doesn't change during a calculation*/
/* and so it need less memory and is hopefully slightly faster      */
/* (especially on maschines with fast integer operations)       */
/*                                  */
void unitsphere_to_grid(Surf *surf)
{
  int ***help;
  int i,j,k,n,m, len;

  /* determine the grid dimensions - 2.0=radius of a unit sphere */
  if ((ceil((double)(*surf).cubes/2.0))==(floor((double)(*surf).cubes/2.0)))
     {muerror("Cubes should be an odd integer. unitsphere_to_grid()");
     }
  len=(int)((double)((*surf).cubes-1)/2.0);
  (*surf).spacing=2.0/(double)((*surf).cubes-1);
  (*surf).inv_spacing=1.0/(*surf).spacing;

  /* first look for the maximal length of a dot_no-vector */
  help=i3tensor(-len, len, -len, len, -len, len);

  for(i=-len;i<=len;i++)    /* Initializing */
     for(j=-len;j<=len;j++)
         for(k=-len;k<=len;k++)
             {help[i][j][k]=0;
             }
  for(n=1; n<=(*surf).n_dot; n++) /* counting */
     {i=rint((*surf).xus[n][1]*(*surf).inv_spacing);
      j=rint((*surf).xus[n][2]*(*surf).inv_spacing);
      k=rint((*surf).xus[n][3]*(*surf).inv_spacing);
      help[i][j][k]++;
     }
  (*surf).nmax=0;   /* search the highest value */
  for(i=-len;i<=len;i++)
     for(j=-len;j<=len;j++)
         for(k=-len;k<=len;k++)
            {if ((*surf).nmax<help[i][j][k]) (*surf).nmax=help[i][j][k];
            }
  /* allocate memory for dot_no */
  (*surf).dot_no=i4tensor(-len, len, -len, len, -len, len,0,(*surf).nmax);
  (*surf).imax=ivector(1,3);
  (*surf).imin=ivector(1,3);

  (*surf).imin[1]=-len; (*surf).imax[1]=len;
  (*surf).imin[2]=-len; (*surf).imax[2]=len;
  (*surf).imin[3]=-len; (*surf).imax[3]=len;

  /* initialize the whole grid with zero */
  for(i=-len;i<=len;i++)
     for(j=-len;j<=len;j++)
         for(k=-len;k<=len;k++)
            for(m=0;m<=(*surf).nmax;m++)
               (*surf).dot_no[i][j][k][m]=0;

  /* assign the points to dot_no */
  for(n=1; n<=(*surf).n_dot; n++) /* counting */
     {i=rint((*surf).xus[n][1]*(*surf).inv_spacing);
      j=rint((*surf).xus[n][2]*(*surf).inv_spacing);
      k=rint((*surf).xus[n][3]*(*surf).inv_spacing);
      (*surf).dot_no[i][j][k][0]++;
      (*surf).dot_no[i][j][k][(*surf).dot_no[i][j][k][0]]=n;
     }
  /* check consitency of the numbers and free the help tensor */
  n=0;
  for(i=-len;i<=len;i++)
     for(j=-len;j<=len;j++)
         for(k=-len;k<=len;k++)
            {n=n+help[i][j][k];
             if ((*surf).dot_no[i][j][k][0]!=help[i][j][k])
                muerror("Couning error in unitsphere_to_grid().");
            }

  if (n!=(*surf).n_dot)
     muerror("Number of dots not equal to assigned in  unitsphere_to_grid().");

  free_i3tensor(help, -len, len, -len, len, -len, len);
}
/************************************************************************/
/* The actual surface calculation                   */
/*                                  */
/* .................................................................... */
/* calls the surface calculation procedures             */
/* it return the surface area                       */
/*                                  */
double calculate_surface(double *area, char **flag, double **xyz,
             double *radius, int natom, Surf *surf,
             iListGrid *grid, iList *list)
{
int i;
double surface=0.0;
/* loop over all atoms */
for (i=1; i<=natom; i++)
    {/* remove the atom from the grid */
     remove_from_list(&list[i]);
     /* do the calculation of the solvent accessible area of this atom */
     area[i]=calculate_area(i, flag, xyz, radius, surf, grid);
     surface=surface+area[i];
     /* read the atom to the list */
     add_to_list(grid, &list[i], xyz[i]);
    }
return surface;
}

/* .................................................................... */
/* procedure calculates the solvent accesible area of a given atom  */
/*                                  */
/* when this function is called, the atom should not be in the list.    */
/* otherwise the solvent accessible area is allways be zero     */
/*                                  */
/* Variables                                */
/* il,jl,kl - left hand values for the second grid          */
/* ir,jr,kr - right hand values for the second grid         */
/*                                  */
double calculate_area(int no, char **flag, double **xyz, double *radius,
              Surf *surf, iListGrid *grid)
{
int i,j,k,ih,jh,kh,ix,iy,iz;       /* loop in the first grid  */
int il,jl,kl,ir,jr,kr, ia,ja,ka,c; /* loop in the second grid */
int acc, ne; /* ne==neighbor , acc == number of accessible point */
iList *help;
double d12[4], d2, r12, r22, r1, r2, right,len,fact;

/* set some initial values */
acc=(*surf).n_dot;
r1=radius[no];  r12=r1*r1;

/* map the atom on the grid */
ix=(int)rint(xyz[no][1]*(*grid).inv_spacing);
iy=(int)rint(xyz[no][2]*(*grid).inv_spacing);
iz=(int)rint(xyz[no][3]*(*grid).inv_spacing);


/* loop over all neighbors and look which surface dots are buried */
for (i=-(*grid).look; i<=(*grid).look; i++)
  {ih=ix+i;
   if(!((ih>(*grid).imax[1])||(ih<(*grid).imin[1])))
     {for (j=-(*grid).look; j<=+(*grid).look; j++)
        {jh=iy+j;
     if (!((jh>(*grid).imax[2])||(jh<(*grid).imin[2])))
       {for (k=-(*grid).look; k<=+(*grid).look; k++)
           {kh=iz+k;
            if (!((kh>(*grid).imax[3])||(kh<(*grid).imin[3])))
              {help=&(*grid).root[ih][jh][kh];
/* ======= code folded from here */
while((*help).next!=NULL) /* The in no entry in the grid!! */
  {help=(*help).next; ne=(*help).number;
   /* check via triangle inequility if the spheres interfere */
   SubVector(xyz[ne],xyz[no],d12);    d2=SqrLength(d12);
   r2=radius[ne]; r22=r2*r2;
   if (d2<=((r1+r2)*(r1+r2)))
      {/* calculate the right hand side value for the inequility */
       right=(d2+r12-r22)/(2*r1);
       /* determine in which cells in the grid second grid must be looked */
       fact=1.0/((*surf).spacing*r1);
       len=((double)((*surf).cubes-1)/2.0);

       il=(int)floor(MAXIMUM(((d12[1]-r2)*fact),-len));
       jl=(int)floor(MAXIMUM(((d12[2]-r2)*fact),-len));
       kl=(int)floor(MAXIMUM(((d12[3]-r2)*fact),-len));

       ir=(int)ceil(MINIMUM(((d12[1]+r2)*fact),len));
       jr=(int)ceil(MINIMUM(((d12[2]+r2)*fact),len));
       kr=(int)ceil(MINIMUM(((d12[3]+r2)*fact),len));

       /* loop over the grid and the surface dots */
       for(ia=il; ia<=ir; ia++)
      {for(ja=jl; ja<=jr; ja++)
          {for(ka=kl; ka<=kr; ka++)
          {/*fprintf(stderr,"%d  %d  %d\n",ia,ja,ka);*/
           if (0!=(*surf).dot_no[ia][ja][ka][0])
            {for(c=1; c<=(*surf).dot_no[ia][ja][ka][0]; c++)
              {if (flag[no][(*surf).dot_no[ia][ja][ka][c]]==OUTSIDE)
                  {if (ScalarProduct(d12, (*surf).xus[(*surf).dot_no[ia][ja][ka][c]])>right)
                  {flag[no][(*surf).dot_no[ia][ja][ka][c]]=INSIDE;
                   acc--;
                  }
              }
              }
            }
          }
          }
          }
      }
  }
/* ======= unfolding the code */
              }
            }
       }
        }
     }
  }

/* do the area calculation */
return Area(acc,(*surf).n_dot,r1);
}

/* .................................................................... */
/* calculates the dotsurface of a protein               */
/*                                  */

void calculate_dotsurface(DotSurf *dotsurf, char **flag, Surf *surf,
              double **xyz, double *radius, int natoms)
{
int i,j,k;

k=0;
for (i=1; i<=natoms; i++)
    {for(j=1; j<=(*surf).n_dot;j++)
    {if (flag[i][j]==OUTSIDE)
        {k++;
             if (k>(*dotsurf).ndots)
                {muerror("Too many dots in calculate_dotsurface()");}
         (*dotsurf).dots[k][1]=(radius[i]*(*surf).xus[j][1])+xyz[i][1];
         (*dotsurf).dots[k][2]=(radius[i]*(*surf).xus[j][2])+xyz[i][2];
         (*dotsurf).dots[k][3]=(radius[i]*(*surf).xus[j][3])+xyz[i][3];
        }
     else {if (flag[i][j]!=INSIDE)
          {muwarn("Unknown flag in calculate_dotsurface().");
           fprintf(stderr,"%d  %d  %c %d\n",i,j,flag[i][j],(int)flag[i][j]);;
          }
          }
    }
    }

}
/* .................................................................... */
/* This routine calculates the surface does all memory allocation etc   */
/*                                                                      */

double surface(double *atomsurf, double **xyz, double *radius,
               int natoms, int densit, double prob, DotSurf *dotsurf,
               int surfflag)
{
  int i,j;
  double max[4], min[4], off_set, srf, *rad;
  double len;
  int itest;
  Surf surf;
  iListGrid atom_list;
  iList *atoms;
  char **flag;  /* [1..natom][1..n_dot] INSIDE - OUTSIDE */

  rad=dvector(1,natoms);
  for (i=1; i<=natoms; i++)
        rad[i]=radius[i]+prob;

  /* determinte which tessellation type should be used */
  itest=tessellation_type(densit);
  /* calculate the tesselation of a unit sphere */
  switch (itest)
      {case UNSP_ICO_ARC:  fprintf(stdout,"Use tessellation type II\n"),
                           ico_dot_arc(densit, &surf);
                           fprintf(stdout,"I use %d dots\n",surf.n_dot);
                           break;
       case UNSP_ICO_DOD:  fprintf(stdout,"Use tessellation type I\n");
                           ico_dot_dod(densit, &surf);
                           fprintf(stdout,"I use %d dots\n",surf.n_dot);
                           break;
       default:            muerror("Don't know this tessellation type");
      }

   /* ============================================================= */
  /* renormalize in order to be sure that it is a unit sphere       */
   for (j=1; j<=surf.n_dot; j++)
      {len = sqrt(SqrLength(surf.xus[j]));
       surf.xus[j][1]=surf.xus[j][1]/len;
       surf.xus[j][2]=surf.xus[j][2]/len;
       surf.xus[j][3]=surf.xus[j][3]/len;
      }
   /* ============================================================= */
   /* sort the dots in a Grid linked cubes=cubes in one dimension   */
   surf.cubes=5;
   unitsphere_to_grid(&surf);


  /* ============================================================ */
  /* allocate and initilize the flags with OUTSIDE */
  /* look for the needed grid dimension */
  atom_list.imax=ivector(1,3);
  atom_list.imin=ivector(1,3);

  off_set=10.0;
  atom_list.spacing=2.0*max_element(rad, natoms);
  atom_list.inv_spacing=1.0/atom_list.spacing;
  min_max_coord(min, max, xyz, natoms);
  atom_list.inv_spacing=1.0/atom_list.spacing;
  atom_list.imax[1]=ceil((max[1]+off_set) *atom_list.inv_spacing);
  atom_list.imax[2]=ceil((max[2]+off_set) *atom_list.inv_spacing);
  atom_list.imax[3]=ceil((max[3]+off_set) *atom_list.inv_spacing);
  atom_list.imin[1]=floor((min[1]-off_set)*atom_list.inv_spacing);
  atom_list.imin[2]=floor((min[2]-off_set)*atom_list.inv_spacing);
  atom_list.imin[3]=floor((min[3]-off_set)*atom_list.inv_spacing);
  atom_list.look=1;
  /* make the grid spacing to odd integer, if they are not odd integer */
  odding_down(atom_list.imin,3);
  odding_up(atom_list.imax,3);

 /* allocation */
  flag=charmatrix(1,natoms,1,surf.n_dot);
  atoms=iList_vector(1,natoms);
  atom_list.root=iList_3tensor(atom_list.imin[1], atom_list.imax[1],
                               atom_list.imin[2], atom_list.imax[2],
                               atom_list.imin[3], atom_list.imax[3]);

  for (j=1; j<=natoms; j++)
         {init_charvector(flag[j],surf.n_dot,OUTSIDE);
          atomsurf[j]=0.0;
         }
  init_iList_vector(atoms,natoms);
  init_iListGrid(&atom_list);
  fill_iListGrid(&atom_list, atoms, xyz, natoms);
  srf=calculate_surface(atomsurf, flag, xyz,rad, natoms,
                       &surf,&atom_list, atoms);

   if (surfflag==1)
      {/* calculate the dot surface */
       /* count the surface dots and allocate memory for the surface dots */
       (*dotsurf).ndots=0;
       for (i=1; i<=natoms; i++)
           for(j=1; j<=surf.n_dot;j++)
              if (flag[i][j]==OUTSIDE) (*dotsurf).ndots++;

       (*dotsurf).dots=dmatrix(1,(*dotsurf).ndots,1,3);
       /* do the calculation */
       calculate_dotsurface(dotsurf,flag,&surf,xyz,
                        rad,natoms);
      }

  /* release memory */
  free_i4tensor(surf.dot_no, surf.imin[1], surf.imax[1],
                             surf.imin[2], surf.imax[2],
                             surf.imin[3], surf.imax[3],
                             0,surf.nmax);
  free_dmatrix(surf.xus,1,surf.n_dot, 1,3);
  free_dmatrix(surf.help,1,surf.n_dot, 1,3);
  free_ivector(surf.imin, 1,3);
  free_ivector(surf.imax, 1,3);
  free_dvector(rad,1,natoms);
  free_charmatrix(flag,1,natoms,1,surf.n_dot);
  free_iList_vector(atoms,1,natoms);
  free_iList_3tensor(atom_list.root,atom_list.imin[1], atom_list.imax[1],atom_list.imin[2], atom_list.imax[2],atom_list.imin[3], atom_list.imax[3]);
  free_ivector(atom_list.imax,1,3);
  free_ivector(atom_list.imin,1,3);
  return srf;
}
