//
// C++ Implementation: grid_helper
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "grid_helper.h"


/** Map a proteins atoms coordinates on a iGrid

*/
void ProteinToiGrid(iGrid *grid, MUS_atom *p, double maxrad, mc_var *vars) {
  double min[4], max[4];
  int i, j, k;

  min_max_coord(min, max, (*p).xyz, (*p).number_of_atoms);
  max[1] = (*vars).max[1] + maxrad;
  max[2] = (*vars).max[2] + maxrad;
  max[3] = (*vars).max[3] + maxrad;
  min[1] = (*vars).min[1] - maxrad;
  min[2] = (*vars).min[2] - maxrad;
  min[3] = (*vars).min[3] - maxrad;

  /* Assigning the values for the grid */
  (*grid).inv_spacing = 1.0 / (*grid).spacing;
  (*grid).imax[1] = (int) ceil(max[1] * (*grid).inv_spacing);
  (*grid).imax[2] = (int) ceil(max[2] * (*grid).inv_spacing);
  (*grid).imax[3] = (int) ceil(max[3] * (*grid).inv_spacing);

  (*grid).imin[1] = (int) floor(min[1] * (*grid).inv_spacing);
  (*grid).imin[2] = (int) floor(min[2] * (*grid).inv_spacing);
  (*grid).imin[3] = (int) floor(min[3] * (*grid).inv_spacing);

  /* make the grid spacing to odd integer, if they are not odd integer */
  odding_down((*grid).imin, 3);
  odding_up((*grid).imax, 3);
  (*grid).grid = i3tensor((*grid).imin[1], (*grid).imax[1],
                          (*grid).imin[2], (*grid).imax[2],
                          (*grid).imin[3], (*grid).imax[3]);

  /* Initialization */
  for (i = (*grid).imin[1]; i <= (*grid).imax[1]; i++)
    for (j = (*grid).imin[2]; j <= (*grid).imax[2]; j++)
      for (k = (*grid).imin[3]; k <= (*grid).imax[3]; k++) {
        (*grid).grid[i][j][k] = 0;
      }

}

/** Set iGrid to some values

*/
void set_iGrid(iGrid &g, int min1, int min2, int min3, int max1, int max2, int max3) {
  g.imin[1] = min1;
  g.imin[2] = min2;
  g.imin[3] = min3;
  g.imax[1] = max1;
  g.imax[2] = max2;
  g.imax[3] = max3;
}

/** Initialize an iGrid to a value

\param ini The value for initialization
*/
void initialize_iGrid(iGrid &g, int ini) {
  for (int i = g.imin[1]; i <= g.imax[1]; i++) {
    for (int j = g.imin[2]; j <= g.imax[2]; j++) {
      for (int k = g.imin[3]; k <= g.imax[3]; k++) {
        g.grid[i][j][k] = ini;
      }
    }
  }
}

/** Set all 0 value to x

Within an iGrid, set all values which are 0 to x
\param x
*/
void set_0_to_x(iGrid &g, int x) {
  for (int i = g.imin[1]; i <= g.imax[1]; i++) {
    for (int j = g.imin[2]; j <= g.imax[2]; j++) {
      for (int k = g.imin[3]; k <= g.imax[3]; k++) {
        if (g.grid[i][j][k] == 0) {
          g.grid[i][j][k] = x;
        }
      }
    }
  }
}

/** Get the volume of a iGrid box

*/
double get_box_volume(iGrid &g) {
  return (g.imax[1] - g.imin[1]) * (g.imax[2] - g.imin[2]) * (g.imax[3] - g.imin[3]);
}


