//
// C++ Interface: mit_surface
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
// These routines implement Protein dot Surfaces for C++. Method is based on Eisenhaber, some Routines are from G. Matthias Ullmann

#ifndef MIT_SURFACE_H
#define MIT_SURFACE_H
#include <iostream>
#include <string>
#include "surface.h"
#include "util.h"
#include "util_c.h"
#include "nrutil.h"
#include "mathematik.h"
#include "constants.h"
#include "grid.h"

/** Point of the dot Surface

Point for the linked list of dot surface points
*/
class spoint {
  public:
    int n;
    spoint *next;
    spoint *prev;
    spoint(spoint *n, spoint *p): next(n), prev(p) {};
    spoint(): next(NULL), prev(NULL) {};
};


/** Wrapper class for an dot surface

This class handels everything necessary to deal with an dot surface.
*/
class mit_surface {
  private:
    double surface_in_A; /**< The surface in A^2 */
    DotSurf dotsurf; /**< The actual dot surface */
    vec<float> surface_per_dot; /**< The calculated surface per dot */
    vec<short> surface_clusters;/**< Belonging of each point to a surface cluster */
    int ngraph; /**< Number of surface clusters */
  public:
    mit_surface(double *atomsurf, double **xyz, double *radius,
                int natoms, int densit, double prob, int surfflag);

    double get_surface(){return surface_in_A;};
    DotSurf *get_DotSurf_p(){return &dotsurf;};
    int get_ndots(){return dotsurf.ndots;}
    void print_dotsurf_pqr(ofstream &out);
    void print_clusters_pqr_surf();
    void print_clusters_pqr_cav();
    void print_clusters_surfaces();
    void save_atom_radii_with_points(DotSurf *dotsurf, char **flag, Surf *surf,double *radius, int natoms);
    int cluster(double dist, iListGrid &dotgrid);
    /** Destructor

    Free memory
    */
    ~mit_surface() {
      free_dmatrix(dotsurf.dots, 1, dotsurf.ndots, 1, 3);
    }
};
#endif
