//
// C++ Interface: monte_carlo_run
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef MONTE_CARLO_RUN_H
#define MONTE_CARLO_RUN_H
#include <iostream>
#include <string>
#include <cfloat>
//#include "mathematik.h"
#include "random.h"
#include "mus_atom.h"
#include "util.h"
//#include "util_c.h"
//#include "nrutil.h"
//#include "energy.h"
#include "grid.h"
//#include "nb-list.h"
#include "surface.h"
#include "volume.h"
//#include "cavity_search.h"
//#include "cavity2.h"
//#include "mit_surface.h"
//#include "grid_helper.h"

/** Container for all variables neede by a Monte Carlo run

*/
struct mc_var {
  double min[4]; /**< Minimum coordinates of the grid */
  double max[4]; /**< Maximum coordinates of the grid */
  int nmc;       /**< Number of Monte Carlo steps*/
  double maxrad; /**< Maximum radius of the atoms */
  double prob;   /**< Probe sphere radius */
  int nprt;      /**< Print an output each nprt MC steps */
  int blab;
  /** Constructor

  Initialize all variables to 0
  */
  mc_var(): nmc(0), maxrad(0), prob(0), nprt(0), blab(0) {};
};


/** Wrapper class for the Monte Carlo Volume calculation

This class contains the whole Monte Carlo volume calculation stuff. Variables needed by the MC routine...
*/
class monte_carlo_run {
  public:
    mc_var vars; /**< MC variables */
    monte_carlo_run(){};
    ~monte_carlo_run(){};
    double volume_mc(iListGrid *atomgrid, iListGrid *dotgrid, iGrid  *solvgrid, MUS_atom *p, DotSurf *dotsurf, long *seed);
    int pt_inside_atom(double *pt, iListGrid *atomgrid, double **xyz, vec<double> &pr2, vec<double> &r2);
    int pt_inside_void(double *pt, iListGrid *dotgrid, double **dots, double prob2);
    void set_vars(int *minc, int *maxc, int n, double mr, double pr, int npr, int blab);
};

#endif
