#ifndef D4TENSOR_
#define D4TENSOR_
#include <iostream>
#include <iomanip>
#include "matrix.h"
#include "tensor.h"


/** 4d tensor

4 dimensional tensor. Allocates Memory for a four dimensional array and manages this memory. The memory is freed in the destructor to prevent memory leaks. Arbitraty interger values can be used as indices. Memory is allocated in the init_arr function, needs 8 parameters showing the borders in each direction or two ints, a vector, a matrix and a 3d tensor if the number of rows is not the same in each dimension. Elements are indexd by 4d_tesnso_name(x,y,z,t).
*/
template<class T>
class d4_tensor {
  private:
    int dim1, dim2, dim3, dim4; /**< dimensions if not spare */
    int start1, start2, start3, start4; /**< Start or left hand value for each dimension */
    float space;/**< space allocated */
    int *spare;/** spare vector */
    matrix<int> spare_mat;
    tensor<int> spare_ten;
  public:
    T ****arr; /**< The real data structure */
    d4_tensor(): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), space(sizeof(d4_tensor)), spare(NULL), arr(NULL) {

    };
    d4_tensor(int x, int x2, int y, int y2, int z, int z2, int k, int k2): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), space(sizeof(d4_tensor)), spare(NULL), arr(NULL) {
      init_arr(x,x2,y,y2,z,z2,k,k2);
    };
    d4_tensor(int x, int x2, int y, int y2, int z, int z2, int k, int k2, int ini): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), space(sizeof(d4_tensor)), spare(NULL), arr(NULL) {
      init_arr(x,x2,y,y2,z,z2,k,k2,ini);
    };
    d4_tensor(int x, int x2, int ini, int *sizes1, matrix<int> &sizes2, tensor<int> &sizes3): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), space(sizeof(d4_tensor)), spare(NULL), arr(NULL) {
      init_arr(x,x2,sizes1,sizes2,sizes3);
    };
    void init_arr(int x, int x2, int y, int y2, int z, int z2, int k, int k2) {
      dim1 = x2 - x +1;
      dim2 = y2 - y +1;
      dim3 = z2 - z +1;
      dim4 = k2 - k +1;
      start1 = x;
      start2 = y;
      start3 = z;
      start4 = k;
      try {
        arr = new T***[dim1];
        space += dim1 * sizeof(T***);
        arr = arr - start1;
        arr[x] = new T**[dim1*dim2+1];
        space += (dim1 * dim2 +1) * sizeof(T**);
        arr[x] -= start2;
        arr[x][y] = new T*[dim1*dim2*dim3+1];
        space += (dim1 * dim2 * dim3 + 1.0) * (float)sizeof(T*);
        arr[x][y] -= start3;
        arr[x][y][z] = new T[dim1*dim2*dim3*dim4+1];
        space += (dim1 * dim2 *dim3*dim4 +1) * sizeof(T);
        arr[x][y][z] -= start4;
        // set second dimension
        for (int i=y+1; i<=y2; i++){arr[x][i] = arr[x][i-1]+dim3;}
        // set third dimension
        for (int i=x+1; i<=x2; i++) {
          arr[i] = arr[i-1] + dim2;
          arr[i][y] = arr[i-1][y] + dim2*dim3;
          arr[i][y][z] = arr[i-1][y][z] + dim2*dim3*dim4;
          for (int j=y+1;j<=y2;j++) {
            arr[i][j] = arr[i][j-1]+dim3;
          }
        };
        for (int i=x; i<=x2; i++) {
          for (int j=y+1;j<=y2;j++) {
            arr[i][j][z] = arr[i][j-1][z] + dim3*dim4;
          };
        };
        for (int i=x; i<=x2; i++) {
          for (int j=y;j<=y2;j++) {
            for (int h=z+1; h<=z2; h++) {
              arr[i][j][h] = arr[i][j][h-1] + dim4;
            };
          };
        };
      } catch (bad_alloc) {
        cerr << "To less memory for tesor!" << endl;
        cerr << "dimensions: " << dim1 << " " << dim2 << " " << dim3 << endl;
        exit(0);
      };
    };
    void init_arr(int x, int x2, int y, int y2, int z, int z2, int k, int k2, T ini) {
      init_arr(x,x2,y,y2,z,z2,k,k2);
      for (int h1=x; h1<=x2; h1++) {
        for (int h2=y; h2<=y2; h2++) {
          for (int h3=z; h3<=z2; h3++) {
            for (int h4=k; h4<=k2; h4++) {
              arr[h1][h2][h3][h4] = ini;
            };
          };
        };
      };
    };
    void init_arr(int x, int x2, vec<int> &sizes1, matrix<int> &sizes2, tensor<int> &sizes3) {
      if (spare == NULL) {
        spare = new int[x2-x + 2];
        space += sizeof(int)*(x2-x+2);
        spare_mat.init_arr(x,x2,sizes1);
        space += spare_mat.get_space();
        spare_ten.init_arr(x,x2,sizes1,sizes2);
        space += spare_ten.get_space();
        for (int i=x; i<= x2; i++) {
          spare[i] = sizes1(i);
          for (int j=1; j<= sizes1(i); j++) {
            spare_mat(i,j) = sizes2(i,j);
            for (int k=1; k<= sizes2(i,j); k++) {
              spare_ten(i,j,k) = sizes3(i,j,k);
            }
          }
        }
      }
      dim1 = x2 - x +1;
      dim2 = 0;
      dim3 = 0;
      dim4 = 0;
      start1 = x;
      start2 = 1;
      start3 = 1;
      start4 = 1;
      try {
        arr = new T***[dim1];
        arr = arr - start1;
        space += sizeof(T***)*dim1;
        for (int i=start1;i<=dim1;++i) {
          arr[i] = new T**[sizes1(i)];
          arr[i] = arr[i] - start2;
          space += sizeof(T**)*sizes1(i);
        }
        for (int i=start1;i<=dim1;i++) {
          for (int j=start2;j<=sizes1(i);++j) {
            arr[i][j] = new T*[sizes2(i,j)];
            arr[i][j] = arr[i][j] - start3;
            space += sizeof(T*)*sizes2(i,j);
          }
        }
        for (int i=start1;i<=dim1;i++) {
          for (int j=start2;j<=sizes1(i);++j) {
            for (int t=start3;t<=sizes2(i,j);t++) {
              arr[i][j][t] = new T[sizes3(i,j,t)];
              arr[i][j][t] = arr[i][j][t] - start4;
              space += sizeof(T)*sizes3(i,j,t);
            };
          }
        }
      } catch (bad_alloc) {
        cout << "To less memory for tesor!" << endl;
        exit(0);
      };
    };
    void init_arr(int x, int x2, vec<int> &sizes1, matrix<int> &sizes2, tensor<int> &sizes3, T ini) {
      init_arr(x,x2,sizes1,sizes2,sizes3);
      for (int h1=x; h1<=x2; h1++) {
        for (int h2=1; h2<=sizes1(h1); h2++) {
          for (int h3=1; h3<=sizes2(h1,h2); h3++) {
            for (int h4=1; h4<=sizes3(h1,h2,h3); h4++) {
              arr[h1][h2][h3][h4] = ini;
            };
          };
        };
      };
    };

    void init_arr(int x, int x2, int *sizes1, matrix<int> &sizes2, tensor<int> &sizes3) {
      vec<int> tmp_vec(x,x2,0);
      for (int i=x; i<=x2; i++) {
        tmp_vec(i) = sizes1[i];
      };
      init_arr(x,x2,tmp_vec,sizes2,sizes3);
    };

    void kill_arr() {

      if (spare!=NULL) {
        for (int i=start1;i<=start1 + dim1-1;i++) {
          for (int j=start2;j<=spare[i];++j) {
            for (int t=start3;t<=spare_mat(i,j);t++) {
              arr[i][j][t] = arr[i][j][t] + start4;
              delete [] arr[i][j][t];
            }
          }
        };
        for (int i=start1;i<=dim1;i++) {
          for (int j=start2;j<=spare[i];++j) {
            arr[i][j] = arr[i][j] + start3;
            delete [] arr[i][j];
          }
        }
        for (int i=start1;i<=start1 + dim1-1;++i) {
          arr[i] = arr[i] + start2;
          delete [] arr[i];
        };
        arr = arr + start1;
        delete [] arr;
        delete [] spare;
        spare_ten.kill_arr();
        spare_mat.kill_arr();
      } else {
        if (arr != NULL) {
          arr[start1][start2][start3] += start4;
          delete [] arr[start1][start2][start3];
          arr[start1][start2] += start3;
          delete [] arr[start1][start2];
          arr[start1] += start2;
          delete [] arr[start1];
          arr += start1;
          delete [] arr;
        };
      };
      spare = NULL;
      arr = NULL;
      space = 0;
    };
    int get_dim1(){return dim1;}
    int get_dim2(int x){return spare[x];}
    int get_dim3(int x, int y){return spare_mat(x,y);}
    int get_dim4(int x, int y, int z){return spare_ten(x,y,z);}
    double get_space(){return space;};
    ~d4_tensor() {
      if (arr) {
        kill_arr();
      };
    };
    inline T &operator()(int x, int y, int z, int k) {
      return arr[x][y][z][k];
    };
    T *operator()(int x, int y, int z) {
      return arr[x][y][z];
    };
};









#endif
