//
// C++ Interface: adv_doubly_ll
//
// Description:
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef ADV_DOUBLY_LL_H
#define ADV_DOUBLY_LL_H
#include <iostream>
#include "mempool.h"

template<class T>
class adv_doubly_LL {
  private:
    struct node {
      T *item;
      node *next;
      node *prev;
      node():item(NULL),next(NULL),prev(NULL) {};
      node(T *x):item(x),next(NULL),prev(NULL) {}
    };
    node *head, *tail;
    mempool<node> pool;
    int nelements;
    char allow_clear;
  public:
    class iterator {
      public:
        node *inode;
        iterator(node *p):inode(p) {};
        bool operator==(const iterator &other) {
          return inode == other.inode;
        }
        bool operator!=(const iterator &other) {
          return inode != other.inode;
        }
        iterator& operator++() {
          if (inode!=NULL) {
            inode = inode->next;
          }
          return (*this);
        }
        iterator& operator--() {
          if (inode!=NULL) {
            inode = inode->prev;
          }
          return (*this);
        }
        iterator& operator++(int) {
          if (inode!=NULL) {
            inode = inode->next;
          }
          return(*this);
        }
        iterator& operator--(int) {
          if (inode!=NULL) {
            inode = inode->prev;
          }
          return(*this);
        }
        T* operator*() {
          return inode->item;
        }
    };
    adv_doubly_LL():head(NULL),tail(NULL),nelements(0),allow_clear(1) {}
    bool empty() const {return head == NULL;}
    void add_tail(T *x) {
      node *t = tail;
      tail = pool.get_el();
      tail->next = NULL;
      tail->prev = t;
      tail->item = x;
      if (head == NULL) {
        head = tail;
      } else{t->next = tail;}
      nelements++;
    }
    T* get_first() {
      if (head == NULL){return NULL;}
      T *v = head->item;
      node *t = head->next;
      head->item = NULL;
      pool.return_el(head);
      head = t;
      if (t!=NULL){head->prev = NULL;}
      nelements--;
      return v;
    }
    T* delete_element(iterator &ite) {
      node *it = ite.inode;
      ite++;
      if (it == NULL) {
        return NULL;
      }
      if ((*it).prev != NULL) {
        (*it).prev->next = (*it).next;
      }
      if ((*it).next != NULL) {
        (*it).next->prev = (*it).prev;
      }
      if (it == head) {
        head = head->next;
      }
      if (it == tail) {
        tail = tail->prev;
      }
      T *v = it->item;
      pool.return_el(it);
      nelements--;
      return v;
    };
    double size(){return sizeof(adv_doubly_LL) + pool.size();}
    int nel(){return nelements;}
    void clear() {
      T *iter;
      iter = get_first();
      while (iter != NULL) {
        delete iter;
        iter = get_first();
      };
    }
    ~adv_doubly_LL() {
      if (allow_clear == 1) {
        clear();
      };
    };
    void set_allow_clear(char s){allow_clear = s;}
    iterator begin(){return iterator(head);}
    iterator last(){return iterator(tail);}
    iterator end(){return NULL;}
};

#endif
