//
// C++ Implementation: energy_calcs
//
// Description:
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "energy_calcs.h"

boltzmann_energies get_partition_function(matrix<float> &gint, d4_tensor<float> &ww, vec<int> &max_vec, vec<int> &min_vec, int target, int target_start, int target_stop, double corr_val, boltzmann_energies &result, double histo_step, double histo_min, double histo_max){
    double Z, Z2, curr_en, nstates, min, max;
    int i,j, inc_ret, start_flag;
    char filename[100];
    boltzmann_energies return_val;
    histogram histo(histo_step,histo_min, histo_max);
    sprintf(filename,"histo_%d.dat", target);
    ofstream histo_out(filename);
    vec<int> local_start_vec, local_max_vec, local_vec;
    local_start_vec.init_arr(1,max_vec.get_dim());
    local_max_vec.init_arr(1,max_vec.get_dim());
    local_vec.init_arr(1,max_vec.get_dim());
    nstates=1;
    for(int i=1; i<=max_vec.get_dim(); i++){
        local_vec(i) = min_vec(i);
        local_max_vec(i) = max_vec(i);
        local_start_vec(i) = min_vec(i);
    };
    // cout << target_start << " " << target_stop << endl;
    local_max_vec(target) = target_stop;
    local_start_vec(target) = target_start;
    local_vec(target) = target_start;
    local_vec(1) = local_vec(1) - 1;
    Z = Z2 = 0;
    max = 1E100;
    min = 0;
    nstates=1;

    for(int i=1; i<=max_vec.get_dim(); i++){
        nstates = nstates * (local_max_vec(i) - local_start_vec(i) + 1);
    };
    //cout << "Calculating partition function for " << nstates << " states" << endl;
    start_flag = 1;
    //curr_en = 0;
    curr_en = 1;
    while((inc_ret = increase_protstate(local_vec, max_vec.get_dim(), local_max_vec, target, local_start_vec, target_start))){
        if(inc_ret < max_vec.get_dim() && start_flag == 0){
            /*
            curr_en -= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en += gint(inc_ret,local_vec(inc_ret));
            */
            curr_en /= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en *= gint(inc_ret,local_vec(inc_ret));
            for(i=1; i<=max_vec.get_dim(); i++){
                /*
                curr_en -= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en += ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
                */
                curr_en /= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en *= ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
            };
           // cout << curr_en << " ";
        }else{
        start_flag = 0;
        curr_en = 1;
        for(i=1; i<=max_vec.get_dim(); i++){
            curr_en *= gint(i,local_vec(i));
            //curr_en += gint(i,local_vec(i));
            for(j=i+1; j<=local_vec.get_dim(); j++){
                curr_en *= ww(i,local_vec(i),j,local_vec(j));
                //curr_en += ww(i,local_vec(i),j,local_vec(j));
            };
        };
        //cout << curr_en << endl;
        };
        /*
        Z += exp((curr_en)/-util_rt);
        Z2 += (exp(curr_en/-util_rt))*curr_en;
        */
        Z += curr_en;
        Z2 += curr_en;
        if(histo_step > 0 && curr_en > histo_min && curr_en < histo_max){
            histo.add_element(curr_en);
        };
        curr_en < max ? max = curr_en: 1;
        curr_en > min ? min = curr_en: 1;
    };
    //cout << "Partition function calculated " << Z << endl;
    result.Z = Z;
    result.mean_energy = Z2/Z;
    result.min_energy = log(min)*-util_rt;
    result.max_energy = log(max)*-util_rt;
   // histo.sort_elements();
    if(histo_step > 0){
        histo_out << histo << endl;
    };
    return result;
}
/** Get targeted partition function

*/
boltzmann_energies get_partition_function(matrix<float> &gint, d4_tensor<float> &ww, vec<int> &max_vec, vec<int> &min_vec, int target, int target_start, int target_stop, boltzmann_energies &result){
  double Z, Z2, curr_en, nstates, min, max;
  int i,j, inc_ret, start_flag;
  boltzmann_energies return_val;
  vec<int> local_start_vec, local_max_vec, local_vec;
  local_start_vec.init_arr(1,max_vec.get_dim());
  local_max_vec.init_arr(1,max_vec.get_dim());
  local_vec.init_arr(1,max_vec.get_dim());
  nstates=1;
  for(int i=1; i<=max_vec.get_dim(); i++){
    local_vec(i) = min_vec(i);
    local_max_vec(i) = max_vec(i);
    local_start_vec(i) = min_vec(i);
  };
    // cout << target_start << " " << target_stop << endl;
  local_max_vec(target) = target_stop;
  local_start_vec(target) = target_start;
  local_vec(target) = target_start;
  local_vec(1) = local_vec(1) - 1;
  Z = Z2 = 0;
  max = 1E100;
  min = 0;
  nstates=1;

  for(int i=1; i<=max_vec.get_dim(); i++){
    nstates = nstates * (local_max_vec(i) - local_start_vec(i) + 1);
  };
    //cout << "Calculating partition function for " << nstates << " states" << endl;
  start_flag = 1;
    //curr_en = 0;
  curr_en = 1;
  while((inc_ret = increase_protstate(local_vec, max_vec.get_dim(), local_max_vec, target, local_start_vec, target_start))){
    if(inc_ret < max_vec.get_dim() && start_flag == 0){
            /*
      curr_en -= gint(inc_ret,local_vec(inc_ret)-1);
      curr_en += gint(inc_ret,local_vec(inc_ret));
            */
      curr_en /= gint(inc_ret,local_vec(inc_ret)-1);
      curr_en *= gint(inc_ret,local_vec(inc_ret));
      for(i=1; i<=max_vec.get_dim(); i++){
                /*
        curr_en -= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
        curr_en += ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
                */
        curr_en /= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
        curr_en *= ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
      };
           // cout << curr_en << " ";
    }else{
      start_flag = 0;
      curr_en = 1;
      for(i=1; i<=max_vec.get_dim(); i++){
        curr_en *= gint(i,local_vec(i));
            //curr_en += gint(i,local_vec(i));
        for(j=i+1; j<=local_vec.get_dim(); j++){
          curr_en *= ww(i,local_vec(i),j,local_vec(j));
                //curr_en += ww(i,local_vec(i),j,local_vec(j));
        };
      };
        //cout << curr_en << endl;
    };
        /*
    Z += exp((curr_en)/-util_rt);
    Z2 += (exp(curr_en/-util_rt))*curr_en;
        */
    Z += curr_en;
    Z2 += curr_en;
    curr_en < max ? max = curr_en: 1;
    curr_en > min ? min = curr_en: 1;
  };
    //cout << "Partition function calculated " << Z << endl;
  result.Z = Z;
  result.mean_energy = Z2/Z;
  result.min_energy = log(min)*-util_rt;
  result.max_energy = log(max)*-util_rt;
   // histo.sort_elements();
  return result;
}


/** Calculate any partition function

Calculates the partiton function for a set of states defined by two vectors. The calcualtion is increased a lot by only adding the energies of the last changed site instead of recalculationg the whole story
\param gint Intrinsic energies. Needs to be exp(ener/-RT)!!!!
\param ww Interaction energies. Needs to be exp(ener/-RT)!!!!
\param max_vec Vector holding the maximum instance for each site.
\param min_vec Vector holding the minimum instance for each site.
\param result Class to store the result of this calculation.
*/
boltzmann_energies get_partition_function(matrix<float> &gint, d4_tensor<float> &ww, vec<int> &max_vec, vec<int> &min_vec, boltzmann_energies &result){
    double Z, Z2, curr_en, nstates, max, min;
    int i,j, start_flag, inc_ret;
    vec<int> local_start_vec, local_max_vec, local_vec;
    local_start_vec.init_arr(1,max_vec.get_dim());
    local_max_vec.init_arr(1,max_vec.get_dim());
    local_vec.init_arr(1,max_vec.get_dim());
    nstates=1;
    for(int i=1; i<=max_vec.get_dim(); i++){
        local_vec(i) = min_vec(i);
        local_max_vec(i) = max_vec(i);
        local_start_vec(i) = min_vec(i);
    };

    // FIXME reduce first site because it is increased already at the first step
    local_vec(1) -= 1;
    Z = Z2 = 0;
    max = 1E100; // funny thing, but since we go with exp(ener/-rt)ed energies...
    min = 0; // funny thing,  but since we go with exp(ener/-rt)ed energies...
    nstates=1;
    // Just to inform you...
    for(int i=1; i<=max_vec.get_dim(); i++){
      nstates = nstates * (local_max_vec(i) - local_start_vec(i) + 1);
    };

    cout << "Calculating partition function for " << nstates << " states" << endl;
    cout << "Minimum vector: " << min_vec;
    cout << "Maximum vector: " << max_vec;
    start_flag = 1;
    curr_en = 1;
    // Go for all states
    while((inc_ret = increase_protstate(local_vec, max_vec.get_dim(), local_max_vec, local_start_vec))){
      // The magic decicion! If we have already a state energy, only subtract the changes and add the new energies
        if(inc_ret < max_vec.get_dim() && start_flag == 0){


            /* This would be the case, if we have no exped energies
            curr_en -= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en += gint(inc_ret,local_vec(inc_ret));
            */
            curr_en /= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en *= gint(inc_ret,local_vec(inc_ret));
            for(i=1; i<=max_vec.get_dim(); i++){
                /* This would be the case, if we have no exped energies
                curr_en -= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en += ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
                */
                curr_en /= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en *= ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
            };
           // cout << curr_en << " ";
        }else{
            start_flag = 0;
            curr_en = 1;
        for(i=1; i<=max_vec.get_dim(); i++){
            //curr_en += gint(i,local_vec(i));
            curr_en *= gint(i,local_vec(i));
         //   cout << local_vec(i) << " ";
            for(j=i+1; j<=local_vec.get_dim(); j++){
                //curr_en += ww(i,local_vec(i),j,local_vec(j));
                curr_en *= ww(i,local_vec(i),j,local_vec(j));
            };
        };

        };
        Z += curr_en;
        Z2 += curr_en;
        curr_en < max ? max = curr_en: 1;
        curr_en > min ? min = curr_en: 1;
    };
    //Thats all. Store the result.
    result.Z = Z;
    result.mean_energy = Z2/Z;
    result.min_energy = log(min)*-util_rt;
    result.max_energy = log(max)*-util_rt;
    return result;
}

void get_entropy(matrix<float> &gint, d4_tensor<float> &ww, vec<int> &max_vec, vec<int> &min_vec, int target, int target_start, int target_stop, boltzmann_energies &result){
    double entrop, p, curr_en, nstates;
    int i,j, start_flag, inc_ret;
    vec<int> local_start_vec, local_max_vec, local_vec;
    local_start_vec.init_arr(1,max_vec.get_dim());
    local_max_vec.init_arr(1,max_vec.get_dim());
    local_vec.init_arr(1,max_vec.get_dim());
    nstates=1;
    for(int i=1; i<=max_vec.get_dim(); i++){
        local_vec(i) = min_vec(i);
        local_max_vec(i) = max_vec(i);
        local_start_vec(i) = min_vec(i);
    };
    local_max_vec(target) = target_stop;
    local_start_vec(target) = target_start;
    local_vec(target) = target_start;
    entrop = 0;
    nstates=1;

    for(int i=1; i<=max_vec.get_dim(); i++){
        nstates = nstates * (local_max_vec(i) - local_start_vec(i) + 1);
    };

    //cout << "Calculating partition function for " << nstates << " states" << endl;
    start_flag = 1;
    curr_en = 1;
    while((inc_ret = increase_protstate(local_vec, max_vec.get_dim(), local_max_vec, target, local_start_vec, target_start))){
        if(inc_ret < max_vec.get_dim() && start_flag == 0){
            /*
            curr_en -= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en += gint(inc_ret,local_vec(inc_ret));
            */
            curr_en /= gint(inc_ret,local_vec(inc_ret)-1);
            curr_en *= gint(inc_ret,local_vec(inc_ret));
            for(i=1; i<=max_vec.get_dim(); i++){
                /*
                curr_en -= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en += ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
                */
                curr_en /= ww(inc_ret,local_vec(inc_ret)-1,i,local_vec(i));
                curr_en *= ww(inc_ret,local_vec(inc_ret),i,local_vec(i));
            };
           // cout << curr_en << " ";
        }else{
            start_flag = 0;
            curr_en = 1;
        for(i=1; i<=max_vec.get_dim(); i++){
            //curr_en += gint(i,local_vec(i));
            curr_en *= gint(i,local_vec(i));
         //   cout << local_vec(i) << " ";
            for(j=i+1; j<=local_vec.get_dim(); j++){
                //curr_en += ww(i,local_vec(i),j,local_vec(j));
                curr_en *= ww(i,local_vec(i),j,local_vec(j));
            };
        };
     //   cout << " " << curr_en << endl;
        };
        p=(curr_en)/result.Z;
        entrop += -(util_R*util_j2kcal) * p * log(p);
    };
    //cout << "Partition function calculated " << Z << endl;
    result.entropy = entrop;
}

ostream& operator<<(ostream &output, boltzmann_energies &en){
    output << "Partition funktion: " << en.Z << " kcal/mol" << endl;
    output << "Mean energy: " << en.mean_energy << " kcal/mol" << endl;
    output << "Maximal Energy: " << en.max_energy << " kcal/mol" << endl;
    output << "Minimal Energy: " << en.min_energy << " kcal/mol" << endl;
    output << "Entropy: " << en.entropy << " kcal/(K*mol)" << endl;
    output << "Entropy at 300K: " << en.entropy*300 << " kcal/(K*mol)" << endl;
    return output;
}

void get_energy_histo(matrix<float> &gint, d4_tensor<float> &ww, vec<int> &max_vec, vec<int> &min_vec, int target, int target_start, int target_stop, double min, double max, double step, ofstream &output){
    double curr_en;
    int i,j, h, nsteps;
    vec<int> local_start_vec, local_max_vec, local_vec, histo_count;
    vec<double> histo_steps;
    local_start_vec.init_arr(1,max_vec.get_dim());
    local_max_vec.init_arr(1,max_vec.get_dim());
    local_vec.init_arr(1,max_vec.get_dim());
    nsteps = (int)ceil((ceil(max-min)/step));
    histo_steps.init_arr(1,nsteps);
    histo_count.init_arr(1,nsteps);
    for(int i=1; i<=max_vec.get_dim(); i++){
        local_vec(i) = min_vec(i);
        local_max_vec(i) = max_vec(i);
        local_start_vec(i) = min_vec(i);
    };
    for(int i=1; i<=nsteps; i++){
        histo_steps(i) = min + (i-1)*step;
        histo_count(i) = 0;
    };
    local_max_vec(target) = target_stop;
    local_start_vec(target) = target_start;
    local_vec(target) = target_start;

    //cout << "Calculating partition function for " << nstates << " states" << endl;
    while(increase_protstate(local_vec, max_vec.get_dim(), local_max_vec, target, local_start_vec, target_start)){
        curr_en = 0;
        for(i=1; i<=max_vec.get_dim(); i++){
            curr_en += gint(i,local_vec(i));
         //   cout << local_vec(i) << " ";
            for(j=i+1; j<=local_vec.get_dim(); j++){
                curr_en += ww(i,local_vec(i),j,local_vec(j));
            };
        };
        h = 1;
        while(h <= nsteps){
            if(curr_en < histo_steps(h)){break;}
            h++;
        }
        histo_count(h) += 1;
     //   cout << " " << curr_en << endl;
    };

    for(int k = 1; k<=nsteps; k++){
        output << histo_steps(k) << "   " << histo_count(k) << endl;
    };
}



int increase_protstate(vec<int> &prot, int nsites, vec<int> &nin, int target, vec<int> &start_vec, int target_start)
{
    int j, ret;
    ret = 1;
    for (j=1; j<=nsites; j++){
        if (prot(j)<nin(j)){
            prot(j)++;
            ret<=nsites?ret=j:ret=ret;
            return ret; /* valid state */
        }else {
            ret = nsites+1;
            if(j != target){
                prot(j)=start_vec(j);
            }else{
                prot(j)=target_start;
            };
        }
    }
    return 0; /* no more states */
}

/** Increase a state vector and return what we have increased

Pimp my increase_protstate. This function increases the state vector and returns what we have increased. This enables the calling function to calculate energy differences instead of the whole energy. Save ages of time so.
\param prot The state vector
\param nsites The number of sites in prot
\param nin The max vector. Sets for each site where have to stop the increment
\param start_vector The min vector. Sets for each site where we are going back after we have increased to the max value.
*/
int increase_protstate(vec<int> &prot, int nsites, vec<int> &nin, vec<int> &start_vec)
{
    int j, ret;
    ret = 1;
    for (j=1; j<=nsites; j++){
        if (prot(j)<nin(j)){
            prot(j)++;
            ret<=nsites?ret=j:ret=ret;
            return ret; /* valid state return site we have changed*/
        }else {
            ret = nsites + 1;
                prot(j)=start_vec(j);
        }
    }
    return 0; /* no more states */
}
