//
// C++ Interface: file_io
//
// Description:
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef FILE_IO_
#define FILE_IO_
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <math.h>
#include <cstring>
#include "vector.h"
#include "matrix.h"
#include "tensor.h"
#include "4d_tensor.h"
#include "gzstream.h"



using namespace std;



class atom{
    public:
        atom(): name("DUM"), resname("DUM"), resnum(1), number(0), is_heavy(0), kind(0), is_bb(0), radius(0), charge(0), x(0), y(0), z(0){};
        void set_atom(int n, int k, float r, float c, float x1, float y1, float z1, string &set_name, string &rname, int rnum);
        atom(const atom &tmp_at);
        string name;
        string resname;
        int resnum;
        int number;
        char is_heavy;
        int kind; // 0 for neutral, -1 for h-bond acceptor, +1 for donor, -2 means lone pair, since the distance is shorter!
        int is_bb;
        float radius;
        float charge;
        float x,y,z;
        friend ostream& operator<<(ostream &out, atom &at);
    private:
};

class atom_file{
    public:
	int natoms;
	vec<atom> atoms;
	atom_file(string filename);
};

class instance{
    public:
        instance(): number(0), pot(0){};
        void set_inst(const string &set_label, const int site, const int n, const int natom);
        instance(const instance &tmp_inst);
        string label;
        int number;
        vec<atom> atoms;
        int isprot;
        int isel;
        float pot; /**< \todo move that out of here! */
};

class residue{
    public:

        residue(): number(0), ninst(0), natoms(0), ndonor(0), nacceptor(0), nrotation_groups(0), is_in_sites(0){};
        residue(residue& tmp_res);
        void set_res(const string &na, const int num, const int nins, const int nat, const int ndon, const int naccept);
        int delete_atom(int num);
        ~residue(){
            atoms.kill_arr();
            inst.kill_arr();
        };
        int number;
        int ninst;
        int natoms;
        int ndonor;
        int nacceptor;
        int nrotation_groups;
        int is_tit;
        int max_nprot;
        int min_nprot;
        int is_in_sites;
        matrix<int> rotations; // Number of rotations per group is saved in rotations[group][0]
        double pot;
        string name;
        vec<atom> atoms;
        vec<instance> inst;
        vec<int> con;
        matrix<string> con_atoms;
        residue& operator=(const residue &tmp_res);
    private:
};

class don_acc_list{
    public:
        don_acc_list(){};
        don_acc_list(char *filename);
        matrix<char> donor;
        matrix<char> acc;
        matrix<char> lp;
        int ndonor;
        int nacceptor;
        int nlp;

};

class prot{
    public:
        prot(){};
        prot(char *filename, don_acc_list & don_acc);
        prot(char *filename);
        prot(const prot &tmp_prot);
        void set_prot(int nr){nres = nr; res.init_arr(1,nr); cout << "nr " << nr << endl;};
        void print_as_pdb(ostream& outp);
        vec<residue> res;
        string name;
        vec<string> tit_res;
        int nres;
};

class vis{
    // Store information, in wich instances which atoms are visible as protons.
    public:
    int ninst;
    int natoms;
    matrix<char> names;
    matrix<int> visible;

    vis(char *filename, residue& res);
    ~vis();

};

class siterep_site{
    public:
        int ninst;
	int natoms;
        vec<int> nacc;
        vec<int> ndon;
        vec<int> nlp;
        int nheavy;
        int max_nprot;
        int min_nprot;
        int max_nel;
        string name;
        vec<int> prot;
        vec<int> el;
        matrix<string> acc;
        matrix<string> don;
        matrix<string> lp;
        vec<string> heavy;
        vec<string> labels;
	vec<string> atoms;
	matrix<string> atom_change;
        siterep_site();
        void set_site(int nin, int max_natoms);
        int find_inst_num(string &st);
};

class siterep{
    public:
        siterep(ifstream &file);
        siterep(ifstream &file, int t);
        int titsites;
        vec<siterep_site> sites;
        vec<int> ninst;
        int max_n_prot;
        int max_n_el;
        friend ostream &operator<<(ostream &out,siterep &sr);
};

int get_ninst(char *filename, const char *molname);
int add_instances(siterep &sr, residue &res);
int read_connect(char *filename,prot &ga);
void read_rotations(char *filename, prot &prot);
void read_sites(const char *filename, vec<string> &tit_sites, prot *whole_prot);
int number_of_lines(ifstream &file);

/** Reading the interaction energies without restrictions like in gmct read functions (new)
    \param filename The file name as charakter array
    \param sys Pointer to the system parameter structure
    \see System
 */
int read_gint(char *filename, vec<string> &resname, matrix<string> &instlabel, matrix<double> &gi);
int read_gint(char *filename, vec<string> &resname, matrix<string> &instlabel, matrix<float> &gi);
/** Reading the intrinsic energies without reastrictions like in gmct read functions (new)
    \param filename The file name as charakter array
    \param sys Pointer to the system parameter structure
    \see Systemcout << "fertig" << endl;
 */
void read_w(char *filename, vec<string> &resname, matrix<string> &instlabel, d4_tensor<float> &ints);
int get_nsites_and_names_and_labels_from_gint(char *filename, vec<string> &resname, matrix<string> &labels, char ad_EC_CP = 0);
int get_nsites_and_names_and_labels_from_gint(igzstream &file, vec<string> &names, matrix<string> &labels);
istream &my_get_line(stringstream &to_fill, istream &file);

int cut_string_begin(string &st);
int read_col(const string &st, string &out);
double calc_err(vec<int> &val, int n);
double calc_err(vec<double> &val, int n);
void write_line_pdb(ostream &out, string atom, int atomnum, string atomlabel, string reslabel, int resid, double x, double y, double z, double beta, double occup);
void write_line_pqr(ostream &out, string atom, int atomnum, string atomlabel, string reslabel, int resid, double x, double y, double z, double charge, double radius);
#endif //FILE_IO_
