//
// C++ Interface: lists
//
// Description:
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef LISTS_H
#define LISTS_H
#include <iostream>
#include "mempool.h"
using namespace std;
class list_arg{
    public:
        list_arg *next;
        list_arg *prev;
        int n;
        list_arg():next(NULL),prev(NULL),n(0){};
        list_arg(list_arg *n, list_arg *p):next(n),prev(p),n(0){};
        friend bool operator==(const list_arg &l1, const list_arg &l2);
        friend bool operator==(const int l1, const list_arg &l2);
        friend bool operator==(const list_arg &l2, const int l1);
};

/** Template calss for a single linked list
	@author Mirco Till <Mirco.Till@uni-bayreuth.de>
 */
template<class T>
class single_LL{
public:
    T* head;
    T* tail;
    single_LL():head(NULL),tail(NULL){};
    T* add(){
        if(head == NULL){
          try{
            head = new T;
          }catch(bad_alloc){
            cout << "Unable to allocate element in single_LL first element" << endl;
            exit(0);
          };
            tail = head;
            (*head).next = NULL;
        }else{
          try{
              (*tail).next = new T;
          }catch(bad_alloc){
              cout << "Unable to allocate element in single_LL following elements" << endl;
              exit(0);
            };
            tail = (*tail).next;
            (*tail).next = NULL;
        }
        return tail;
    }
    T* search(T &search_el){
        T* iter = head;
        while(iter != NULL){
            if(*iter == search_el){
                return iter;
            };
            iter = iter->next;
        };
        return NULL;
    }
    T* search(int &search_el){
        T* iter = head;
        while(iter != NULL){
            if(*iter == search_el){
                return iter;
            };
            iter = iter->next;
        };
        return NULL;
    }
    ~single_LL(){
        T* to_delete;
        while(head != NULL){
            to_delete = head;
            head = (*head).next;
            delete to_delete;
        };
        head = tail = NULL;
    };
};

/** Doubly linked list
    The list argument T needs two pointers to itself, called 'next' and 'prev'; The constructor has to take two arguments to set the next and the prev poitner for example T(T* n, T* p):next(n),prev(p)
	@author Mirco Till <Mirco.Till@uni-bayreuth.de>
*/
template<class T>
class double_LL{
public:
    T* head;
    T* tail;
    double_LL():head(NULL),tail(NULL){};
    T* add(){
        if(head == NULL){
          try{
            head = new T;
        }catch(bad_alloc){
          cout << "Unable to allocate element in double_LL first element" << endl;
          exit(0);
        };
            tail = head;
        }else{
          try{
            (*tail).next = new T(NULL,tail);
        }catch(bad_alloc){
          cout << "Unable to allocate element in double_LL following elements" << endl;
          exit(0);
        };
            tail = (*tail).next;
        }
        return tail;
    }
    T *get_last(){
        T* to_return = tail;
        if(tail != NULL && (*tail).prev != NULL){
            (*tail).prev->next = NULL;
            tail = (*tail).prev;
        }else if((*tail).prev == NULL){// Last element
            head = tail = NULL;
        };
        return to_return;
    };
    T *get_first(){
        T* to_return = head;
        if(head != NULL && (*head).next != NULL){
            (*head).next->prev = NULL;
            head = (*head).next;
        }else if((*head).next == NULL){// Last element
            head = tail = NULL;
        };
        return to_return;
    };
    ~double_LL(){
        T* to_delete, *iter;
        iter = head;
        while(iter != NULL){
            to_delete = iter;
            iter= (*iter).next;
            delete to_delete;
        };
        //delete head;
        head = tail = NULL;
    };
};

template<class T>
class FIFO{
     private:
        struct node{
            T *item;
            node *next;
            node(){};
            node(T *x){item = x; next=NULL;}
        };
        node *head, *tail;
       mempool<node> pool;
       int nelements;
    public:
        FIFO():head(NULL),nelements(0){}
        bool empty() const {return head == NULL;}
        void add(T *x){
            node *t = tail;
            tail = pool.get_el();
            tail->next = NULL;
            tail->item = x;
            if(head == NULL){
                head = tail;
            }else{t->next = tail;}
            nelements++;
        }
        T* get(){
	  if(head == NULL){return NULL;}
            T *v = head->item;
            node *t = head->next;
            head->item = NULL;
            pool.return_el(head);
            head = t;
            nelements--;
            return v;
        }
        double size(){return sizeof(FIFO) + pool.size();}
        int nel(){return nelements;}
        void clear(){
          T *iter;
          iter = get();
          while(iter != NULL){
            delete iter;
            iter = get();
          };
        }
        ~FIFO(){
          clear();
        };
};

template<class T>
    class cycle_FIFO{
      private:
        struct node{
          T *item;
          node *next;
          node(){};
          node(T *x){item = x; next=NULL;}
        };
        node *head, *tail, *cycle;
        mempool<node> pool;
        int nelements;
      public:
        cycle_FIFO():head(NULL),tail(NULL),cycle(NULL), nelements(0){}
        bool empty() const {return head == NULL;}
        void add(T *x){
          node *t = tail;
          tail = pool.get_el();
          tail->next = head;
          tail->item = x;
          if(head == NULL){
            head = tail;
            tail->next = NULL;
          }else{t->next = tail;}
          nelements++;
        }
        T* get(){
          if(head == NULL){return NULL;}
          T *v = head->item;
          node *t = head->next;
          head->item = NULL;
          pool.return_el(head);
          head = t;
          tail->next = head;
          if(tail == tail->next){tail->next = NULL;}
          nelements--;
          return v;
        };

        T* cycle_increase(){
          if(cycle==NULL){
            cycle = head;
          }else{
            cycle = cycle->next;
          };
          return cycle->item;
        };
        void cycle_reset(){
          cycle = NULL;
        };
        double size(){return sizeof(cycle_FIFO) + pool.size();}
        int nel(){return nelements;}
        void clear(){
          T *iter;
          //make a linear FIFO
          if(tail == NULL){return;}
          tail->next = NULL;
          iter = get();
          while(iter != NULL){
            delete iter;
            iter = get();
          };
        }
        ~cycle_FIFO(){
          clear();
        };
    };

#endif
