#ifndef MATRIX_
#define MATRIX_
#include <iostream>
#include <iomanip>
#include <typeinfo>

using namespace std;
/** Matrix template

  Template class for a one dimensional vector. Cares about all the memory allocation things and initialisatio if asked. <br>
  Any given range is possible, no need to start at 0!<br>
  Examples: Interger vector:<br>
  matrix<int> example; followed by example.init_arr(1,100,1,100) gives you an interger matrix from 1 to 100 and 1 to 100 with 10000 elemets. <br>
  matrix<int> eample(1,100,1,100); Same matrix as above;<br>
  matrix<int> example; followed by example.init_arr(1,100,1,100,0); Same as above, but all elements initialised to 0;<br>
  matrix<int> example; followed by example.init_arr(1,100,vec<int>); Give you a spare matrix with vec(i) elemetns in each row.<br>
  example(1,1) = 1; Assignes 1 to the first element in the first dimension.<br>

*/
template<class T>
class matrix {
  private:
    int dim1;  /**< Number of rows */
    int dim2; /**< Number of columns */
    short start1; /**< Starting index of the first dimension */
    short start2; /**< Starting index of the second dimension */
    int *spare; /**< Starting index of the second dimension */
    float space; /**< Memory usage */
  public:
    T **arr; /**< The array itself */
    /** Constructor without allocation and initialisation

    initial has to be called for allocating memory
    */
    matrix(): dim1(0), dim2(0), start1(0), start2(0), spare(NULL), space(sizeof(matrix)), arr(NULL) {
    };
    /** Constructor with allocation but without initialisation

        \param d1 Starting index for rows
        \param d1_2 Last index for rows
        \param d2 Starting index for columns
        \param d2_2 Last index for columns

    */
    matrix(int d1,int d1_2,int d2,int d2_2): dim1(0), dim2(0), start1(0), start2(0), spare(NULL), space(sizeof(matrix)), arr(NULL) {
      init_arr(d1,d1_2,d2,d2_2);
    }
    /** Constructor with allocation and initialisation

        \param d1 Starting index for rows
        \param d1_2 Last index for rows
        \param d2 Starting index for columns
        \param d2_2 Last index for columns
        \param ini Initialisation value
    */
    matrix(int d1,int d1_2,int d2,int d2_2, T ini): dim1(0), dim2(0), start1(0), start2(0), spare(NULL), space(sizeof(matrix)), arr(NULL) {
      init_arr(d1,d1_2,d2,d2_2,ini);
    };
    /** Constructor with allocation and initialisation for a sparse matrix
    \deprecated DO NOT USE THIS!

    */
    matrix(int d1,int d1_2,T ini, int *sizes): dim1(0), dim2(0), start1(0), start2(0), spare(NULL), space(sizeof(matrix)), arr(NULL) {
      spare = new int[d1_2 - d1 + 2];
      init_arr(d1,d1_2, sizes, ini);
      space += sizeof(int)*(d1_2 - d1 + 2);
    };
    /** Copyconstructor

        \param &ini_mat Reference matrix for copying
    */
    matrix(const matrix<T> &ini_mat): dim1(0), dim2(0), start1(0), start2(0), spare(NULL), space(sizeof(matrix)), arr(NULL) {
      init_arr(ini_mat);
    }
    /** Initialise matrix from another matrix

        Takes the start and dimension values to calcualted the size of the array. Memory is requested and the new arry is initialised with the old values.
        \param &ini_mat Reference matrix for copying
    */
    void init_arr(const matrix<T> &ini_mat) {
      init_arr(ini_mat.get_start1(),ini_mat.get_dim1() + ini_mat.get_start1() - 1
               ,ini_mat.get_start2(),ini_mat.get_dim2() + ini_mat.get_start2() - 1);
      for (int i=ini_mat.get_start1(); i<= ini_mat.get_dim1() + ini_mat.get_start1() - 1; i++) {
        for (int j=ini_mat.get_start2(); j<=ini_mat.get_dim2() + ini_mat.get_start2() - 1; j++) {
          arr[i][j] = ini_mat(i,j);
        };
      };
    };
    /** Initialise a dens matrix

         Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
         For fast memory allocation the elemetns for all columns are allocated at once. Reduces new calls.
         \param x Starting index for rows
         \param x2 Last index for rows
         \param y Starting index for columns
         \param y2 Last index for columns
    */

    void init_arr(int x, int x2, int y, int y2) {
      kill_arr(); // prevent memory leaks
      dim1 = x2 - x +1;
      dim2 = y2 - y +1;
      start1 = x;
      start2 = y;
      if (dim1 <=0){dim1 = 1; x2 = x;}
      try {
        arr = new T*[dim1];
        arr = arr - start1;
        space += dim1 * sizeof(T*);
        arr[start1] = new T[dim1*dim2+1];
        arr[start1] -= y;
        space += (dim1*dim2+1)*sizeof(T);
        for (int i=start1+1;i<=x2;++i) {
          arr[i] = arr[i-1]+dim2;
          //arr[i] = arr[i] - start2;
        };
      } catch (bad_alloc) {
        cout << "Unable to allocate array with dims " << dim1 << " " << dim2 << endl;
        exit(0);
      };
    };
    /** Initialise a dens matrix and values in it.

        Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
        For fast memory allocation the elemetns for all columns are allocated at once. Reduces new calls.
        Afterwars all values are set to ini
        \param x Starting index for rows
        \param x2 Last index for rows
        \param y Starting index for columns
        \param y2 Last index for columns
        \param ini Initialisation value

    */
    void init_arr(int x, int x2, int y, int y2, T ini) {
      init_arr(x,x2,y,y2);
      for (int i=x; i<= x2; i++) {
        for (int j=y; j<= y2; j++) {
          arr[i][j] = ini;
        }
      }
    };
    /** Initialise a sparse matrix.

        Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
        Second dimension are allocated row by row. Much slower than allocating a block, but enables us to
        allocated only as many memory as necessary. Number of values in the second dimension is stored for convenience.
        \param x Starting index for rows
        \param x2 Last index for rows
        \param &sizes Vector stroing the information about the second dimension

    */
    void init_arr(int x, int x2,vec<int> &sizes) {
      kill_arr(); //prevent memory leaks
      dim1 = x2 - x +1;
      dim2 = 0;
      start1 = x;
      start2 = 1;
      if (dim1 <=0){dim1 = 1; x2 = x;}
      spare = new int[x2 - x + 2];
      space += sizeof(int)*(x2 - x + 2);
      for (int i=x; i<= x2; i++) {
        spare[i] = sizes(i);
      };
      try {
        arr = new T*[dim1];
        arr = arr - start1;
        space += sizeof(T*)*dim1;
        for (int i=start1;i<=start1 + dim1 -1;++i) {
          arr[i] = new T[sizes(i)+1];
          //arr[i][0] = sizes(i);
          space += sizeof(T)*(sizes(i)+1);
        };

      } catch (bad_alloc) {
        exit(0);
      };
    };
    /** Initialise a sparse matrix and values in it.

        Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
        Second dimension are allocated row by row. Much slower than allocating a block, but enables us to
        allocated only as many memory as necessary. number of values in the second dimension is stored for convenience.
        All values are initialised to ini
        \param x Starting index for rows
        \param x2 Last index for rows
        \param &sizes Vector stroing the information about the second dimension
        \param ini Initialisation value
    */
    void init_arr(int x, int x2,vec<int> &sizes, T ini) {
      init_arr(x,x2,sizes);
      for (int i=x; i<=x2;i++) {
        for (int j=1; j<=sizes(i);j++) {
          arr[i][j] = ini;
        };
      };
    }
    /** Care about interger vectors

        Only for convenience. init_arr can be called with (int, int, vec<int>) or with (int, int, int[]) which calls this funktion.
        Converges int[] to vsc<int> and calls init_arr(int, int, vec<int>
        \param x Starting index for rows
        \param x2 Last index for rows
        \param *sizes Array stroing the information about the second dimension
    */
    void init_arr(int x, int x2,int *sizes) {
      vec<int> tmp_vec(x,x2);
      for (int i=x; i<=x2; i++) {
        tmp_vec(i) = sizes[i];
      };
      init_arr(x,x2,tmp_vec);
    };
    /** Care about interger vectors

        Only for convenience. init_arr can be called with (int, int, vec<int>, T) or with (int, int, int[], T) which calls this funktion.
        Converges int[] to vsc<int> and calls init_arr(int, int, vec<int>
        Afterwards all values are initialised to ini
        \param x Starting index for rows
        \param x2 Last index for rows
        \param *sizes Array stroing the information about the second dimension
        \param ini Initialisation value
    */
    void init_arr(int x, int x2,int *sizes, T ini) {
      vec<int> tmp_vec(x,x2);
      for (int i=x; i<=x2; i++) {
        tmp_vec(i) = sizes[i];
      };
      init_arr(x,x2,tmp_vec, ini);
    };
    /** Adding a row to the matrix

        Allocates a new matrix with one row more than the old one and copies all values. Only possible for dens Matrix.
        Very slow!
    */
    void add_item() {
      if (arr && spare == NULL) {
        T **tmp_arr;
        try {
          tmp_arr = new T*[dim1+1];
          tmp_arr = tmp_arr - start1;
          tmp_arr[start1] = new T[((dim1+1)*dim2)+1];
          for (int i=start1+1;i<=start1 + dim1;++i) {
            tmp_arr[i] = tmp_arr[i-1]+dim2;
          };
        } catch (bad_alloc) {
          exit(0);
        };
        for (int i=start1; i<=start1 + dim1 - 1; i++) {
          for (int j=start2; j<=start2 + dim2 - 1; j++) {
            tmp_arr[i][j] = arr[i][j];
          };
        };
        kill_arr();
        arr = tmp_arr;
        for (int i=start1; i<=dim1 + 1; i++) {
          arr[i] = tmp_arr[i];
        };
        tmp_arr = 0;
        dim1 = dim1 + 1;
      }
    };
    /** Number of rows */
    int get_dim1() const {return dim1;}
    /** Number of columns

        If matrix is initialised as a sparse matrix, return 0, otherwise returns number of columns
    */
    int get_dim2() const {
      if (spare == NULL) {
        return dim2;
      } else {
        return 0;
      };
    };
    /** Starting index of dimension 1 */
    int get_start1() const {return start1;};
    /** Starting index of dimension 2 */
    int get_start2() const {return start2;};
    /** Number of columns in a spare matrix

        \param x Row index the dimension is asked for
    */
    int get_spare_dim(int x) {
      if (spare == NULL) {
        return dim2;
      } else {
        return spare[x];
      }
    };
    /** Memory usage */
    double get_space(){return space;}

    double get_sum(int row) {
      T sum = 0;
      for (int i=start2; i<= dim2 + start2 -1; i++) {
        sum += arr[row][i];
      }
      return sum;
    }

    int is_inside(int i, int s, int up_to = -1) {
      if (up_to == -1 || up_to > dim2 +start2 - 1) {
        up_to = dim2 +start2 - 1;
      };
      if (dim2 > 0) {
        for (int j=1; j<=up_to; j++) {
          if (arr[i][j] == s){return 1;};
        };
      };
      return 0;
    }
    /** Free memory

        Frees memory for both, dens and sparse matrices. Since the allocation is differen (second dimenson in one block or not)
        we have to free them in different ways.
    */
    void kill_arr() {
      if (arr != NULL) {
        if (dim2 == 0) {
          for (int i=start1;i<=start1 + dim1 -1;++i) {
            if (dim2 > 0) {
              arr[i] = arr[i] + start2;
              space -= dim2*sizeof(T);
            } else {
              space -= (spare[i]+1) * sizeof(T);
            }
            delete [] arr[i];


          };
          arr = arr + start1;
          delete [] arr;
          delete [] spare;
          space -= dim1*sizeof(T*);
          arr = NULL;
        } else {
          arr[start1] += start2;
          delete [] arr[start1];
          space -= dim1*dim2*sizeof(T);
          arr = arr + start1;
          delete [] arr;
          space -= dim1*sizeof(T*);
          arr = NULL;
        }
      };
    };
    /** Destructor

        Just call kill_arr()
    */
    ~matrix() {
      if (arr && dim1 > 0) {
        kill_arr();
      };
    };
    /** Index operator

         Overloaded () operator for easy way of indexing
         \param x Row index
         \param y Column index
    */
    T &operator()(int x, int y) const {
      return arr[x][y];
    };
    /** Index operator

          Overloaded () operator for easy way of indexing
          Returns a pointer to a row. useful if we have stored chars and wana handle them as strings.
          But using a vec<string> is better!
          \param Row index
    */
    T *operator()(int x)const {
      return arr[x];

    };
    /** Output operator

        \todo make the output much better looking
    */
    friend ostream& operator<<(ostream &output, matrix &matrix) {
      if (matrix.get_dim2() > 0) {
        for (int i=matrix.start1;i<=matrix.start1+matrix.dim1-1;i++) {
          for (int j=matrix.start2;j<=matrix.start2+matrix.dim2-1;j++) {
            cout << setw(10) << fixed << setprecision(2) << matrix(i,j) << " ";
          }
          cout << endl;
        }
      } else if (matrix.dim2 == 0) {
        for (int i=matrix.start1;i<=matrix.start1 + matrix.dim1 - 1;i++) {
          for (int j=1;j<=matrix.get_spare_dim(i);j++) {
            cout << fixed << setprecision(2) << matrix(i,j) << " ";
          }
          cout << endl;
        }
      }
      return output;
    }
    /** Asignment operator

        Assigns all values of the rhs matrix to the lhs matrix. Only working for dens matrices. Checks which matrix is samller
        and assigns only these values.
        \todo Make it working for sparse matrix
        \todo Reallocate memory if necessary
        \param &ref Reference array
    */
    matrix<T>& operator=(const matrix<T> &ref) {
      // only copy the smaller one
      int d1,d2;
      (dim1 + start1 -1) < (ref.dim1 + ref.start1 -1) ?  d1 = (dim1 + start1 -1) : d1 = (ref.dim1 + ref.start1 -1);
      (dim2 + start2 -1) < (ref.dim2 + ref.start2 -1) ?  d2 = (dim2 + start2 -1) : d2 = (ref.dim2 + ref.start2 -1);
      for (int i=start1; i <= d1; i++) {
        for (int j=start2; j<=d2; j++) {
          arr[i][j] = ref.arr[i][j];
        };
      };
      return *this;
    };
};


#endif
