#ifndef MEMPOOL_H
#define MEMPOOL_H
#include "vector.h"
using namespace std;

/**
	@author Mirco Till <Mirco.Till@uni-bayreuth.de>
*/
template<class T>
class mempool {
  private:
    int block_size;
    T *free_list;
    vec<T*> block_p;
  public:
    mempool() :block_size(1000), free_list(NULL) {
      block_p.init_arr(1,1);
      block_p(1) = NULL;
    }
    mempool(int block_s) :block_size(block_s), free_list(NULL) {
      block_p.init_arr(1,1);
      block_p(1) = NULL;
    }
    ~mempool() {
      for (int i=1; i<=block_p.get_dim();i++) {
        delete [] block_p(i);
      }
    };
    void set_bs(int s) {block_size = s;};
    int get_nblocks() {return block_p.get_dim();}
    T *get_el() {
      T *list_el, *new_block;
      list_el = free_list;
      if (list_el != NULL) {
        free_list = free_list->next;
      } else {
        try {
          new_block = new T[block_size];
        } catch (bad_alloc) {
          cerr << "unable to allocate memory for " << block_size << " blocks! Exiting! " << endl; exit(0);
        }
        block_p.add_item();
        block_p(block_p.get_dim()) = new_block;
        for (int i=1; i<block_size -1; i++) {
          new_block[i].next = &new_block[i+1];
        };
        new_block[block_size-1].next = NULL;
        free_list = &new_block[1];
        list_el = new_block;
      }
      return list_el;
    }
    void return_el(T *el) {
      if (el != NULL) {
        el->next = free_list;
        free_list = el;
      }
    }
    double size() {return block_size * block_p.get_dim() * sizeof(T);}
    double bs(){return block_size;}
};

#endif
