//
// C++ Interface: pq
//
// Description:
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef PQ_H
#define PQ_H
#include <vector>
#include "util.h"
/**
	@author Mirco Till <Mirco.Till@uni-bayreuth.de>
*/
template <class T>
class PQ{
  private:
    vec<T> &a;
    int* qp, *pq;
    int N;
public:
    PQ(int n, const vec<T> &a):a(a),N(0){
      pq = new T[n+1];
      qp = new int[n+1];
    };
    void fixUP(T a[], int k){
      while(k>1 && a[k/2] < a[k]){
        exch(a[k],a[k/2]);
        k=k/2;
      }
    };
    void fixdown(T a[], int k, int N){
      while(2*k <= N){
        int j= 2*k;
        if(j<N && a[j] < a[j+1]) j++;
        if(!(a[k] < a[j])) break;
        exch(a[k],a[j]);
        k=j;
      }
    };
    int less(int i, int j){return i<j;};
    int empty(){return N==0;};
    void exch(T i, T j){
      int t;
      t = qp[i]; qp[i] = qp[j]; qp[j] = t;
      pq[qp[i]] = i; pq[qp[j]] = j;
    }
    void insert(T v){
      pq[++N] = v;
      qp[v] = N;
      fixUP(pq,N);
    }

    T getmax(){
     exch(pq[1],pq[N]);
     fixdown(pq,1,N-1);
     return pq[N--];
    };
    T getmin(){
    };
    void change(T i){
     fixUP(pq,qp[i]);
     fixdown(pq,qp[i],N);
    };
    void print_PQ(){
      for(int i=0; i<=N; i++){
        cout << pq[i] << " ";
      };
      cout << endl;
    };
    void del(int i);
    ~PQ(){
      delete [] pq;
      delete [] qp;
    };
};
// Priority q smalles key as highest priority
template <class keyType> class PQi
{
  const vec<keyType> &a;
  vector<int> pq, qp;
  int N, d;

  void exch(int i, int j)
  { int t = pq[i]; pq[i] = pq[j]; pq[j] = t;
  qp[pq[i]] = i; qp[pq[j]] = j; }

  void fixUp(int k)
  { while (k > 1 && a(pq[(k+d-2)/d]) > a(pq[k]))
  { exch(k, (k+d-2)/d); k = (k+d-2)/d; } }

  void fixDown(int k, int N)
  { int j;
  while ((j = d*(k-1)+2) <= N)
  {
    for (int i = j+1; i < j+d && i <= N; i++)
      if (a(pq[j]) > a(pq[i])) j = i;
    if (!(a(pq[k]) > a(pq[j]))) break;
    exch(k, j); k = j;
  }
  }
  public:
  PQi(int N, const vec<keyType> &a, int d = 3) :
    a(a), pq(N+1, 0), qp(N+1, 0), N(0), d(d) { }
  int empty() const { return N == 0; }
  void insert(int v)
  { pq[++N] = v; qp[v] = N; fixUp(N); }
  int getmin()
  { exch(1, N); fixDown(1, N-1); return pq[N--]; }
  void lower(int k)
  { fixUp(qp[k]); }
  void print(){
  }
};

#endif
