//
// C++ Implementation: qmpb_energies
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qmpb_energies.h"


qmpb_gint::qmpb_gint(char *file, vec<string> &resname, matrix<string> &tmp_labels, int nsites, vec<int> &nin){
    gint.kill_arr();
    gint.init_arr(1,nsites,nin);
    gint_exp.kill_arr();
    gint_exp.init_arr(1,nsites,nin);
    names.kill_arr();
    names.init_arr(1,nsites);
    labels.kill_arr();
    labels.init_arr(1,nsites,nin);

    read_gint(file, resname, tmp_labels, gint);
    for(int i=1;i<=nsites;i++){
        names(i) = resname(i);
        for(int j=1; j<=tmp_labels.get_spare_dim(i); j++){
            labels(i,j) = tmp_labels(i,j);
        };
    };
    cout << "Space allocated for internal energies: " << get_size()/1E6  << " MB" <<  endl;
}

qmpb_gint::qmpb_gint(igzstream &bin_in){
    read_binary(bin_in);
}

int qmpb_gint::check_labels(qmpb_gint &gi1, qmpb_gint &gi2){
    if(gi1.names.get_dim() != gi2.names.get_dim()){
        cerr << "Number of sites is not identical within conformations" << endl;
    }
    for(int h1 = 1; h1<= gi1.names.get_dim(); h1++){
            if(gi1.names(h1) != gi2.names(h1)){
                return 0;
            };
            for(int h2 = 1; h2<= gi1.labels.get_spare_dim(h1); h2++){
                if(gi1.labels(h1,h2) != gi2.labels(h1,h2)){
                    return 0;
                };
            };
    };
    return 1;
}

qmpb_gint::~qmpb_gint(){

    gint.kill_arr();
    names.kill_arr();
    labels.kill_arr();
}

int qmpb_gint::write_binary(ogzstream &bin_out){
    int nsites = gint.get_dim1();
    float energy;
    short i,j;
    vec<int> ninst;
    char tmp_char[20]; // resnames should not be longer than 20 charakters
    for(int i=0; i<20; i++){
        tmp_char[i] = 0;
    };
    ninst.init_arr(1,nsites);
    for(int i=1; i<=nsites; i++){
        ninst(i) = gint.get_spare_dim(i);
    };
    bin_out.write(reinterpret_cast<char *>(&nsites),sizeof(int));
    ninst.write_binary(bin_out);
    for(int i=1; i<= nsites; i++){
        strcpy(tmp_char,names(i).c_str());
        bin_out.write(tmp_char, sizeof(char)*20);
    };

    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            strcpy(tmp_char,labels(i,j).c_str());
            bin_out.write(tmp_char, sizeof(char)*20);
        };
    };

    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            bin_out.write(reinterpret_cast<char *>(&i),sizeof(short));
            bin_out.write(reinterpret_cast<char *>(&j),sizeof(short));
            energy = gint(i,j);
            bin_out.write(reinterpret_cast<char *>(&energy),sizeof(float));
            //cout << "write: " << i << " " << j << " " << gint(i,j) << endl;
        }
    }


    bin_out.close();

    return 1;
}

int qmpb_gint::read_binary(igzstream &bin_in){
    int nsites = 0;
    short site, inst;
    short i,j;
    float energy;
    vec<int> ninst;
    char tmp_char[20];
    bin_in.read(reinterpret_cast<char *>(&nsites),sizeof(nsites));
    //cout << nsites << endl;
    ninst.init_arr(1,nsites);
    ninst.read_binary(bin_in);
    labels.kill_arr();
    labels.init_arr(1,nsites,ninst);
    gint.kill_arr();
    gint.init_arr(1,nsites,ninst);
    gint_exp.kill_arr();
    gint_exp.init_arr(1,nsites,ninst);
    names.kill_arr();
    names.init_arr(1,nsites);
    //cout << ninst << endl;

    for(i=1; i<= nsites; i++){
        bin_in.read(tmp_char,sizeof(char)*20);
        names(i) = tmp_char;
        //cout << names(i) << endl;
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            bin_in.read(tmp_char,sizeof(char)*20);
            labels(i,j) = tmp_char;
            //cout << labels(i,j) << endl;
        };
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            //cout << "reading " << i << " " << j << endl;
            bin_in.read(reinterpret_cast<char *>(&site),sizeof(short));
            bin_in.read(reinterpret_cast<char *>(&inst),sizeof(short));
            bin_in.read(reinterpret_cast<char *>(&energy),sizeof(float));
            gint(site,inst) = energy;
            //cout << names(site) << " " << labels(site,inst) << " " << energy << endl;
        };
    };
    return 1;
}
int qmpb_gint::read_binary(igzstream &bin_in, vec<int> &sys_ninst, vec<string> &sys_names, matrix<string> &sys_labels){
    int nsites = 0;
    short site, inst;
    short i,j;
    float energy;
    int flag;
    vec<int> ninst;
    char tmp_char[20];
    bin_in.read(reinterpret_cast<char *>(&nsites),sizeof(nsites));
    //cout << nsites << endl;
    ninst.init_arr(1,nsites);
    ninst.read_binary(bin_in);
    labels.kill_arr();
    labels.init_arr(1,nsites,ninst);
    gint.kill_arr();
    gint.init_arr(1,nsites,ninst);
    gint_exp.kill_arr();
    gint_exp.init_arr(1,nsites,ninst);
    names.kill_arr();
    names.init_arr(1,nsites);
    //cout << ninst << endl;

    for(i=1; i<= nsites; i++){
        bin_in.read(tmp_char,sizeof(char)*20);
        names(i) = tmp_char;
        //cout << names(i) << endl;
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            bin_in.read(tmp_char,sizeof(char)*20);
            labels(i,j) = tmp_char;
            //cout << labels(i,j) << endl;
        };
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            //cout << "reading " << i << " " << j << endl;
            bin_in.read(reinterpret_cast<char *>(&site),sizeof(short));
            bin_in.read(reinterpret_cast<char *>(&inst),sizeof(short));
            bin_in.read(reinterpret_cast<char *>(&energy),sizeof(float));

            //cout << names(site) << " " << labels(site,inst) << " " << energy << endl;
            // now we have to find the systemwide site and instance and safe the energy for that!
            // names and labels can differ
            for(int n=1; n<=sys_names.get_dim(); n++){
              if(sys_names(n) == names(site)){
                site = n;
                break;
              }
            }
            flag = 0;
            for(int n=1; n<=sys_names.get_dim(); n++){
              for(int in = 1; in <=sys_ninst(n); in++){
                if(sys_names(n) == names(site) && sys_labels(n,in) == labels(site,inst)){
                  inst = in;
                  flag = 1;
                  break;
                };
                if(flag) break;
              };
            };
            if(site && inst){
              gint(site,inst) = energy;
            };
        };
    };
    return 1;
}

void qmpb_gint::exp_with(double fac){
    for(int i=1; i<= gint.get_dim1(); i++){
        for(int j=1; j<= gint.get_spare_dim(i); j++){
           gint_exp(i,j) = exp(gint(i,j)/fac);
        };
    };
}

ostream& operator<<(ostream &output, qmpb_gint &gi){
    for(int i=1; i<= gi.gint.get_dim1(); i++){
        for(int j=1; j<= gi.gint.get_spare_dim(i); j++){
            output << gi.names(i) << " " << gi.labels(i,j) << " " << gi.gint(i,j) << endl;
        };
    };
    return output;
}


qmpb_w::qmpb_w(char *file, vec<string> &resname, matrix<string> &tmp_labels, int nsites, vec<int> &nin){
    matrix<int> tmp_matrix;
    tensor<int> tmp_tensor;
    tmp_matrix.init_arr(1,nsites,nin);
    for(int i=1; i<=nsites;i++){
        for(int j=1; j<= nin(i);j++){
            tmp_matrix(i,j) = nsites;
        };
    };
    tmp_tensor.init_arr(1,nsites,nin,tmp_matrix);
    for(int i=1; i<=nsites;i++){
        for(int j=1; j<= nin(i);j++){
            for(int k=1; k<=nsites;k++){
                tmp_tensor(i,j,k) = nin(k);
            };
        };
    };
    ww.kill_arr();
    ww.init_arr(1,nsites,nin,tmp_matrix, tmp_tensor);
    ww_exp.kill_arr();
    ww_exp.init_arr(1,nsites,nin,tmp_matrix, tmp_tensor);
    cutof_sites.init_arr(1,nsites,nin,tmp_matrix, tmp_tensor,0);
    cutof_sites_single.init_arr(1,nsites,1,nsites);
    cutof_sites_single_inst.init_arr(1,nsites,nin,tmp_matrix,0);
    names.kill_arr();
    names.init_arr(1,nsites);
    labels.kill_arr();
    labels.init_arr(1,nsites,nin);
    read_w(file, resname, tmp_labels, ww);

    for(int i=1;i<=nsites;i++){
        names(i) = resname(i);
        for(int j=1; j<=tmp_labels.get_spare_dim(i); j++){
            labels(i,j) = tmp_labels(i,j);
        };
    };
    cout << "Space allocated for interaction energies: " << get_size()/1E6  << " MB" <<  endl;
}

qmpb_w::qmpb_w(igzstream &bin_in){
    read_binary(bin_in);
}


int qmpb_w::write_binary(ogzstream &bin_out){
    int nsites = labels.get_dim1();
    float energy;
    short i,j,k,l;
    vec<int> ninst;
    char tmp_char[20]; // resnames should not be longer than 20 charakters
    for(int i=0; i<20; i++){
        tmp_char[i] = 0;
    };
    ninst.init_arr(1,nsites);
    for(int i=1; i<=nsites; i++){
        ninst(i) = labels.get_spare_dim(i);
    };
    bin_out.write(reinterpret_cast<char *>(&nsites),sizeof(int));
    ninst.write_binary(bin_out);
    for(int i=1; i<= nsites; i++){
        strcpy(tmp_char,names(i).c_str());
        bin_out.write(tmp_char, sizeof(char)*20);
    };

    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            strcpy(tmp_char,labels(i,j).c_str());
            bin_out.write(tmp_char, sizeof(char)*20);
        };
    };

    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            for(k=1; k<= nsites; k++){
                for(l=1; l<=labels.get_spare_dim(k); l++){
                bin_out.write(reinterpret_cast<char *>(&i),sizeof(short));
                bin_out.write(reinterpret_cast<char *>(&j),sizeof(short));
                bin_out.write(reinterpret_cast<char *>(&k),sizeof(short));
                bin_out.write(reinterpret_cast<char *>(&l),sizeof(short));
                energy = ww(i,j,k,l);
                bin_out.write(reinterpret_cast<char *>(&energy),sizeof(float));
                //cout << "write: " << i << " " << j << " " << k << " " << l << " " << ww(i,j,k,l) << endl;
                };
            };
        }
    }


    bin_out.close();

    return 1;
}

int qmpb_w::read_binary(igzstream &bin_in){
    int nsites = 0;
    short site, inst, site2, inst2;
    short i,j,k,l;
    float energy;
    vec<int> ninst;
    char tmp_char[20];
    bin_in.read(reinterpret_cast<char *>(&nsites),sizeof(nsites));
    //cout << nsites << endl;
    matrix<int> tmp_matrix;
    tensor<int> tmp_tensor;
    ninst.init_arr(1,nsites);
    ninst.read_binary(bin_in);
    tmp_matrix.init_arr(1,nsites,ninst);
    for(int i=1; i<=nsites;i++){
        for(int j=1; j<= ninst(i);j++){
            tmp_matrix(i,j) = nsites;
        };
    };
    tmp_tensor.init_arr(1,nsites,ninst,tmp_matrix);
    for(int i=1; i<=nsites;i++){
        for(int j=1; j<= ninst(i);j++){
            for(int k=1; k<=nsites;k++){
                tmp_tensor(i,j,k) = ninst(k);
            };
        };
    };
    names.kill_arr();
    names.init_arr(1,nsites);
    labels.kill_arr();
    labels.init_arr(1,nsites,ninst);
    ww.kill_arr();
    ww.init_arr(1,nsites,ninst, tmp_matrix, tmp_tensor);
    cutof_sites.init_arr(1,nsites,ninst,tmp_matrix, tmp_tensor,0);
    cutof_sites_single.init_arr(1,nsites,1,nsites);
    cutof_sites_single_inst.init_arr(1,nsites,ninst,tmp_matrix,0);
    ww_exp.kill_arr();
    ww_exp.init_arr(1,nsites,ninst, tmp_matrix, tmp_tensor);
    for(i=1; i<= nsites; i++){
        bin_in.read(tmp_char,sizeof(char)*20);
        names(i) = tmp_char;
        //cout << names(i) << endl;
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            bin_in.read(tmp_char,sizeof(char)*20);
            labels(i,j) = tmp_char;
            //cout << labels(i,j) << endl;
        };
    };
    for(i=1; i<= nsites; i++){
        for(j=1; j<=labels.get_spare_dim(i); j++){
            for(k=1; k<= nsites; k++){
                for(l=1; l<=labels.get_spare_dim(k); l++){
            //cout << "reading " << i << " " << j << endl;
                bin_in.read(reinterpret_cast<char *>(&site),sizeof(short));
                bin_in.read(reinterpret_cast<char *>(&inst),sizeof(short));
                bin_in.read(reinterpret_cast<char *>(&site2),sizeof(short));
                bin_in.read(reinterpret_cast<char *>(&inst2),sizeof(short));
                bin_in.read(reinterpret_cast<char *>(&energy),sizeof(float));
                ww(site,inst,site2,inst2) = energy;
                //cout << names(site) << " " << labels(site,inst) << " " << names(site2) << " " << labels(site2,inst2) << " "<< energy << endl;
                }
            };
        };
    };
    return 1;
}

int qmpb_w::read_binary(igzstream &bin_in, vec<int> &sys_ninst, vec<string> &sys_names, matrix<string> &sys_labels){
  int nsites = 0;
  short site, inst, site2, inst2;
  short i,j,k,l;
  float energy;
  int flag, labels_ok;
  vec<int> ninst;
  char tmp_char[20];
  bin_in.read(reinterpret_cast<char *>(&nsites),sizeof(nsites));
    //cout << nsites << endl;
  matrix<int> tmp_matrix;
  tensor<int> tmp_tensor;
  ninst.init_arr(1,nsites);
  ninst.read_binary(bin_in);
  tmp_matrix.init_arr(1,nsites,ninst);
  for(int i=1; i<=nsites;i++){
    for(int j=1; j<= ninst(i);j++){
      tmp_matrix(i,j) = nsites;
    };
  };
  tmp_tensor.init_arr(1,nsites,ninst,tmp_matrix);
  for(int i=1; i<=nsites;i++){
    for(int j=1; j<= ninst(i);j++){
      for(int k=1; k<=nsites;k++){
        tmp_tensor(i,j,k) = ninst(k);
      };
    };
  };
  names.kill_arr();
  names.init_arr(1,nsites);
  labels.kill_arr();
  labels.init_arr(1,nsites,ninst);
  ww.kill_arr();
  ww.init_arr(1,nsites,ninst, tmp_matrix, tmp_tensor);
  cutof_sites.init_arr(1,nsites,ninst,tmp_matrix, tmp_tensor,0);
  cutof_sites_single.init_arr(1,nsites,1,nsites);
  cutof_sites_single_inst.init_arr(1,nsites,ninst,tmp_matrix,0);
  ww_exp.kill_arr();
  ww_exp.init_arr(1,nsites,ninst, tmp_matrix, tmp_tensor);
  labels_ok = 1;
  for(i=1; i<= nsites; i++){
    bin_in.read(tmp_char,sizeof(char)*20);
    names(i) = tmp_char;
    if(!(names(i) == sys_names(i))){
      labels_ok = 0;
    };
        //cout << names(i) << endl;
  };
  for(i=1; i<= nsites; i++){
    for(j=1; j<=labels.get_spare_dim(i); j++){
      bin_in.read(tmp_char,sizeof(char)*20);
      labels(i,j) = tmp_char;
      if(!(labels(i,j) == sys_labels(i,j))){
        labels_ok = 0;
      };
            //cout << labels(i,j) << endl;
    };
  };
  for(i=1; i<= nsites; i++){
    for(j=1; j<=labels.get_spare_dim(i); j++){
      for(k=1; k<= nsites; k++){
        for(l=1; l<=labels.get_spare_dim(k); l++){
            //cout << "reading " << i << " " << j << endl;
          bin_in.read(reinterpret_cast<char *>(&site),sizeof(short));
          bin_in.read(reinterpret_cast<char *>(&inst),sizeof(short));
          bin_in.read(reinterpret_cast<char *>(&site2),sizeof(short));
          bin_in.read(reinterpret_cast<char *>(&inst2),sizeof(short));
          bin_in.read(reinterpret_cast<char *>(&energy),sizeof(float));

          //cout << names(site) << " " << labels(site,inst) << " " << names(site2) << " " << labels(site2,inst2) << " "<< energy << endl;
          if(labels_ok == 0){
            flag = 0;
            for(int n=1; n<=sys_names.get_dim(); n++){
              if(sys_names(n) == names(site)){
                for(int in = 1; in <=sys_ninst(n); in++){
                  if(sys_labels(n,in) == labels(site,inst)){
                    inst = in;
                    break;
                  };
                }
                site = n;
                flag++;
              }
              if(sys_names(n) == names(site2)){
                for(int in = 1; in <=sys_ninst(n); in++){
                  if(sys_labels(n,in) == labels(site2,inst2)){
                    inst2 = in;
                    break;
                  };
                }
                site2 = n;
                flag++;
              };
              if(flag == 2){ cout << "break at: " << n << endl;break;}
            }
          }
          ww(site,inst,site2,inst2) = energy;
        };
      };
    };
  };
  return 1;
}

void qmpb_w::exp_with(double fac){
    for(int i=1; i<= labels.get_dim1(); i++){
        for(int j=1; j<= labels.get_spare_dim(i); j++){
            for(int k=1; k<= labels.get_dim1(); k++){
                for(int l=1;l<= labels.get_spare_dim(k); l++){
                    ww_exp(i,j,k,l) = exp(ww(i,j,k,l)/fac);
                };
            };
        };
    };
}

void qmpb_w::create_cutof_list(double cutof, int blab){
    ww_cutof_el *iter, *list_iter;
    int break_flag, count;
    vec<int> tmp;
    double max_cutof, tmp_ww;
    int max_site;
    tmp.init_arr(1,names.get_dim());
    for(int i=1; i<= labels.get_dim1(); i++){
        tmp(i) = labels.get_spare_dim(i);
    };
    ww_cutof.kill_arr();
    ww_cutof.init_arr(1,names.get_dim(),tmp);
    w_cutof.init_arr(1,names.get_dim(),tmp);
    for(int i=1; i<= labels.get_dim1(); i++){
        for(int j=1; j<= tmp(i); j++){
            ww_cutof(i,j) = NULL;
        };
    };
    max_site = 0;
    max_cutof = 0;
    list_iter = NULL;
    for(int i=1; i<= labels.get_dim1(); i++){

        for(int j=1; j<= tmp(i); j++){
            count = 0;
            iter = ww_cutof(i,j);
            while(iter != NULL && (*iter).next != NULL){
                iter = (*iter).next;
            };
            for(int k=1; k<= labels.get_dim1(); k++){
                break_flag = 0;
                max_cutof = 0;
                max_site = 0;
                for(int l=1;l<= labels.get_spare_dim(k); l++){
                    ww(i,j,k,l)<0 ? tmp_ww = ww(i,j,k,l) * -1 : tmp_ww = ww(i,j,k,l);
                    if(i!=k && tmp_ww > cutof){
                        //cout << i << " " << j << " " << k << " " << l << " " << tmp_ww << " ";
			cutof_sites(i,j,k,l) = 1;
			cutof_sites(k,l,i,j) = 1;
			cutof_sites_single(i,k) = 1;
			cutof_sites_single(k,i) = 1;
			cutof_sites_single_inst(i,j,k) = 1;
			cutof_sites_single_inst(k,l,i) = 1;
                        if(break_flag == 0){
                          list_iter = w_cutof(i,j).add();
                          list_iter->site = k;
                        }
                        list_iter->instances.add_item();
                        list_iter->instances(list_iter->instances.get_dim()) = l;
                        break_flag = 1;
                    }else if(i!=k){
                        //rescue, if we dont find any sites
                        if(ww(i,j,k,l) < 0){tmp_ww = ww(i,j,k,l)*-1;}else{tmp_ww = ww(i,j,k,l);}
                        if(tmp_ww > max_cutof){max_cutof = tmp_ww; max_site = k;}
                    };
                };
            if(break_flag){
                if(iter != NULL){
                    (*iter).next = new ww_cutof_el;
                    (*(*iter).next).site = k;
                    iter = (*iter).next;
                    count++;
                    //cout << "Site added1: " << k << endl;
                }else{
                    iter = new ww_cutof_el;
                    (*iter).site = k;
                    ww_cutof(i,j) = iter;
                    count++;
                    //cout << "Site added2: " << k << endl;
                };
            };
        };
        if(blab > 2){
            cout << "Cutof sites for site " << i << " " << " inst " << j << " " << count << endl;
        };
        if(count == 0){
            cerr << "WARNING: no cutof sites found for site " << i << " instance " << j << endl;
            cerr << "Maximum interaction energy found: " << setprecision(5) << max_cutof << " with site " << max_site << endl;
            }
        };
    };
    create_cutof_list_site();
    // Set interaction energies below cutof to 0
    for(int i=1; i<= labels.get_dim1(); i++){
      for(int j=1; j<= tmp(i); j++){
        for(int k=1; k<= labels.get_dim1(); k++){
          for(int l=1;l<= labels.get_spare_dim(k); l++){
            ww(i,j,k,l)< 0 ? tmp_ww = ww(i,j,k,l) * -1 : tmp_ww = ww(i,j,k,l);
            if(i!=k && tmp_ww < cutof){
              ww(i,j,k,l) = 0;
	      ww(k,l,i,j) = 0;
            };
          };
        };
      };
    };

    /*
    // check consistency

    for(int i=1; i<= w_cutof.get_dim1(); i++){
      for(short int j=1; j<=w_cutof.get_spare_dim(i); j++){
        list_iter = w_cutof(i,j).head;
        while(list_iter != NULL){
          for(int k=1; k<= list_iter->instances.get_dim(); k++){
            iter = w_cutof(list_iter->site,list_iter->instances(k)).head;
            while(iter != NULL){
              if(iter->site == i && !iter->instances.is_inside(j)){
                cerr << "Cutof not consistent! " << endl;
                break;
              };
             iter = iter->next;
            }
          };
          list_iter = list_iter->next;
        };
      };
    };
    */
    /*
    for(int i=1; i<= w_cutof.get_dim1(); i++){
      for(int j=1; j<=w_cutof.get_spare_dim(i); j++){
        cout << "Site " << i << " instance " << j << endl;
        list_iter = w_cutof(i,j).head;
        while(list_iter != NULL){
          cout << "with site " << list_iter->site << " instances: ";
          for(int k=1; k<= list_iter->instances.get_dim(); k++){
            cout << list_iter->instances(k) << " ";
          };
          cout << endl;
          list_iter = list_iter->next;
        };
        list_iter = ww_cutof(i,j);
        while(list_iter != NULL){
          cout << "with site " << list_iter->site << endl;
          list_iter = list_iter->next;
        };
      };
    };
    */
}

void qmpb_w::print_cutof(){
  ww_cutof_el *list_iter;
  for(int i=1; i<= w_cutof.get_dim1(); i++){
    for(int j=1; j<=w_cutof.get_spare_dim(i); j++){
      cout << "Site " << i << " instance " << j << endl;
      list_iter = w_cutof(i,j).head;
      while(list_iter != NULL){
        cout << "with site " << list_iter->site << " instances: ";
        for(int k=1; k<= list_iter->instances.get_dim(); k++){
          cout << list_iter->instances(k) << " ";
        };
        cout << endl;
        list_iter = list_iter->next;
      };
    };
  };
}

double qmpb_w::calc_ww(int site, int conf, vec<short> &vect){
  static ww_cutof_el *iter;
  static double W;
  W = 0;
  iter = w_cutof(site, conf).head;
  //cout << "For Site " << site << " " << conf << endl;
  while(iter != NULL){
    //cout << (*iter).site << "(" << vect((*iter).site) << " " << ww(site,conf,(*iter).site,vect((*iter).site)) << ") ";
    W += ww(site,conf,(*iter).site,vect((*iter).site));
      ;
    iter = (*iter).next;
  };
  //cout << " " << W << endl;
  return W;
}

void qmpb_w::create_cutof_list_site(){
    ww_cutof_el *iter, *local_iter;
    vec<int> tmp;
    double max_cutof;
    int max_site;
    ww_cutof_site.kill_arr();
    ww_cutof_site.init_arr(1,labels.get_dim1());
    for(int i=1; i<= labels.get_dim1(); i++){
            ww_cutof_site(i) = NULL;
    };
    max_site = 0;
    max_cutof = 0;
    tmp.init_arr(1,labels.get_dim1(),0);
    for(int i=1; i<= labels.get_dim1(); i++){
        for(int k=1; k<=labels.get_dim1(); k++){
                tmp(k) = 0;
            };
        for(int j=1; j<= labels.get_spare_dim(i); j++){
            local_iter = ww_cutof(i,j);
            while(local_iter != NULL){
                tmp((*local_iter).site) = 1;
                local_iter = (*local_iter).next;
            };
        };
        for(int k=1; k<=labels.get_dim1(); k++){
            if(tmp(k) == 1){
                iter = ww_cutof_site(i);
                while(iter != NULL && (*iter).next != NULL){
                    iter = (*iter).next;
                };
                if(iter == NULL){
                    ww_cutof_site(i) = new ww_cutof_el;
                    ww_cutof_site(i)->site = k;
                }else{
                    (*iter).next = new ww_cutof_el;
                    (*(*iter).next).site = k;
                };
            };
        };
    };
    /*
    for(int k=1; k<=labels.get_dim1(); k++){
            iter = ww_cutof_site(k);
            cout << "for site " << k << endl;
            while(iter != NULL ){
                cout << (*iter).site << endl;
                iter = (*iter).next;
            };
    };
    */
}


ostream& operator<<(ostream &output, qmpb_w &w){
    for(int i=1; i<= w.labels.get_dim1(); i++){
        for(int j=1; j<= w.labels.get_spare_dim(i); j++){
            for(int k=1; k<= w.labels.get_dim1(); k++){
                for(int l=1;l<= w.labels.get_spare_dim(k); l++){
                    output << w.names(i) << " " << w.labels(i,j) << " " << w.names(k) << " " << w.labels(k,l) << " "<< w.ww(i,j,k,l) << endl;
                };
            };
        };
    };
    return output;
}

qmpb_w::~qmpb_w(){
    ww_cutof_el *to_delete, *iter;
    for(int i=1; i <= ww_cutof.get_dim1(); i++){
        for(int j=1; j<= ww_cutof.get_spare_dim(i); j++){
            if(ww_cutof(i,j) != NULL){
                iter = ww_cutof(i,j);
                while(iter != NULL){
                    to_delete = iter;
                    iter = (*iter).next;
                    delete to_delete;
                };
            ww_cutof(i,j) = NULL;
            };
        };
    };
    for(int i=1; i <= ww_cutof.get_dim1(); i++){
            if(ww_cutof_site(i) != NULL){
                iter = ww_cutof_site(i);
                while(iter != NULL){
                    to_delete = iter;
                    iter = (*iter).next;
                    delete to_delete;
                };
                ww_cutof_site(i) = NULL;
        };
    };
    ww.kill_arr();
    names.kill_arr();
    labels.kill_arr();
}
