//
// C++ Interface: qmpb_energies
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef QMPB_ENERGY_
#define QMPB_ENERGY_
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <math.h>
#include <cstring>
#include "file_io.h"
#include "vector.h"
#include "matrix.h"
#include "tensor.h"
#include "4d_tensor.h"
#include "gzstream.h"
#include "lists.h"

class ww_cutof_el{
    public:
    int site;
    vec<short> instances;
    ww_cutof_el *next;
    ww_cutof_el():site(0),next(NULL){};
};


class qmpb_gint{
    public:
        char filename[100];
        char buf[300];
        matrix<float> gint;
        matrix<float> gint_exp;
        vec<string> names;
        matrix<string> labels;
        qmpb_gint(char *file, vec<string> &resname, matrix<string> &labels, int nsites, vec<int> &nin);
        qmpb_gint(igzstream &bin_in);
        qmpb_gint(){};
        ~qmpb_gint();
        int write_binary(ogzstream &bin_out);
        int read_binary(igzstream &bin_in);
        int read_binary(igzstream &bin_in, vec<int> &sys_ninst, vec<string> &sys_names, matrix<string> &sys_labels);
        void exp_with(double fac);
        double get_size(){return gint.get_space();};
        int check_labels(qmpb_gint &gi1, qmpb_gint &gi2);
        friend ostream& operator<<(ostream &output, qmpb_gint &gi);
};


class qmpb_w{
    public:
        char filename[100];
        char buf[300];
        d4_tensor<float> ww;
        d4_tensor<float> ww_exp;
	d4_tensor<char> cutof_sites;
	tensor<char> cutof_sites_single_inst;
	matrix<char> cutof_sites_single;

        vec<string> names;
        matrix<ww_cutof_el*> ww_cutof;
        matrix< single_LL<ww_cutof_el> > w_cutof;
        vec<ww_cutof_el*> ww_cutof_site;
        matrix<string>labels;
        qmpb_w(char *file, vec<string> &resname, matrix<string> &labels, int nsites, vec<int> &nin);
        qmpb_w(igzstream &bin_in);
        qmpb_w(){};
        ~qmpb_w();
        int write_binary(ogzstream &bin_out);
        int read_binary(igzstream &bin_in);
        int read_binary(igzstream &bin_in, vec<int> &sys_ninst, vec<string> &sys_names, matrix<string> &sys_labels);
        void exp_with(double fac);
        void create_cutof_list(double cutof, int blab);
        void create_cutof_list_site();
        void print_cutof();
        double get_size(){return ww.get_space();};
        double calc_ww(int site, int conf, vec<short> &vect);
        friend ostream& operator<<(ostream &output, qmpb_w &w);
};
#endif
