//
// C++ Interface: sparse_matrix
//
// Description: 
//
//
// Author: Mirco Till <Mirco.Till@uni-bayreuth.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef SPARSE_MATRIX_
#define SPARSE_MATRIX_
#include <iostream>
#include <iomanip>
#include <typeinfo>
using namespace std;

template<class T>
        class LL{
            public:
                LL<T> *next;
                T value;
                unsigned short index;
                LL(){value = 0; next = NULL;}
        };

template<class T>
class sparse_matrix{
    private:
        int dim1, dim2;
        int start1, start2;
        int *spare;
        double space;
        T zero_val;
        LL<T> **li;
    public:
        T **arr;
        
        sparse_matrix(int d1,int d1_2,T ini, vec<int> &sizes):space(0){
            if(typeid(T) != typeid(string)){zero_val = (T) 0;}
            spare = new int[d1_2 - d1 + 2];
            init_arr(d1,d1_2, sizes);
            space += sizeof(int)*(d1_2 - d1 + 2);
            li = NULL;
        };
        sparse_matrix(): dim1(0), dim2(0), start1(0), start2(0), space(0){
            if(typeid(T) != typeid(string)){zero_val = (T) 0;}
            arr = NULL;
            spare = NULL;
            li = NULL;
        };
        void init_arr(int x, int x2,vec<int> &sizes){
            dim1 = x2 - x +1;
            dim2 = 0;
            start1 = x;
            start2 = 1;
            spare = new int[x2 - x + 2];
            space += sizeof(int)*(x2 - x + 2);
            for(int i=x; i<= x2; i++){
                spare[i] = sizes(i);
            };
            try{
                arr = new T*[dim1];
                arr = arr - start1;
                space += sizeof(T*)*dim1;
                for (int i=start1;i<=start1 + dim1 -1;++i){
                    arr[i] = new T[sizes(i)+1];
                    arr[i][0] = sizes(i);
                    space += sizeof(T)*(sizes(i)+1);
                };

              }catch(bad_alloc){
                  exit(0);
              };
        };
        int zero_out(){
        // converting from array to linked list! This may increase access times!!!!
             if(li != NULL){cerr << "There went something wrong in zero_out() for sparse matrices.\n";
                cerr << "Maybe you have already run this function?\n";
                exit(0);
             };
             int tmp_dim;
             LL<T> *last;
             li = new LL<T>*[dim1];
             space += dim1*sizeof(LL<T>*);
             li = li - start1;
             for(int i=start1; i<=start1 + dim1 - 1; i++){
                 li[i] = new LL<T>;
                 space += sizeof(LL<T>);
                 (*li[i]).value= arr[i][start2];
                 (*li[i]).index = start2;
                 last = li[i];
                 if(dim2 == 0){tmp_dim = get_spare_dim(i);}else{tmp_dim = dim2;};
                     for(int j=start2+1; j<=start2 + tmp_dim - 1; j++){
                          if(arr[i][j] < 0.001 && arr[i][j] > -0.001){
                          }else{
                              (*last).next = new LL<T>;
                              space += sizeof(LL<T>);
                              (*last).next->value = arr[i][j];
                              (*last).next->index = j;
                              last = (*last).next;
                              (*last).next = NULL;
                           }
                       }
                   }
                   kill_arr();
                   return 1;
         }
         double precalc_space_superspare(){
             int tmp_dim, max_dim2;
             LL<T> *last;
             double sup_space = 0;
             sup_space = 0;
             sup_space += sizeof(int)*(dim1 + 1); // index array
             sup_space += dim1*sizeof(LL<T>*);
             for(int i=start1; i<=start1 + dim1 - 1; i++){
                 sup_space += sizeof(LL<T>);
                 last = li[i];
                 if(dim2 == 0){tmp_dim = get_spare_dim(i);}else{tmp_dim = dim2;};
                 for(int j=start2+1; j<=start2 + tmp_dim - 1; j++){
                     if(arr[i][j] < 0.001 && arr[i][j] > -0.001){
                    
                     }else{
                         sup_space += sizeof(LL<T>);
                     }
            
                 }
             }
             return sup_space;
         }
         int get_dim1(){return dim1;}
         int get_dim2(){return dim2;};
         int get_spare_dim(int x){return spare[x];};
         double get_space(){return space;}
         void kill_arr(){
             if(arr != NULL){
                 for (int i=start1;i<=start1 + dim1 -1;++i){
                     if(dim2 > 0){
                         arr[i] = arr[i] + start2;
                         space -= dim2*sizeof(T);
                     }else{
                         space -= (spare[i]+1) * sizeof(T);
                     }
                     delete [] arr[i];
                };
                arr = arr + start1;
                delete [] arr;
                space -= dim1*sizeof(T*);
                arr = NULL;
              }
          };
          void kill_spare(){
              LL<T> *iter, *to_del;
              for(int i=start1;i<=start1 + dim1 - 1; i++){
                  iter = li[i];
                  while(iter != NULL){
                      to_del = iter;
                      iter = (*iter).next;
                      delete to_del;
                  }
               }
               li = li + start1;
               delete [] li;
          }
          ~sparse_matrix(){
              if(arr && dim1 > 0){
                  kill_arr();
              };
              if(li){
                  kill_spare();
              }
              delete [] spare;
          };
          T &operator()(int x, int y){
              if(arr != NULL){
                  return arr[x][y];
              }else{
                  return get_spare(x,y);
              }
          };
          T *operator()(int x){
              return arr[x];
          };
          T& get_spare(int x, int y){
              LL<T> *iter;
              iter = li[x];
              while(iter != NULL && (*iter).index != y){
                  if((*iter).index > y){return zero_val;};
                      iter = (*iter).next;
                  }
                  if(iter == NULL){return zero_val;}else{
                  return (*iter).value;
               }
           }
           friend ostream& operator<<(ostream &output, sparse_matrix &matrix){
               if(matrix.get_dim2() > 0){
                   for(int i=matrix.start1;i<=matrix.start1+matrix.dim1-1;i++){
                       for(int j=matrix.start2;j<=matrix.start2+matrix.dim2-1;j++){
                           cout << setw(10) << fixed << setprecision(2) << matrix(i,j) << " ";
                       }
                       cout << endl;
                   }
               }else if(matrix.dim2 == 0){
                   for(int i=matrix.start1;i<=matrix.start1 + matrix.dim1 - 1;i++){
                       for(int j=1;j<=matrix.get_spare_dim(i);j++){
                           cout << setw(10) << fixed << setprecision(2) << matrix(i,j) << " ";
                       }
                       cout << endl;
                    }
               }
               return output;
           }
};
#endif
