#ifndef VEC_
#define VEC_
#include <iostream>
#include <iomanip>
#include <cstdlib>

using namespace std;
/** Vetor template

  Template class for a one dimensional vector. Cares about all the memory allocation things and initialisatio if asked. <br>
  Any given range is possible, no need to start at 0!<br>
  Examples: Interger vector:<br>
  vec<int> example; followed by example.init_arr(1,100) gives you an interger vector from 1 to 100 with 100 elemets. <br>
  vec<int> eample(1,100); Same vector as above;<br>
  vec<int> example; followed by example.init_arr(1,100,0); Same vector as above, but all elements are initialised with 0;<br>
  example(1) = 1; Assignes 1 to the first element.<br>

*/
template<class T>
class vec {
  private:
    int  dim1; /**< Dimentsion of the vector: rhs - lhs + 1 */
    short  start1; /**< Starting point of the vector */
    float space; /**< memory usage */
  public:
    T *arr; /**< The vector itself */
    /** Constructor without allocation and initialisation

        initial has to be called for allocating memory
    */
    vec(): dim1(0), start1(0), space(sizeof(vec)), arr(NULL) {
    };
    /** Constructor with allocation but without initialisation

        Memory is allocated by init_arr(int, int)
        \param st Starting index
        \param stop Last index
    */
    vec(int st, int stop):dim1(stop-st+1),start1(st),space(sizeof(vec)), arr(NULL) {
      init_arr(st,stop);
    };
    /** Constructor with allocation and initialisation

        Memory is allocated by init_arr(int, int, T) and all elements are set to ini
        \param st Starting index
        \param stop Last index
        \param ini Initialising value for all elemetns
    */
    vec(int st, int stop, int ini):dim1(stop-st+1),start1(st),space(sizeof(vec)), arr(NULL) {
      init_arr(st,stop,ini);
    };
    /** Copyconstructor

        \param &ini_vec Constant reference of the vector used for initialisation
    */
    vec(const vec<T> &ini_vec) {
      init_arr(ini_vec);
    };
    /** Initialise vector from another vector

        Takes the start value and the dimension from another vector of the same type and calls init_arr(int, int)
        for the new vector. Afterwards copies the values of the passed vector to the new one.
        \param &ini_vec Constant reference of the vector used for initialisation
    */
    void init_arr(const vec<T> &ini_vec) {
      init_arr(ini_vec.get_start(),ini_vec.get_dim() + ini_vec.get_start()-1);
      for (int i=ini_vec.get_start(); i<=ini_vec.get_dim() + ini_vec.get_start() - 1;i++) {
        arr[i] = ini_vec(i);
      }
    };
    /** Initialises the supproting values and calls memory allocator

        \param x Starting index
        \param x2 Last index
        */
    void init_arr(int x, int x2) {
      kill_arr(); // prevent memory leaks
      dim1 = x2 - x +1;
      start1 = x;
      arr = alloc_arr(arr,dim1,start1);
      space += dim1*sizeof(T);
    };
    /** Initialises the supproting values and calls memory allocator

        After allocation, all elements are set to ini
        \param x Starting index
        \param x2 Last index
        \param ini Initialising value for all elemetns
    */
    void init_arr(int x, int x2, T ini) {
      init_arr(x,x2);
      for (int i=x; i<=x2;i++) {
        arr[i] = ini;
      }
    };
    /** frees memory
       The memory allocated for the array is freed and pointers and space is set back.
    */
    void kill_arr() {
      if (arr != NULL) {
        arr = arr + start1;
        delete [] arr;
      };
      dim1 = start1 = 0;
      space = sizeof(vec);
      arr = NULL;
    };
    void kill_arr_only() {
      if (arr != NULL) {
        arr = arr + start1;
        delete [] arr;
      };
      arr = NULL;
    };
    /** Adds one element at the end of the array */
    void add_item(){add_item(1);};
    /** Adds n elements at the end of the array.

        Allocates memeory for an array with the new size and copies the values of the original array to the new one.
        \param n NUmber of elemetns to add.
    */
    void add_item(int n) {
      if (arr == NULL) {
        init_arr(1,n);
        return;
      };
      T *tmp_arr = NULL;
      tmp_arr = alloc_arr(tmp_arr,dim1+n,start1);
      for (int i = start1; i<start1 + dim1; i++) {
        tmp_arr[i] = arr[i];
      };
      kill_arr_only();
      arr = tmp_arr;
      tmp_arr = NULL;
      dim1 += n;
      space = sizeof(vec) + dim1*sizeof(T);
    };
    /** Memory allocator

        Tries to allocate memory in any asked size for the array
        \param *arr Pointer to the core array
        \param dim1 Number of elements
        \param start1 First index
    */

    T* alloc_arr(T *arr, int dim1, int start1) {
      try {
        arr = new T[dim1];
      } catch (bad_alloc) {
        cerr << "Memory allocation problem in vec. dim1:" << dim1 << endl;
        exit(0);
      };
      arr = arr - start1;
      return arr;
    };
    /** Dimension of the array
    */
    int get_dim() const {return dim1;};
    /** Index of the first element */
    int get_start() const{return start1;}
    /** Memeory usage of the array */
    double get_space(){return space;};

    T max(T start = -1E100) {
      T m = start;
      for (int i=1; i<=dim1 + start1 -1; i++) {
        if (m < arr[i]){m = arr[i];}
      };
      return m;
    }
    T min(T start = 1E100) {
      T m = start;
      for (int i=1; i<=dim1 + start1 -1; i++) {
        if (m > arr[i]){m = arr[i];}
      };
      return m;
    }

    double get_sum() {
      T sum = 0;
      for (int i=start1; i<= dim1 + start1 -1; i++) {
        sum += arr[i];
      }
      return sum;
    }
    double is_inside(T &search) {
      for (int i=start1; i<= dim1 + start1 -1; i++) {
        if (arr[i] == search) {
          return 1;
        }
      }
      return 0;
    }
    int search_inside(T &search) {
      for (int i=start1; i<= dim1 + start1 -1; i++) {
        if (arr[i] == search) {
          return i;
        }
      }
      return 0;
    }
    /** Destructor
        Frees the memory by calling kill_arr()
    */
    ~vec() {
      if (arr) {
        kill_arr();
      };
    };
    /** Index operator

        Overloaded operator for easy way of indexing
        \param x Index
    */
    T &operator()(int x) const {return arr[x];};
    /** Stream output operator */
    friend ostream& operator<<(ostream &output, vec &vector) {
      for (int i=vector.start1;i<=vector.start1+vector.dim1-1;i++) {
        output << setprecision(2) << fixed << vector(i) << " ";
      }
      output << endl;
      return output;
    }
    /** Write the whole vector in binary format */
    void write_binary(ostream &out) {
      out.write(reinterpret_cast<char *>(&arr[start1]), sizeof(T)*dim1);
    }
    /** Write the whole vector in binary format rest n items */
    void write_binary(ostream &out, int ni) {
      if (ni <= dim1) {
        out.write(reinterpret_cast<char *>(&arr[start1]), sizeof(T)*ni);
      } else {
        cerr << "WARNING: ni > dim1 in write_binary!" << endl;
      }
    }
    /** Read a binary vector directly into arr */
    void read_binary(istream &in) {
      in.read(reinterpret_cast<char *>(&arr[start1]), sizeof(T)*dim1);
    };
    /** Assignment operator

        Copies all values from the first start1 + dim1 -1 values from the passed array
        \todo Make this save for the case that the passed array is smaller than the current
        \todo Redimension the array if to the same size as the passed array
        \param &ref Reference array
    */
    vec<T>& operator=(vec<T> &ref) {
      for (int i=start1; i<=dim1 + start1 -1; i++) {
        arr[i] = ref.arr[i];
      };
      return *this;
    };
};

#endif
