//
// C++ Implementation: sys
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "sys.h"
Sys::Sys(char *moln): molname(moln), nmc(10), probe(1.3), surf_points(2500), memb_z_min(0), memb_z_max(0), cav_grid(0.5), start_grid(1.0), min_vol(10.0), water_vol(18.0), dummy_rad(1.7), membrane_dim(3), core_z_min(-1), core_z_max(-1), core_dim(-1), core_flag(0), cleft_d(1), nsour_cleft(1), cleft_method(0), delete_overlapping(1), blab(0), surface_clusters(0), break_after_surface(0) {

}

void Sys::read_setup() {
  char filename[100];
  sprintf(filename, "%s%s", molname.c_str(), ".setup");
  stringstream line;
  ifstream fi(filename);
  string flag;
  if (!fi){fprintf(stderr, "Can not open %s\n", filename);exit(0);};
  while (my_get_line(line, fi)) {
    line >> flag;
    if (flag != "!" && flag != "") {
      if (flag == "nmc"){line >> nmc;} /**< Number of Monte Carlo Steps pre A^3<*/
      else if (flag == "probe"){line >> probe;}
      else if (flag == "surfPT"){line >> surf_points;}
      else if (flag ==  "membZmin"){line >> memb_z_min;}
      else if (flag ==  "membZmax"){line >> memb_z_max;}
      else if (flag ==  "cavgridspacing"){line >> cav_grid;}
      else if (flag ==  "startgridspacing"){line >> start_grid;}
      else if (flag ==  "blab"){line >> blab;}
      else if (flag ==  "DummyRad"){line >> dummy_rad;}
      else if (flag ==  "MembDim"){line >> membrane_dim;}
      else if (flag ==  "CoreZMin"){line >> core_z_min;core_flag = 1;}
      else if (flag ==  "CoreZMax"){line >> core_z_max;core_flag = 1;}
      else if (flag ==  "CoreDim"){line >> core_dim;core_flag = 1;}
      else if (flag ==  "CleftDim"){line >> cleft_d;}
      else if (flag ==  "CleftRel"){line >> nsour_cleft;}
      else if (flag ==  "CleftMethod"){line >> cleft_method;}
      else if (flag ==  "DeleteOverlapping"){line >> delete_overlapping;}
      else if (flag ==  "SurfaceCluster"){line >> surface_clusters;}
      else if (flag ==  "BreakAfterSurface"){line >> break_after_surface;}
      else if (flag ==  "minVol") {
        line >> min_vol;
        if (min_vol <= 1.1) {
          fprintf(stderr, "Do you really want cavities that small? minVol < 1\n");
        };
      } else if (flag == "waterVol"){line >> water_vol;}
      else {
        cerr << "unknown word in setup file: %s" << flag << endl;
      }
    }
  }
  if (core_z_max - core_z_min > 0.1){core_flag = 1;}
  else{core_flag = 0;}
}

void Sys::print_param(ostream &out) {
  out << "###### System Parameters: ######" << endl;
  out << "Molecule Name: " << molname << endl;
  out << "Number of Monte Carlo steps: " << nmc << endl;
  out << "Probe Sphere radius: " << probe << endl;
  out << "###### Surface Parameters: ######" << endl;
  out << "Number of surface points per atom: " << surf_points << endl;
  out << "Surface Clustering Distance: " << surface_clusters << endl;
  out << "###### Cavity Parameters: ######" << endl;
  out << "Cavity Grid Spacing: " << cav_grid << endl;
  out << "Minimum Cavity Volume: " << min_vol << endl;
  out << "Water Volume: " << water_vol << endl;
  out << "Cleft Method: " << cleft_method << endl;
  out << "Cleft Paramter 1: " << cleft_d << endl;
  out << "Cleft Paramter 2: " << nsour_cleft << endl;
  out << "Delete Overlapping Cavities: " << delete_overlapping << endl;
  out << "###### Membrane Parameters: ######" << endl;
  out << "Minimum Z Value: " << memb_z_min << endl;
  out << "Maximum Z Value: " << memb_z_min << endl;
  out << "Thickness of slap: " << membrane_dim << endl;
  out << "Check Core Region: " << core_flag << endl;
  out << "Core Minimum Z Value: " << core_z_min << endl;
  out << "Core Maximum Z Value: " << core_z_max << endl;
  out << "Headgroup Dimension: " << core_dim << endl;
  out << endl;
}

Sys::~Sys() {
}


