//
// C++ Interface: sys
//
// Description:
//
//
// Author: Mirco <mirco@Mirco-Laptop>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef SYS_H
#define SYS_H
#include <iostream>
#include <sstream>
#include <string>
#include "util.h"
/**Holding user given parameters

  Holding all the parameters adjustable in the setup file.
*/
class Sys {
  public:
    string molname; /**< The molecule name */
    int nmc; /**< Number of Monte Carlo steps per A^3 */
    double probe; /**< Probe sphere radius */
    int surf_points; /**< Number of points placed on the iso surface */
    double memb_z_min;/**< Minimum membrane dimension in z direction */
    double memb_z_max;/**< Maximum membrane dimension in z direction */
    double cav_grid;/**< Grid spacing for the finer grid used to calculate the cavities in the second run */
    double start_grid;/**< Grid spacing for the initial Monte Carlo run as well as for the membrane */
    double min_vol; /**< Cavities smaller than this are treated as artefacts */
    double water_vol; /**< Volume of one water molecule */
    double dummy_rad; /**< All dummy atoms written (cavities, clefts, membrane) will have this radius */
    double membrane_dim; /**< Thickness of the membrane slap */
    double core_z_min; /**< Minimum z value of the membrane core region */
    double core_z_max; /**< Maximum z value of the membrane core region */
    double core_dim; /**< Distance of the points in the head group region from the core region */
    char core_flag; /**< Applying core region check? 0 or 1 */
    int cleft_d; /**< Parameter 1 for cleft calculation */
    int nsour_cleft; /**< Parameter 2 for cleft calculation */
    int cleft_method; /**< Choose method for cleft calculation */
    int delete_overlapping; /**< Delete/Merge cavities overlapping with each ohter */
    int blab; /**< Blab level */
    float surface_clusters; /**< Distance criteria for clustering the  dot surface */
    int break_after_surface; /**< Stop calculation after calculating the SAS */
    Sys(char *moln);
    void read_setup();
    void print_param(ostream &out);
    ~Sys();
};

#endif
