/* Matthias Ullmann                                     */
/* Bayreuth, September 2006                             */
/*                                                      */
/* .....................................................*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mathematik.h"
#include "mus_atom.h"
#include "util_c.h"
#include "nrutil.h"
#include "energy.h"
#include "grid.h"
#include "nb-list.h"
#include "surface.h"
#include "volume.h"

/**********************************************************/
/*........................................................*/
/* make a neighbor list for a protein                     */
/*                                                        */
void ProteinToGrid(iListGrid *grid, MUS_atom *p, double maxrad)
{
  double min[4], max[4];
  iList *list;

  min_max_coord(min, max, (*p).xyz, (*p).number_of_atoms);
  max[1] = max[1] + maxrad;
  max[2] = max[2] + maxrad;
  max[3] = max[3] + maxrad;
  min[1] = min[1] - maxrad;
  min[2] = min[2] - maxrad;
  min[3] = min[3] - maxrad;

  /* Assigning the values for the grid */
  (*grid).inv_spacing=1.0/(*grid).spacing;
  (*grid).imax=ivector(1,3);
  (*grid).imin=ivector(1,3);
  (*grid).imax[1]=(int) ceil(max[1]*(*grid).inv_spacing);
  (*grid).imax[2]=(int) ceil(max[2]*(*grid).inv_spacing);
  (*grid).imax[3]=(int) ceil(max[3]*(*grid).inv_spacing);

  (*grid).imin[1]=(int) floor(min[1]*(*grid).inv_spacing);
  (*grid).imin[2]=(int) floor(min[2]*(*grid).inv_spacing);
  (*grid).imin[3]=(int) floor(min[3]*(*grid).inv_spacing);

  /* make the grid spacing to odd integer, if they are not odd integer */
  odding_down((*grid).imin,3);
  odding_up((*grid).imax,3);

  list=iList_vector(1,(*p).number_of_atoms);
  init_iList_vector(list,(*p).number_of_atoms);
  (*grid).root=iList_3tensor((*grid).imin[1], (*grid).imax[1],
                          (*grid).imin[2], (*grid).imax[2],
                          (*grid).imin[3], (*grid).imax[3]);

  init_iListGrid(grid);
  fill_iListGrid(grid, list, (*p).xyz, (*p).number_of_atoms);
}


/*........................................................*/
/* make a a neighbor list for a dot surface               */
/*                                                        */
void SurfaceToGrid(iListGrid *grid, DotSurf *sf)
{
  double min[4], max[4];
  iList *list;

  min_max_coord(min, max, (*sf).dots, (*sf).ndots);


  /* Assigning the values for the grid */
  (*grid).inv_spacing=1.0/(*grid).spacing;
  (*grid).imax=ivector(1,3);
  (*grid).imin=ivector(1,3);
  (*grid).imax[1]=(int) ceil(max[1]*(*grid).inv_spacing);
  (*grid).imax[2]=(int) ceil(max[2]*(*grid).inv_spacing);
  (*grid).imax[3]=(int) ceil(max[3]*(*grid).inv_spacing);

  (*grid).imin[1]=(int) floor(min[1]*(*grid).inv_spacing);
  (*grid).imin[2]=(int) floor(min[2]*(*grid).inv_spacing);
  (*grid).imin[3]=(int) floor(min[3]*(*grid).inv_spacing);

  /* make the grid spacing to odd integer, if they are not odd integer */
  odding_down((*grid).imin,3);
  odding_up((*grid).imax,3);

  list=iList_vector(1,(*sf).ndots);
  init_iList_vector(list,(*sf).ndots);
  (*grid).root=iList_3tensor((*grid).imin[1], (*grid).imax[1],
                          (*grid).imin[2], (*grid).imax[2],
                          (*grid).imin[3], (*grid).imax[3]);

  init_iListGrid(grid);
  fill_iListGrid(grid, list, (*sf).dots, (*sf).ndots);
}

/*........................................................*/
/* This function assigns an pt to a grid cell             */
/*                                                        */
/* returns 0 if the point is within the grid otherwise 1  */
/*                                                        */
int pt_to_cell2(int *ipt, double *pt, iGrid *grd)
{

    /* map the pt on the grid */
    ipt[1]=(int)rint(pt[1]*(*grd).inv_spacing);
    ipt[2]=(int)rint(pt[2]*(*grd).inv_spacing);
    ipt[3]=(int)rint(pt[3]*(*grd).inv_spacing);

    /*fprintf(stderr,"%d  %d %d\n", ipt[1], ipt[2], ipt[3]);TEST */

    if (  (ipt[1]<(*grd).imin[1]) || (ipt[1]>(*grd).imax[1])
        ||(ipt[2]<(*grd).imin[2]) || (ipt[2]>(*grd).imax[2])
        ||(ipt[3]<(*grd).imin[3]) || (ipt[3]>(*grd).imax[3]))
        {return 1;
        }
    else {return 0;}
}

/*........................................................*/
/* This function assigns an pt to a grid cell             */
/*                                                        */
/* returns 0 if the point is within the grid otherwise 1  */
/*                                                        */
int pt_to_cell(int *ipt, double *pt, iListGrid *grd)
{

    /* map the pt on the grid */
    ipt[1]=(int)rint(pt[1]*(*grd).inv_spacing);
    ipt[2]=(int)rint(pt[2]*(*grd).inv_spacing);
    ipt[3]=(int)rint(pt[3]*(*grd).inv_spacing);

    /*fprintf(stderr,"%d  %d %d\n", ipt[1], ipt[2], ipt[3]);TEST*/

    if (  (ipt[1]<(*grd).imin[1]) || (ipt[1]>(*grd).imax[1])
        ||(ipt[2]<(*grd).imin[2]) || (ipt[2]>(*grd).imax[2])
        ||(ipt[3]<(*grd).imin[3]) || (ipt[3]>(*grd).imax[3]))
        {return 1;
        }
    else {return 0;}
}

